#! /bin/sh
# $Id$

if [ $# -ne 1 ] ; then
	echo "apple-release.sh ARCH"
	exit 1
fi

ARCH=$1
SRC=../src
BIN=../bin.$ARCH
VERSION_FILE=$SRC/version.h

TITLE=baretorrent
NAME=baretorrent
SOURCE=$BIN/baretorrent.app
TEMP_ARCHIVE=pack.temp.dmg

echo checking $VERSION_FILE

LINE=`grep "#define BARETORRENT_VERSION_MAJOR" $VERSION_FILE`
VERSION_MAJOR=`echo "$LINE" | cut -d " " -s -f 3`
VERSION_MAJOR=`echo "$VERSION_MAJOR" | sed 's/[[:space:]]*$//g'`

LINE=`grep "#define BARETORRENT_VERSION_MINOR" $VERSION_FILE`
VERSION_MINOR=`echo "$LINE" | cut -d " " -s -f 3`
VERSION_MINOR=`echo "$VERSION_MINOR" | sed 's/[[:space:]]*$//g'`

LINE=`grep "#define BARETORRENT_VERSION_MAINTENANCE" $VERSION_FILE`
VERSION_MAINTENANCE=`echo "$LINE" | cut -d " " -s -f 3`
VERSION_MAINTENANCE=`echo "$VERSION_MAINTENANCE" | sed 's/[[:space:]]*$//g'`

if [ -z $VERSION_MAJOR ] ; then
	echo "bad version"
	exit 1
fi

if [ -z $VERSION_MINOR ]  ; then
	echo bad version
	exit 1
fi

VERSION_STRING="${VERSION_MAJOR}.${VERSION_MINOR}"

if [ ! -z $VERSION_MAINTENANCE ] ; then
	VERSION_STRING="${VERSION_STRING}.${VERSION_MAINTENANCE}"
fi

echo $VERSION_STRING

ARCHIVE=baretorrent-${VERSION_STRING}-osx-$ARCH


echo creating image $TEMP_ARCHIVE from $SOURCE
rm -f $TEMP_ARCHIVE
hdiutil create -quiet -srcfolder "${SOURCE}" -volname "${TITLE}" -fs HFS+ -fsargs "-c c=64,a=16,e=16" -format UDRW -size 100000k ${TEMP_ARCHIVE}

echo mounting dmg
DEVICE=`hdiutil attach -readwrite -noverify -noautoopen "${TEMP_ARCHIVE}" | egrep '^/dev/' | sed 1q | awk '{print $1}'`
sleep 2

echo device is $DEVICE

echo setting up the view with title=$TITLE and name=$NAME
echo '
	tell application "Finder"
		tell disk "'${TITLE}'"
			open
			set current view of container window to icon view
			set toolbar visible of container window to false
			set statusbar visible of container window to false
			set the bounds of container window to {400, 100, 885, 430}
			set theViewOptions to the icon view options of container window
			set arrangement of theViewOptions to not arranged
			set icon size of theViewOptions to 72
			make new alias file at container window to POSIX file "/Applications" with properties {name:"Applications"}
			set position of item "'${NAME}'" of container window to {100, 100}
			set position of item "Applications" of container window to {375, 100}
			close
			open
			update without registering applications
			delay 5
			eject
		end tell
	end tell
' | osascript

VOLUME="/Volumes/${TITLE}"

echo unmounting dmg $VOLUME
chmod -Rf go-w $VOLUME
sync
sync

echo converting and compressing dmg from $TEMP_ARCHIVE to $ARCHIVE
rm -f "${ARCHIVE}"
hdiutil convert -quiet "${TEMP_ARCHIVE}" -format UDZO -imagekey zlib-level=9 -o "${ARCHIVE}"
rm -f "${TEMP_ARCHIVE}"



