@echo off

setlocal EnableDelayedExpansion

set NAME=%1
set ARCH=%2

if "%NAME%"=="" goto bad_args
if "%ARCH%"=="" goto bad_args

if "%NAME%"=="windows" set PLATFORM=v110
if "%NAME%"=="xp" set PLATFORM=v110_xp
set SRC=..\src
set BIN=..\bin.%ARCH%.%PLATFORM%
set VERSION_FILE=%SRC%\version.h
set SEVENZ=C:\Program Files\7-Zip\7z.exe

call:get_version
call:make_archive
call:make_installer

exit /b 0


:make_installer
makensis ^
	/DARCH=%ARCH% /DPLATFORM=%PLATFORM% /DPNAME=%NAME% ^
	/DVERSION_STRING=%VERSION_STRING% /DVERSION_MAJOR=%VERSION_MAJOR% ^
	/DVERSION_MINOR=%VERSION_MINOR% /DVERSION_RES=%VERSION_RES% /V2 ^
	baretorrent.nsi

if %errorlevel% neq 0 goto error

goto:eof


:get_version
@echo checking %VERSION_FILE%

set MAJOR_RE=BARETORRENT_VERSION_MAJOR [0-9][0-9]*
set MINOR_RE=BARETORRENT_VERSION_MINOR [0-9][0-9]*
set MAINTENANCE_RE=BARETORRENT_VERSION_MAINTENANCE [0-9][0-9]*

set FIND_MAJOR='findstr /r /c:"%MAJOR_RE%" %VERSION_FILE%'
set FIND_MINOR='findstr /r /c:"%MINOR_RE%" %VERSION_FILE%'
set FIND_MAINTENANCE='findstr /r /c:"%MAINTENANCE_RE%" %VERSION_FILE%'

set LINE=
set VERSION_MAJOR=
for /f "tokens=*" %%i in (%FIND_MAJOR%) do (
	set LINE=%%i
	if %errorlevel% neq 0 goto error
)

for /f "tokens=3" %%i in ("%LINE%") do (
	set VERSION_MAJOR=%%i
	if %errorlevel% neq 0 goto error
)

set LINE=
set VERSION_MINOR=
for /f "tokens=*" %%j in (%FIND_MINOR%) do (
	set LINE=%%j
	if %errorlevel% neq 0 goto error
)

for /f "tokens=3" %%j in ("%LINE%") do (
	set VERSION_MINOR=%%j
	if %errorlevel% neq 0 goto error
)

set LINE=
set VERSION_MAINTENANCE=
for /f "tokens=*" %%k in (%FIND_MAINTENANCE%) do (
	set LINE=%%k
	if %errorlevel% neq 0 goto error
)

for /f "tokens=3" %%k in ("%LINE%") do (
	set VERSION_MAINTENANCE=%%k
	if %errorlevel% neq 0 goto error
)

if "%VERSION_MAJOR%"=="" goto bad_version
if "%VERSION_MINOR%"=="" goto bad_version

set VERSION_STRING=%VERSION_MAJOR%.%VERSION_MINOR%
set VERSION_RES=%VERSION_MAJOR%.%VERSION_MINOR%

if not "%VERSION_MAINTENANCE%"=="" (
	set VERSION_STRING=%VERSION_STRING%.%VERSION_MAINTENANCE%
	set VERSION_RES=%VERSION_RES%.%VERSION_MAINTENANCE%.0
) else (
	set VERSION_RES=%VERSION_RES%.0.0
)

@echo !VERSION_STRING!

goto:eof


:make_archive

set TARGET=baretorrent-%VERSION_STRING%-%NAME%-%ARCH%
set ARCHIVE=%TARGET%.7z
set MSVC=C:\Program Files (x86)\Microsoft Visual Studio 11.0\VC
set REDIST=%MSVC%\redist\%ARCH%

@echo copying to %TARGET%

if not exist "%TARGET%" mkdir "%TARGET%"

xcopy /i locales "%TARGET%\locales" > NUL
if %errorlevel% neq 0 goto error

copy %BIN%\baretorrent.exe "%TARGET%\baretorrent.exe" > NUL
if %errorlevel% neq 0 goto error

copy %BIN%\baretorrent.dll "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy %BIN%\boost_python-vc110-mt-1_52.dll "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy "%REDIST%\Microsoft.VC110.CRT\msvcp110.dll" "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy "%REDIST%\Microsoft.VC110.CRT\msvcr110.dll" "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy %BIN%\python33.dll "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy %BIN%\wxmsw294u_vc_custom.dll "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy %BIN%\*.pyd "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy %BIN%\python33.zip "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

copy ..\CHANGELOG "%TARGET%\%" > NUL
if %errorlevel% neq 0 goto error

copy ..\COPYING "%TARGET%\%" > NUL
if %errorlevel% neq 0 goto error

@echo archiving to %ARCHIVE%

"%SEVENZ%" a "%ARCHIVE%" "%TARGET%\" > NUL
if %errorlevel% neq 0 goto error

@rmdir /s /q "%TARGET%"

goto:eof


:error
exit /b 1

:bad_version
@echo bad version
exit /b 1


:bad_args
@echo usage: windows-release.bat NAME ARCH
@echo.
@echo options
@echo NAME		'windows' or 'xp'
@echo ARCH		'x86' or 'x64'
exit /b 1
