import baretorrent
import libtorrent
import http.server
import threading
import traceback
import json
import urllib
import cgi
import os
import tempfile

httpd = None
http_thread = None

class bad_request(Exception):
	pass

def make_json(v):
	s = json.dumps(v, sort_keys=True, indent=4)
	return ["json", s]

def handle_baretorrent(r, u, q):
	s = []
	
	if u.path == "/list":
		m = baretorrent.manager()
		
		s = []
		for i in range(0, m.size()):
			t = m.get(i)
			s.append({"name": t.name()})
		
		return make_json({"torrents": s})
		
	elif u.path == "/add":
		if "url" not in q:
			baretorrent.log_error("missing url field in add request")
			raise bad_request()
		
		file = q["url"][0]
		baretorrent.log("adding " + file)
		
		m = baretorrent.manager()
		m.add_torrent(file)
		
		return ["text", "1"]
		
	raise bad_request()
	
def handle_utorrent(r, u, q):
	if u.path == "/gui/token.html":
		baretorrent.log("utorrent: asking for token")
		s = ("<html><div id='token' style='display:none;'>"
			"0000000000000000000000000000000000-"
			"000000000000000000000000"
			"</div></html>")
			
		return ["html", s]

	baretorrent.log("utorrent: action")
	if "action" not in q:
		baretorrent.log_error("utorrent: missing action field")
		raise bad_request()

	a = q["action"][0]
	baretorrent.log("utorrent: action is " + a)

	if a == "add-url":
		if "s" not in q:
			baretorrent.log_error("utorrent: missing s field in add request")
			raise bad_request()
	
		s = q["s"][0]
		baretorrent.log("utorrent: url is " + s)
	
		m = baretorrent.manager()
		baretorrent.log("utorrent: adding " + s)
		m.add_torrent(s)
			
		return ["text", ""]
		
	elif a == "add-file":
		form = cgi.FieldStorage(
			fp=r.rfile,
			headers=r.headers,
			environ={
				'REQUEST_METHOD':'POST',
				'CONTENT_TYPE': r.headers['Content-Type']})
		
		if "torrent_file" not in form:
			baretorrent.log_error("utorrent: missing torrent_file field")
			raise bad_request()
		
		item = form["torrent_file"]
		baretorrent.log("utorrent: " + item.filename)
		
		f = tempfile.NamedTemporaryFile(delete=False)
		fn = f.name
		f.write(item.value)
		f.close()
			
		m = baretorrent.manager()
		baretorrent.log("utorrent: adding " + fn)
		m.add_torrent(fn)
			
		os.remove(fn)
		return ["text", "1"]
				
	raise bad_request()
					
def send_ok_response(r, s):
	r.send_response(200)
	if s[0] == "json":
		r.send_header("Content-type", "application/json")
	elif s[0] == "html":
		r.send_header("Content-type", "text/html")
	else:
		r.send_header("Content-type", "text/plain")

	r.end_headers()
	r.wfile.write(bytes(s[1], "UTF-8"))

def send_bad_response(r):
	r.send_response(400)
	r.send_header("Content-type", "text/plain")
	r.end_headers()
	r.wfile.write(b"400 Bad Request")
	
class http_handler(http.server.BaseHTTPRequestHandler):
	def do_GET(self):
		try:
			baretorrent.log(self.path)

			u = urllib.parse.urlparse(self.path)
			q = urllib.parse.parse_qs(u.query)

			baretorrent.log(u.path)
			
			s = None
			if u.path.startswith("/gui/"):
				s = handle_utorrent(self, u, q)
			else:
				s = handle_baretorrent(self, u, q)
			
			send_ok_response(self, s)
			return
			
		except bad_request:
			pass
		except:
			baretorrent.log_error(traceback.format_exc())
		
		send_bad_response(self)
			
	def do_POST(self):
		try:
			baretorrent.log(self.path)

			u = urllib.parse.urlparse(self.path)
			q = urllib.parse.parse_qs(u.query)

			baretorrent.log(u.path)
			
			s = None
			if u.path.startswith("/gui/"):
				s = handle_utorrent(self, u, q)
			else:
				s = handle_baretorrent(self, u, q)
			
			send_ok_response(self, s)
			return
			
		except bad_request:
			pass
		except:
			baretorrent.log_error(traceback.format_exc())
			
		send_bad_response(self)
		

def thread_fun():
	global httpd

	port = int(baretorrent.get_opt("port"))
	baretorrent.log("starting httpserver on " + str(port))
	httpd = http.server.HTTPServer(('', port), http_handler)
	baretorrent.log("serving")
	httpd.serve_forever()
	baretorrent.log("done serving")

def start():
	baretorrent.log("starting thread")
	http_thread = threading.Thread(target=thread_fun)
	http_thread.start()

def stop():
	global httpd

	if httpd != None:
		httpd.shutdown()
		httpd.socket.close()
		httpd = None

def on_option_changed(name, value):
	if name == "port":
		start()
	
def on_icon_clicked():
	global httpd

	if httpd == None:
		baretorrent.add_status("icon_16.png", False)
		start()
	else:
		baretorrent.add_status("icon_16.png", True)
		stop()
	
def init():
	baretorrent.register("option_changed", "on_option_changed")
	baretorrent.register("icon_clicked", "on_icon_clicked")
	
	baretorrent.add_opt("port", "2999")
	baretorrent.add_status("icon_16.png", False)
	start()

def cleanup():
	global httpd
	baretorrent.remove_status()
	baretorrent.log("cleaning up")
	stop()
