<?php
define("bt_include_is_in_script", 1);
define("NO_DATABASE", 1);

require_once(__DIR__ . "/../scripts/forum/layout.php");

class bad_database extends \Exception {}

abstract class installer
{
	abstract public function output();
	abstract public function verify();
	abstract public function install();
}

class config_installer extends installer
{
	private $host = "";
	private $name = "";
	private $username = "";
	private $password = "";
	private $error = "";
	
	public function __construct()
	{
		if (isset($_POST["submit"]))
		{
			$this->host = trim(safe_post("db_host"));
			$this->name = trim(safe_post("db_name"));
			$this->username = trim(safe_post("db_username"));
			$this->password = trim(safe_post("db_password"));
		}
	}

	public function output()
	{
		$s = "";
		
		if ($this->error !== "")
			$s .= html::wrap_div("bt_forum_error", $this->error) . "<br />";
		
		echo $s;
	}

	public function verify()
	{
		if (!is_writable("scripts/"))
		{
			$this->error = loc("install_config_not_writable");
			return false;
		}
		
		return true;
	}
	
	public function install()
	{
		$config = 
			"<?php\n" .
			"define(\"DATABASE_HOST\", \"$this->host\");\n" .
			"define(\"DATABASE_NAME\", \"$this->name\");\n" .
			"define(\"DATABASE_USERNAME\", \"$this->username\");\n" .
			"define(\"DATABASE_PASSWORD\", \"$this->password\");\n" .
			"?>";
	
		if (file_put_contents("scripts/config.php", $config) === FALSE)
		{
			$this->error = loc("install_config_failed");
			return false;
		}
		
		include("scripts/config.php");
		
		return true;
	}
}

class database_installer extends installer
{
	private $host = "";
	private $name = "";
	private $username = "";
	private $password = "";
	private $error = "";
	
	public function __construct()
	{
		if (isset($_POST["submit"]))
		{
			$this->host = trim(safe_post("db_host"));
			$this->name = trim(safe_post("db_name"));
			$this->username = trim(safe_post("db_username"));
			$this->password = trim(safe_post("db_password"));
		}
	}
	
	public function output()
	{
		$s = "";
		
		if ($this->error !== "")
			$s .= html::wrap_div("bt_forum_error", $this->error) . "<br />";
			
		$s .= "
			" . loc("install_db_host") . "<br />
			" . html::text("db_host", $this->host) . "<br /><br />
			" . loc("install_db_name") . "<br />
			" . html::text("db_name", $this->name) . "<br /><br />
			" . loc("install_db_username") . "<br />
			" . html::text("db_username", $this->username) . "<br /><br />
			" . loc("install_db_password") . "<br />
			" . html::password("db_password") . "<br />";

		echo make_section(loc("install_database"), $s);
	}
	
	public function verify()
	{
		$this->error = test_connection(
			$this->host, $this->name, $this->username, $this->password);
		
		if ($this->error !== "")
			return false;
		
		return true;
	}
	
	public function install()
	{
		if (!$this->verify())
			return false;
	
		$this->create_tables();
		db();

		options::install_defaults();
		
		return true;
	}

	private function create_tables()
	{
		db()->execute(
			"create table forums (
				id int(10) unsigned not null auto_increment,
				name varchar(100) not null,
				primary key (id)
			) engine=innodb default charset=utf8;");

		db()->execute(
			"create table ip_posts (
				id int(10) unsigned not null auto_increment,
				ip_id int(10) unsigned not null,
				post_id int(10) unsigned not null,
				primary key (id)
			) engine=innodb default charset=utf8;");
		
		db()->execute(
			"create table ips (
				id int(10) unsigned not null auto_increment,
				ip varchar(50) not null,
				date_added datetime not null,
				banned int(10) unsigned not null,
				primary key (id)
			) engine=innodb default charset=utf8;");
		
		db()->execute(
			"create table marks (
				id int(10) unsigned not null auto_increment,
				user int(10) unsigned not null,
				thread int(10) unsigned not null,
				mark_time datetime not null,
				primary key (id)
			) engine=innodb default charset=utf8;");
		
		db()->execute(
			"create table options (
				id int(10) unsigned not null auto_increment,
				name text not null,
				value text not null,
				primary key (id)
			) engine=innodb default charset=utf8;");

		db()->execute(
			"create table posts (
				id int(10) unsigned not null auto_increment,
				thread int(10) unsigned not null,
				parent int(10) unsigned not null,
				content text not null,
				user int(10) unsigned not null,
				post_date datetime not null,
				edit_date datetime default null,
				author text,
				forum int(10) unsigned not null,
				queued int(10) unsigned not null,
				mod_comment text not null,
				primary key (id)
			) engine=innodb default charset=utf8;");

		db()->execute(
			"create table profiles (
				id int(10) unsigned not null auto_increment,
				user int(10) unsigned not null,
				name text,
				tz text not null,
				auto_subscribe int(10) unsigned not null,
				locale text no null,
				primary key (id)
			) engine=innodb default charset=utf8;");

		db()->execute(
			"create table recovery (
				id int(10) unsigned not null auto_increment,
				uuid varchar(36) not null,
				user int(10) unsigned not null,
				date_sent datetime not null,
				primary key (id)
			) engine=innodb default charset=utf8;");
		
		db()->execute(
			"create table registration (
				id int(10) unsigned not null auto_increment,
				uuid varchar(36) not null,
				user int(10) unsigned not null,
				date_sent datetime not null,
				primary key (id)
			) engine=innodb default charset=utf8;");

		db()->execute(
			"create table subscriptions (
				id int(10) unsigned not null auto_increment,
				user int(10) unsigned not null,
				thread int(10) unsigned not null,
				notified int(10) unsigned not null,
				primary key (id)
			) engine=innodb default charset=utf8;");

		db()->execute(
			"create table threads (
				id int(10) unsigned not null auto_increment,
				forum int(10) unsigned not null,
				title varchar(100) not null,
				poster int(10) unsigned not null,
				creation_date datetime not null,
				last_post_date datetime not null,
				last_post_id integer not null,
				changed_date datetime not null,
				primary key (id)
			) engine=innodb default charset=utf8;");

		db()->execute(
			"create table users (
				id int(10) unsigned not null auto_increment,
				email text not null,
				password varchar(255) not null,
				permissions int(10) unsigned not null,
				last_login datetime default null,
				reg_date datetime not null,
				confirmed int(10) unsigned not null,
				banned int(10) unsigned not null,
				automod int(10) unsigned not null,
				last_mark_date datetime not null,
				mod_notified int(10) not null,
				primary key (id)
			) engine=innodb default charset=utf8;");
	}
}

class admin_account_installer extends installer
{
	private $email = "";
	private $name = "";
	private $password1 = "";
	private $password2 = "";
	private $error = "";
	private $reg = null;
	
	public function __construct()
	{
		$this->reg = new registration();
	}
	
	public function output()
	{
		echo make_section(
			loc("install_admin_account"), $this->reg->make_fields());
	}
	
	public function verify()
	{
		return $this->reg->verify();
	}

	public function install()
	{
		return $this->reg->go(user_permission_administrator);
	}
}

class options_installer extends installer
{
	private $base_url = "";
	private $error = "";
	
	public function __construct()
	{
		if (isset($_POST["submit"]))
		{
			$this->base_url = trim(safe_post("base_url"));
		}
		else
		{
			$f = $_SERVER["PHP_SELF"];
			$p = strrpos($f, "/");
			if ($p !== FALSE)
				$f = substr($f, 0, $p);
			
			$this->base_url = "http://" . $_SERVER["SERVER_NAME"] . $f;
		}
	}
	
	public function output()
	{
		$s = "";
		
		if ($this->error !== "")
			$s .= html::wrap_div("bt_forum_error", $this->error) . "<br />";

		$opts = array();
		$this->make_option($opts, "base_url", $this->base_url, "text");
				
		$s .= make_options_table($opts);
		
		echo make_section(loc("install_options"), $s);
	}
	
	public function verify()
	{
		if ($this->base_url === "")
		{
			$this->error = loc("missing_fields");
			return false;
		}
		
		$f = $this->base_url . "/index.html";
		if (@file_get_contents($f) === FALSE)
		{
			$this->error = sprintf(loc("install_base_url_failed"), $f);
			return false;
		}
		
		return true;
	}
	
	public function install()
	{
		if (!$this->verify())
			return false;
		
		opt()->set_base_url($this->base_url);
		opt()->save();
		
		return true;
	}

	private function make_option(&$opts, $name, $value, $type)
	{
		$opts[$name] = array(
			"type"  => $type,
			"value" => $value,
			"help"  => loc("cp_option_" . $name . "_help"));
	}
		
}

function verify($installers)
{
	$failed = false;
	foreach ($installers as $i)
	{
		if (!$i->verify())
			$failed = true;
	}
	
	return !$failed;
}

function install($installers)
{
	$failed = false;
	foreach ($installers as $i)
	{
		if (!$i->install())
		{
			$failed = true;
			break;
		}
	}
	
	return !$failed;
}

$installers = array();

array_push($installers, new config_installer);
array_push($installers, new database_installer);
array_push($installers, new admin_account_installer);
array_push($installers, new options_installer);

if (isset($_POST["submit"]))
{
	if (verify($installers))
	{
		if (install($installers))
		{
			header("Location: index.html");
			exit;
		}
	}
}

?><!doctype html>

<html>
<?php output_head("forum install"); ?>
<body>
<div class="bt_forum_content">

<form method="post">

<?php 
foreach ($installers as $i)
	$i->output();

echo
	html::submit(loc("install_submit")) .
	html::wrap_div("bt_forum_install_info", loc("install_info"));
?>

</form>

</div>
</body>
</html>
