<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

require_once(__DIR__ . "/forum/forum.php");
require_once(__DIR__ . "/forum/layout.php");
require_once(__DIR__ . "/locale.php");
require_once(__DIR__ . "/options.php");

define("users_per_page", 100);

function output_banned_users($from)
{
	$r = search_users("", search_status_banned, $from, users_per_page);
	
	echo loc("cp_user_list_banned_caption") . "<br /><br />";
	output_user_table($r["users"], $from, $r["count"]);
}

function output_banned_ips($from)
{
	$r = banned_ips_list($from, users_per_page);

	echo loc("cp_user_list_banned_ips_caption") . "<br /><br />";
	output_ip_table($r["ips"], $from, $r["count"]);
}

function output_all_users($from)
{
	$r = search_users("", search_status_none, $from, users_per_page);
	
	echo loc("cp_user_list_all_caption") . "<br /><br />";
	output_user_table($r["users"], $from, $r["count"]);
}

function output_single_user($id)
{
	output_user_table(array(get_user($id)), 0, 1);
}

function output_search_users($s, $from)
{
	$r = search_users($s, search_status_none, $from, users_per_page);
	
	printf(loc("cp_user_list_search_caption"), $s);
	echo "<br /><br />";
	output_user_table($r["users"], $from, $r["count"]);
}

function output_cp_moderation()
{
	if (!opt()->moderated() || !logged_user()->can_moderate())
		return;
	
	$c = queued_posts_count();

	$body = "";
	if ($c == 0)
		$body = loc("cp_moderation_nothing");
	else if ($c == 1)
		$body = html::a("forum/mod.html", loc("cp_moderation_one"));
	else
		$body = html::a("forum/mod.html", sprintf(loc("cp_moderation_many"), $c));
	
	echo make_section(loc("cp_moderation_caption"), $body);
}

function output_moderation()
{
	$ps = queued_posts();
	
	if (count($ps) == 0)
	{
		echo loc("mod_nothing");
		return;
	}

	foreach ($ps as $p)
	{
		$u = $p->user();
		
		$s =
			"<div class=\"bt_forum_post_mod\">
				<div class=\"bt_forum_post_mod_col\">
				<form method=\"post\">
					" . html::submit(loc("mod_erase")) . "<br />
					" . html::radio("e", "this", loc("mod_erase_this"), true) . "<br />
					";
		
		if ($u->is_anonymous())
		{
			if ($p->has_ip())
			{
				$s .=
					html::radio("e", "ip", sprintf(loc("mod_erase_ip"), $p->ip())) . "<br />
					" . html::radio("e", "ban_ip", sprintf(loc("mod_erase_ban_ip"), $p->ip()));
			}
			else
			{
				$s .= "<div class=\"bt_forum_post_mod_no_ip\">" . loc("mod_ip_not_available") . "</div>";
			}
		}
		else
		{
			$s .=
				html::radio("e", "user", loc("mod_erase_user")) . "<br />
					" . html::radio("e", "ban_user", loc("mod_erase_ban_user"));
			
			if ($p->has_ip())
			{
				$s .=
					"<br />
					" . html::radio("e", "ban_user_and_ip", sprintf(loc("mod_erase_ban_user_and_ip"), $p->ip()));
			}
		}
		
		$s .=
					"<br />
					" . html::hidden("m", "e") . "
					" . html::hidden("p", $p->id()) . "
				</form>
				</div>
				
				<form method=\"post\" style=\"display: inline\">
				<div class=\"bt_forum_post_mod_col\">
					" . html::submit(loc("mod_accept")). "<br />
					" . html::radio("a", "this", loc("mod_accept_this"), true) . "<br />
					";
		
		if (!$u->is_anonymous())
		{
			$s .=
					html::radio("a", "user", loc("mod_accept_user")) . "<br />
					" . html::radio("a", "automod", loc("mod_accept_automod")) . "<br />
					";
		}
		
		$s .=
					html::hidden("m", "a") . "
					" . html::hidden("p", $p->id()) . "
				</div>
				
				<div class=\"bt_forum_post_mod_col\">
					" . loc("mod_comment") . "<br />
					" . html::textarea("c", 72, 3) . "
				</div>
				</form>
			</div>";
		
		echo make_single_post($p, 0, false, $s);
	}
	
	echo html::wrap_div("bt_forum_mod_help", loc("mod_help"));
}

function output_cp_user_delete($u)
{
	$us = array();
	array_push($us, $u);
	output_user_table($us, 0, 1);
	
	echo
		"<br />
		" . loc("cp_user_delete_confirm") . '<br />
		<br />
		<form method="post">
			' . html::submit(loc("cp_user_delete_submit")) . '
			' . html::hidden("u", $u->id()) . '
			' . make_back_inputs() . '
		</form>';
}

function make_cp_option(&$opts, $name, $type)
{
	$opts[$name] = array(
		"type"  => $type,
		"value" => opt()->{$name}(),
		"help"  => loc("cp_option_" . $name . "_help"));
}

function output_cp_options()
{
	$opts = array();

	make_cp_option($opts, "indent", "text");
	make_cp_option($opts, "readonly", "checkbox");
	make_cp_option($opts, "moderated", "checkbox");
	make_cp_option($opts, "base_url", "text");
	make_cp_option($opts, "log_ips", "checkbox");
	make_cp_option($opts, "auto_reg", "checkbox");
	
	make_section(loc("cp_options_caption"), "
		<form method=\"post\">
			" . make_options_table($opts) . "
			<br />
			" . html::submit(loc("cp_options_save")) . "
			" . html::hidden("w", "o") . "
		</form>");
}

function output_cp_user_list()
{
	echo make_section(loc("cp_user_list_caption"), "
		" . html::a("users.html", loc("cp_user_list_all")) . " |
		" . html::a("users.html?t=b", loc("cp_user_list_banned")) . " |
		" . html::a("users.html?t=ip", loc("cp_user_list_banned_ips")) . "
		<br /><br />
				
		<form action=\"users.html\">
			" . html::text("s") . "
			" . html::submit(loc("cp_user_list_search")) . "
		</form>");
}

function output_cp_user($u, $tr_class)
{
	$p = permissions_to_string($u->permissions());

	if (!$u->me())
	{
		$url = make_this_link("permissions.html?id=" . $u->id());
		$p = html::a($url, $p);
	}
	
	$ll = "";
	if (!$u->confirmed())
		$ll = loc("cp_user_unconfirmed");
	else if ($u->last_login() != never_timestamp)
		$ll = make_simple_date($u->last_login());
	
	$st = "";
	if (!$u->is_anonymous() && !$u->me())
	{
		$links = array();
		
		if ($u->banned())
		{
			$url = make_this_link("ban.html?u=" . $u->id() . "&r=1&noc=1");
			array_push($links, html::a($url, loc("cp_user_unban")));
		}
		else
		{
			$url = make_this_link("ban.html?u=" . $u->id() . "&noc=1");
			array_push($links, html::a($url, loc("cp_user_ban")));
		}
		
		if ($u->deletable())
		{
			$url = make_this_link("delete_user.html?u=" . $u->id());		
			array_push($links, html::a($url, loc("cp_user_delete")));
		}
		
		$st = implode(" | ", $links);
	}
	else if ($u->me())
	{
		$st = loc("cp_user_me");
	}
	
	echo "
			<tr $tr_class>
				<td>" . $u->email() . "</td>
				<td>" . $u->profile()->name() . "</td>
				<td>" . $p . "</td>
				<td>" . $st . "</td>
				<td>" . $ll . "</td>
				<td>" . $u->post_count() . "</td>";
	
	$lp = $u->last_post();
	if ($lp)
	{
		$c = make_simple_date($lp->post_date());
		$link = html::a(make_post_link_impl(
			$lp->thread_id(), $lp->id()), $c);
	
		echo "
				<td>$link</td>";
	}
	else
	{
		echo "
				<td>&nbsp;</td>";
	}
	
	echo "
				<td>" . make_simple_date($u->reg_date()) . "</td>
			</tr>";
}

function output_cp_ban($u, $from_post)
{
	if ($from_post !== null)
	{
		echo
			make_breadcrumbs(
				$from_post->forum(), $from_post->thread(), true) .
			"<br />";
	}
	
	$us = array();
	array_push($us, $u);
	output_user_table($us, 0, 1);

	echo
		"<br />" .
		loc("cp_ban_confirm") . '<br />
		<br />
		<form method="post">
			' . html::submit(loc("cp_ban_submit")) . '
			' . html::hidden("u", $u->id()) . '
			' . make_back_inputs() . '
		</form>';
}

function make_user_permissions_legend()
{
	$an = permissions_to_string(user_permission_anonymous, true);
	$r = permissions_to_string(user_permission_registered, true);
	$a = permissions_to_string(user_permission_administrator, true);

	return
		"<div class=\"bt_forum_permissions_legend\">" .
		loc("permission_legend") . "
		<blockquote>" .
		"$an<br />" .
		"$r<br />" .
		"$a" .
		"</blockquote></div>";
}

function output_user_permissions_legend()
{
	echo make_user_permissions_legend();
}

function output_user_table($us, $from, $total)
{
	if ($total == 0)
	{
		echo loc("cp_user_none");
		return;
	}
	
	output_pagination($total, users_per_page, $from);

	echo
		"<table class=\"bt_forum_cp_users_table\">
			<thead>
				<tr>
					<th>" . loc("cp_user_email") . "</th>
					<th>" . loc("cp_user_name") . "</th>
					<th>" . loc("cp_user_permissions") . "</th>
					<th>" . loc("cp_user_status") . "</th>
					<th>" . loc("cp_user_last_login") . "</th>
					<th>" . loc("cp_user_posts") . "</th>
					<th>" . loc("cp_user_last_post") . "</th>
					<th>" . loc("cp_user_registered_date") . "</th>
				</tr>
			</thead>
			
			<tbody>";
	
	$i = 0;
	foreach ($us as $u)
	{
		$tr_class = "";
		if (count($us) >= 3)
			$tr_class = "class=\"" . odd_even_class($i) . "\"";
		
		output_cp_user($u, $tr_class);
		++$i;
	}
	
	echo "
			</tbody>
		</table>";
		
	output_pagination($total, users_per_page, $from);
	output_user_permissions_legend();
}

function output_cp_ip($ip, $tr_class)
{
	echo "
				<tr $tr_class>
					<td>" . $ip->address() . "</td>
					<td>" . make_simple_date($ip->date_added()) . "</td>
					<td>" . html::a(make_this_link("ban.html?noc=1&r=1&ip=" . $ip->id()), loc("cp_ip_unban")) . "</td>
				</tr>";
}

function output_ip_table($ips, $from, $total)
{
	if ($total == 0)
	{
		echo loc("cp_ip_none");
		return;
	}
	
	output_pagination($total, users_per_page, $from);
	
	echo
		"<table class=\"bt_forum_cp_users_table\">
			<thead>
				<tr>
					<th>" . loc("cp_ip_address") . "</th>
					<th>" . loc("cp_ip_date_added") . "</th>
					<th>" . loc("cp_ip_status") . "</th>
				</tr>
			</thead>
			<tbody>";
	
	$i = 0;
	foreach ($ips as $ip)
	{
		$tr_class = "";
		if (count($ips) >= 3)
			$tr_class = "class=\"" . odd_even_class($i) . "\"";
		
		output_cp_ip($ip, $tr_class);
		++$i;
	}
	
	echo "
		</table>";
	
	output_pagination($total, users_per_page, $from);
}

function make_cp_forum($f)
{
	$links = array();
	
	$url = "delete_forum.html?f=" . $f->id();
	array_push($links, html::a($url, loc("cp_forum_delete")));
	
	$st = implode(" | ", $links);

	$s =
		"<tr>
			<td>" . $f->display_name() . "</td>
			<td>" . $f->thread_count() . "</td>
			<td>" . $f->post_count() . "</td>
			<td>$st</td>
		</tr>";
	
	return $s;
}

function make_cp_forums($fs = null)
{
	$s =
		"<table class=\"bt_forum_cp_forums_table\">
			<thead>
				<tr>
					<th>" . loc("cp_forum_name") . "</th>
					<th>" . loc("cp_forum_thread_count") . "</th>
					<th>" . loc("cp_forum_post_count") . "</th>
					<th>" . loc("cp_forum_status") . "</th>
				</tr>
			</thead>
			
			<tbody>
				";
	
	if ($fs === null)
		$fs = forum_list();
	
	foreach ($fs as $f)
		$s .= make_cp_forum($f);
				
	$s .= "
			</tbody>
		</table>";

	return $s;
}

function output_cp_forums()
{
	echo make_section(loc("cp_forums_caption"),
		make_cp_forums() . "
		<br />
		<form method=\"post\">
			" . html::text("f") . "
			" . html::submit(loc("cp_forum_add_submit")) . "
			" . html::hidden("w", "af") . "
		</form>");
}

function output_cp()
{
	output_cp_options();
	output_cp_forums();
	output_cp_user_list();
	output_cp_moderation();
}

function output_cp_delete_forum($f)
{
	$fs = array();
	array_push($fs, $f);
	echo make_cp_forums($fs);
	
	echo
		"<br />" .
		loc("cp_delete_forum_confirm") . '<br />
		<br />
		<form method="post">
			' . html::submit(loc("cp_delete_forum_submit")) . '
			' . html::hidden("f", $f->id()) . '
			' . make_back_inputs() . '
		</form>';
}

function make_permissions_combobox($sel)
{
	$ps = array();
	
	array_push($ps, array(
		"name" => loc("permission_anonymous"),
		"value" => user_permission_anonymous));
	
	array_push($ps, array(
		"name" => loc("permission_registered"),
		"value" => user_permission_registered));
	
	array_push($ps, array(
		"name" => loc("permission_administrator"),
		"value" => user_permission_administrator));
	
	$s =
		"<select name=\"p\">
			<option value=\"0\"></option>\n";

	foreach ($ps as $p)
	{
		$ss = "";
		if ($p["value"] == $sel)
			$ss = "selected ";
		
		$s .=
			"<option $ss value=\"" . $p["value"] . "\">" . $p["name"] .
			"</option>\n";
	}

	$s .= "</select>\n";
	
	return $s;
}

function make_predefined_permissions($u)
{
	$s =
		"<form method=\"post\">" .
			make_permissions_combobox($u->permissions()) .
			make_back_inputs() .
			html::submit(loc("cp_permissions_submit")) .
			make_user_permissions_legend() . "
		</form>";
	
	return $s;
}

function make_permission_checkboxes($u)
{
	$ps = localized_permissions();

	$s = "";
	
	foreach ($ps as $p)
	{
		$n = "p_" . $p["value"];
		
		$ss = false;
		if (($u->permissions() & $p["value"]) != 0)
			$ss = true;
	
		$s .=
			html::checkbox($n, $p["value"], $p["name"], $ss) . "
			<br />";
	}
	
	return $s;
}

function make_custom_permissions($u)
{
	$s =
		"<form method=\"post\">" .
		make_permission_checkboxes($u) . "
		<br />
		" . html::submit(loc("cp_permissions_submit")) . "
		</form>";
	
	return $s;
}

function output_cp_permissions($u)
{
	echo sprintf(loc("cp_permissions_caption"),
		$u->email(), $u->profile()->display_name()) . "<br /><br />
		" . loc("cp_permissions_current") . ": " .
		permissions_to_string($u->permissions(), true) . "<br /><br />";
	
	make_section(loc("cp_permissions_predefined"), 
		make_predefined_permissions($u));
	
	make_section(loc("cp_permissions_custom"),
		make_custom_permissions($u));
}

?>