<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

require_once(__DIR__ . "/../libs/FeedWriter/FeedWriter.php");
require_once(__DIR__ . "/../libs/FeedWriter/FeedTypes.php");
require_once(__DIR__ . "/../libs/FeedWriter/FeedItem.php");

class forum_not_found extends \Exception {}

class forum
{
	private $id = 0;
	private $name = "";
	private $last_post = null;
	
	public static function fields($table = "forums", $prefix = "forum_")
	{
		return "$table.id {$prefix}id, $table.name {$prefix}name";
	}
	
	public static function from_db($row, $prefix = "forum_")
	{
		$f = new forum();
		
		$f->id = (int)$row["{$prefix}id"];
		$f->name = $row["{$prefix}name"];
		$f->last_post = post::from_db($row);
		
		return $f;
	}
	
	public function id()
	{
		return $this->id;
	}
	
	public function name()
	{
		return $this->name;
	}
	
	public function display_name()
	{
		return make_safe_string($this->name);
	}
	
	public function threads($from, $limit)
	{
		$ps = query::get_all_threads($this->id, $from, $limit);
		
		$thread_ids = array();
		$threads = array();
		$index = array();
		
		foreach ($ps as $row)
		{
			$t = thread::from_db($row);

			$index[$t->id()] = count($threads);
			array_push($threads, $t);
			array_push($thread_ids, $t->id());
		}
		
		if (count($thread_ids) == 0)
			return $threads;
		
		$this->check_unread($thread_ids, $threads, $index);
		
		return $threads;
	}
	
	public function last_post()
	{
		return $this->last_post;
	}
	
	public function thread_count()
	{
		$ps = query::thread_count($this->id);
		
		$row = $ps->fetch();
		if ($row === FALSE)
			return 0;
		
		return (int)$row["c"];
	}
	
	public function post_count()
	{
		$ps = query::forum_post_count($this->id);
		
		$row = $ps->fetch();
		if ($row === FALSE)
			return 0;
		
		return (int)$row["c"];
	}
		
	function make_feed_head_links()
	{
		$s =
			'<link ' .
				'rel="alternate" ' .
				'type="application/rss+xml" ' .
				'title="This forum (' . $this->name() . ')" ' .
				'href="' . make_feed_link("f=" . $this->id) . '" />';
		
		return $s;
	}

	public function output_feed()
	{
		$file = make_feed_link("f=" . $this->id);
		$forum_link =
			opt()->base_url() . "/" . make_forum_link_impl($this->id);
		$desc =
			"Messages posted to the baretorrent forum " .
			"'" . $this->display_name() . "'";
		
		$f = new \RSS2FeedWriter();
		
		$f->setTitle($this->name());
		$f->setLink($forum_link);
		$f->setDescription($desc);
		$f->setSelfLink(opt()->base_url() . "/" . $file);
		
		$ps = query::get_last_forum_posts($this->id, posts_per_feed);
		
		foreach ($ps as $row)
		{
			$post = post::from_db($row);
			$i = $f->createNewItem();
			$post->set_feed_item($i);
			
			$f->addItem($i);
		}

		header("Content-Type: application/rss+xml");
		echo $f->generateFeed();
	}
	
	private function check_unread(&$thread_ids, &$threads, &$index)
	{
		if (logged_user()->is_anonymous())
		{
			foreach ($threads as &$t)
				$t->set_unread(false);
			
			return;
		}
		
		$ps = query::get_marks(logged_user()->id(), $thread_ids);
		
		$found_ids = array();
		foreach ($ps as $row)
		{
			$tid = (int)$row["thread"];
			$mt = db()->timestamp($row["mark_time"]);			
			
			array_push($found_ids, $tid);
			
			if (isset($index[$tid]))
			{
				$i = $index[$tid];
				
				if (isset($threads[$i]))
				{
					$t = $threads[$i];
					
					if ($t->changed_date() <= $mt)
						$t->set_unread(false);
				}
			}
		}
		
		foreach ($threads as &$t)
		{
			if ($t->changed_date() < logged_user()->last_mark_date())
			{
				$t->set_unread(false);
				array_push($found_ids, $t->id());
			}
		}
		
		$not_found = array_diff($thread_ids, $found_ids);
		
		if (count($not_found) > 0)
		{
			query::add_marks(
				logged_user()->id(), db()->never(), $not_found);
		}
	}
}

function forum_list()
{
	$ps = query::get_all_forums();

	$fs = array();
	foreach ($ps as $row)
		array_push($fs, forum::from_db($row));
		
	return $fs;
}

function get_forum($id)
{
	$ps = query::get_single_forum($id);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		throw new forum_not_found();
	
	return forum::from_db($row);
}

function add_forum($name)
{
	query::add_forum($name);
}

function delete_forum($f)
{
	db()->begin();
	query::delete_forum($f->id());
	db()->commit();
}

?>