<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

require_once(__DIR__ . "/forum.php");
require_once(__DIR__ . "/utility.php");
require_once(__DIR__ . "/../options.php");
require_once(__DIR__ . "/../locale.php");

define("threads_per_page", 50);
define("posts_per_page", 20);

function make_forum_root_link()
{
	return html::a(make_forum_root_link_impl(), loc("forums"));
}

function make_forum_link($f)
{
	return html::a(make_forum_link_impl($f->id()), $f->display_name());
}

function make_thread_link($t)
{
	return html::a(make_thread_link_impl($t->id()), $t->title());
}

function make_post_link($p)
{
	$c = sprintf(
		loc("post_link"),
		get_user_name($p, false),
		make_simple_date($p->post_date()));
	
	return html::a(make_post_link_impl($p->thread_id(), $p->id()), $c);
}

function make_thread_title($t)
{
	return html::wrap_span("bt_forum_thread_title", $t->title());
}

function make_forum_name($f)
{
	return html::wrap_span("bt_forum_forum_name", $f->display_name());
}

function make_new_thread_link($f)
{
	return html::a("post.html?f=" . $f->id(), loc("forum_new_thread"));
}

function make_modifiable_thread_title($thread)
{
	$u = logged_user();
	
	$edit = false;
	
	if ($u->can_edit_own() && $u->id() == $thread->poster())
		$edit = true;
	else if ($u->can_edit_all())
		$edit = true;
	
	if (!$edit)
		return $thread->title();
	
	$s =
		"<form method=\"post\" class=\"bt_forum_thread_rename_form\">
		" . html::text("t", $thread->title()) . "
		" . html::submit(loc("thread_rename_submit")) . "
		</form>
		";
	
	return $s;
}

function make_breadcrumbs($forum, $thread, $link_last = false)
{
	$parts = array();

	array_push($parts, make_forum_root_link());
	
	if ($forum !== null)
	{
		if ($thread !== null || $link_last)
			array_push($parts, make_forum_link($forum));
		else
			array_push($parts, $forum->display_name());
	}
	
	if ($thread !== null)
	{
		if ($link_last)
			array_push($parts, make_thread_link($thread));
		else
			array_push($parts, make_modifiable_thread_title($thread));
	}
	
	return html::wrap_div("bt_forum_breadcrumbs", implode(" > \n", $parts));
}

function get_display_ip($post)
{
	$s = $post->ip();
	if ($s === "")
		return "";
	
	return " ($s)";
}

function get_user_name($post, $with_user_link = true)
{
	try
	{
		$u = $post->user();
		$s = $u->profile()->display_name();

		if ($u->banned())
			$s = html::wrap_span("bt_forum_banned_user_name", $s);
		
		if (logged_user()->can_see_ips())
			$s .= get_display_ip($post);
		
		if ($with_user_link && logged_user()->can_ban() && !$u->is_anonymous())
			$s = html::a(make_user_link_impl($u->id()), $s);
		
		return $s;
	}
	catch(user_not_found $e)
	{
		return "unknown";
	}
}

function output_forum_list($fs)
{
	if (count($fs) == 0)
	{
		echo loc("no_forums") . "<br />";
		return;
	}
	
	echo
		"<table class=\"bt_forum_forum_table\">
			<thead>
				<tr>
					<th>" . loc("forum_name_caption") . "</th>
					<th>" . loc("forum_last_poster_caption") . "</th>
				</tr>
			</thead>
			<tbody>
			";
	
	$i = 0;
	
	foreach ($fs as $f)
	{
		$p = $f->last_post();
		
		$tr_class = "";
		if (count($fs) >= 3)
			$tr_class = "class=\"" . odd_even_class($i) . "\"";
		
		echo
			"	<tr $tr_class>
					<td>" . make_forum_link($f) . "</td>
				";

		if ($p === null)
			echo "	<td>&nbsp;</td>";
		else
			echo "	<td>" . make_post_link($p) . "</td>";

		echo "
				</tr>
			";
				
		++$i;
	}
	
	echo "
			</tbody>
		</table>";
}

function output_thread_row($t, $tr_class)
{
	$link = make_thread_link($t);
	
	if ($t->unread())
		$link = html::wrap_span("bt_forum_unread_thread", $link);
	
	echo
		"	<tr $tr_class>
				<td>$link</td>
				<td>" . $t->reply_count() . "</td>
			";

	$p = $t->last_post();
	
	if ($p === null)
		echo "	<td>&nbsp;</td>";
	else
		echo "	<td>" . make_post_link($p) . "</td>";
	
	echo "
			</tr>
		";
	
}

function output_thread_table($ts)
{
	echo
		"<table class=\"bt_forum_thread_table\">
			<thead>
				<tr>
					<th>" . loc("thread_title_caption") . "</th>
					<th>" . loc("thread_reply_count_caption") . "</th>
					<th>" . loc("thread_last_post_caption") . "</th>
				</tr>
			</thead>
			
			<tbody>
			";
	
	$i = 0;
	foreach ($ts as $t)
	{
		$tr_class = "";
		if (count($ts) >= 3)
			$tr_class = "class=\"" . odd_even_class($i) . "\"";
		
		output_thread_row($t, $tr_class);
		
		++$i;
	}
		
	echo
		"	</tbody>
		</table>";
}

function make_forum_controls($f)
{
	$left = array();
	$right = array();
	
	if (!opt()->readonly())
		array_push($left, make_new_thread_link($f));
	
	if (!logged_user()->is_anonymous())
	{
		$url = html::a(
			make_forum_link_impl($f->id()) . "&r=1", loc("forum_mark"));
		array_push($right, $url);
	}
	
	$left_div = html::wrap_span(
		"bt_forum_forum_controls_left", implode(" | \n", $left)) . "\n";
		
	$right_div = html::wrap_span(
		"bt_forum_forum_controls_right", implode(" | \n", $right)) . "\n";
	
	return
		html::wrap_div("bt_forum_forum_controls", $left_div . $right_div) .
		"\n\n";
}

function output_forum($f, $page)
{
	echo make_breadcrumbs($f, null) . "<br />";
	
	$ts = $f->threads($page * threads_per_page, threads_per_page);
	$total = $f->thread_count();
	
	$c = make_forum_controls($f);

	if (count($ts) > 0)
	{
		echo $c;
		output_pagination($total, threads_per_page, $page);
		output_thread_table($ts);
	}
	else
	{
		echo loc("forum_empty") . "<br />";
	}

	output_pagination($total, threads_per_page, $page);
	echo $c;
}

function make_thread_controls($t)
{
	$links = array();
	
	$u = logged_user();
	
	if (!opt()->readonly())
	{
		if ($u->can_erase())
		{
			$s = html::a('erase.html?t=' . $t->id(), loc("thread_erase"));
			array_push($links, $s);
		}
	}
	
	if ($u->can_subscribe())
	{
		if ($u->subscribed($t))
			$s = html::a('thread.html?id=' . $t->id() . '&s=0', loc("thread_unsubscribe"));
		else
			$s = html::a('thread.html?id=' . $t->id() . '&s=1', loc("thread_subscribe"));
		
		array_push($links, $s);
	}
	
	return implode(" | \n", $links);
}

function output_thread($t, $page)
{
	$c = html::wrap_div("bt_forum_thread_controls", make_thread_controls($t));

	echo make_breadcrumbs($t->forum(), $t) . "<br />";
	echo $c;
	
	$t->fetch_posts($page * posts_per_page, posts_per_page);
	$total = $t->post_count();
	
	output_pagination($total, posts_per_page, $page);
	
	foreach ($t->root_posts() as $p)
		output_post($p);
	
	echo $c;
}

function make_ban_link($u, $p)
{
	$s = "../ban.html?u=" . $u->id();
	
	if ($p !== null)
		$s .= "&p=" . $p->id();
	
	return make_this_link($s);
}

function bannable($u)
{
	if ($u->is_anonymous())
		return false;
	
	if ($u->banned())
		return false;
	
	if ($u->me())
		return false;
	
	return true;
}

function make_post_header($p, $show_controls)
{
	$links = array();
	
	if ($show_controls)
	{
		if (bannable($p->user()) && logged_user()->can_ban())
		{
			$url = make_ban_link($p->user(), $p);
			$s = html::a($url, loc("post_ban_user"));
	
			array_push($links, $s);
		}
	}
	
	$more = "";
	if (count($links) > 0)
		$more = " \n(" . implode(" | ", $links) . ")";
	
	return sprintf(
		loc("post_header"),
		date_to_string($p->post_date()), get_user_name($p)) .
		$more;
}

function make_post_controls($p)
{
	$u = logged_user();

	$links = array();
	
	if (!opt()->readonly())
	{
		if ($u->can_post())
		{
			$url = 'post.html?p=' . $p->id();
			$s = html::a($url, loc("post_reply"));
			array_push($links, $s);
			
			$url = 'post.html?p=' . $p->id() . '&q=1';
			$s = html::a($url, loc("post_quote"));
			array_push($links, $s);
		}
				
		if ($u->can_erase())
		{
			$url = "erase.html?p=" . $p->id();
			$s = html::a($url, loc("post_erase"));
			
			array_push($links, $s);
		}
		
		if ($p->editable())
		{
			$url = 'post.html?e=' . $p->id();
			$s = html::a($url, loc("post_edit"));
			
			array_push($links, $s);
		}
	}
	
	return implode(" | \n", $links);
}

function make_post_content($p)
{
	$mc = $p->mod_comment();
	
	$s = "";
	
	if ($mc != "")
	{
		$mc = sprintf(loc("post_mod_comment"), $mc);
		$s .= html::wrap_div("bt_forum_post_mod_comment", $mc);
	}
	
	$s .= $p->display_content();
	
	return $s;
}

function make_single_post($p, $indent, $show_controls, $more = "")
{
	if ($indent < 0)
		$indent = 0;
	
	$indent_em = $indent * opt()->indent();

	$s = "
		" .html::anchor("p" . $p->id()) . '
		<div class="bt_forum_post" style="margin-left: ' . $indent_em . 'em">
			' . html::wrap_div("bt_forum_post_header", make_post_header($p, $show_controls)) . '
			<div class="bt_forum_post_body">
				' . html::wrap_div("bt_forum_post_content", make_post_content($p)) . '
				';
	
	if ($show_controls)
		$s .= html::wrap_div("bt_forum_post_controls", make_post_controls($p));
	
	$s .= "
			</div>
			$more
		</div>
		";
		
	return $s;
}

function output_post($p, $indent = -1)
{
	// max indent
	if ($indent > 6)
		$indent = 6;
	
	echo make_single_post($p, $indent, true);
		
	foreach ($p->child_posts() as $cp)
		output_post($cp, $indent + 1);
}

function output_edit_form($p)
{
	echo "		<form method=\"post\">" .
		loc("edit_message") . "<br />
		" . html::textarea("m", 72, 20, $p->content(), true) . "
		<br />
		<br />";
	
	if (logged_user()->can_moderate())
	{
		echo "
			" . loc("edit_mod_comment") . "<br />
			" . html::textarea("c", 72, 3, $p->mod_comment());
	}
	
	echo "<br /><br />
		" . html::submit(loc("edit_submit")) . "
		" . html::hidden("e", $p->id()) .
		"</form>\n";
}

function output_new_post_form($tp)
{
	echo "		<form method=\"post\">";
	
	$af = true;
	
	if ($tp->new_thread())
	{
		echo
			loc("new_post_title") . "<br />
			" . html::text("t", $tp->title(), $af) . "
			<br /><br />";
			
		$af = false;
	}
	
	if (logged_user()->is_anonymous())
	{
		echo 
			loc("new_post_author") . "<br />
			" . html::text("a", $tp->author(), $af) . "
			<br /><br />";
		
		$af = false;
	}
	
	echo
		loc("new_post_message") . " " . loc("new_post_message_cut") . "
		<br />" . html::textarea("m", 72, 20, $tp->content(), $af) . "
		";
	
	if (opt()->moderated() && !logged_user()->automod())
	{
		echo
			"<div class=\"bt_forum_post_mod_warning\">
			" . loc("new_post_mod_warning") . "
			</div>";
	}
	else
	{
		echo "<br /><br />";
	}
	
	echo "
		" . html::submit(loc("new_post_submit")) . "
		";
		
	$af = false;

	if ($tp->reply())
		echo html::hidden("p", $tp->parent_id());
	else
		echo html::hidden("f", $tp->forum_id());
	
	echo "</form>";
}

function output_new_thread($tp, $problem)
{
	$f = get_forum($tp->forum_id());
	
	echo 
		make_breadcrumbs($f, null, true) . "<br />" .
		sprintf(loc("new_thread_caption"), make_forum_name($f)) . "
		<br /><br />";
	
	if ($problem !== "")
		echo "<div class=\"bt_forum_post_problem\">$problem</div>";

	output_new_post_form($tp);
}

function output_new_post($tp, $problem)
{
	$parent = get_single_post($tp->parent_id());
	$thread = $parent->thread();
	$forum = $thread->forum();

	echo
		make_breadcrumbs($forum, $thread, true) . "<br />" .
		sprintf(loc("reply_caption"), make_thread_title($thread)) . "
		<br /><br />";
	
	if ($problem !== "")
		echo "<div class=\"bt_forum_post_problem\">$problem</div>";
	
	output_new_post_form($tp);
}

function output_edit($p, $problem)
{
	echo 
		make_breadcrumbs($p->forum(), $p->thread(), true) . "<br />" .
		loc("edit_caption") . "
		<br /><br />";
	
	if ($problem !== "")
		echo "<div class=\"bt_forum_post_problem\">$problem</div>";

	output_edit_form($p);
}

function output_erase_post($p)
{
	echo
		make_breadcrumbs($p->forum(), $p->thread(), true) . "<br />
		" . loc("erase_post_confirm") . "<br />
		<br />
		" . make_single_post($p, 0, false) . "
		<form method=\"post\">
			" . html::submit(loc("erase_post_submit")) . "
			" . html::hidden("p", $p->id()) . "
		</form>";
}

function output_erase_thread($t)
{
	echo
		make_breadcrumbs($t->forum(), $t, true) . "<br />" .
		loc("erase_thread_confirm") . "<br />
		<br />" .
		make_single_post($t->first_post(), 0, false);
	
	$c = $t->reply_count();
	
	if ($c == 1)
		echo loc("erase_thread_one_more");
	else if ($c >= 2)
		printf(loc("erase_thread_many_more"), $c);
		
	echo
		"<br /><br />
		<form method=\"post\">
			" . html::submit(loc("erase_thread_submit")) . "
			" . html::hidden("t", $t->id()) . "
		</form>";
}

class registration
{
	private $email = "";
	private $name = "";
	private $p1 = "";
	private $p2 = "";
	private $error = "";
	private $focus = false;
	private $pending = false;
	
	public function __construct($focus = false)
	{
		$this->focus = $focus;
		
		if (isset($_POST["submit"]))
		{
			$this->email = trim(safe_post("reg_email"));
			$this->name = trim(safe_post("reg_name"));
			$this->p1 = safe_post("reg_password1");
			$this->p2 = safe_post("reg_password2");
		}
		else if (isset($_GET["h"]))
		{
			$h = safe_get("h");
			
			if (confirm_registration($h))
			{
				header("Location: index.html");
				exit;
			}
			else
			{
				$this->error = loc("register_bad_confirm");
			}
		}
	}
	
	public function output()
	{
		echo $this->make_output();
	}
	
	public function make_output()
	{
		$s =
			"<div class=\"bt_forum_register\">
				<div class=\"bt_forum_register_content\">
			
				<form method=\"post\">
				" . $this->make_fields() . "
				<br />
				" . html::submit(loc("register_submit")) . "
				<br />
				" . make_back_inputs() . " 
				</form>
				
				</div>
			</div>";
		
		return $s;
	}
	
	public function make_fields()
	{
		$s = "";

		if ($this->error !== "")
			$s .= html::wrap_div("bt_forum_error", $this->error) . "<br />";
		
		$s .= "
			" . loc("register_email") . "
			" . html::wrap_div("bt_forum_register_help", loc("register_email_help")) . "<br />
			" . html::text("reg_email", $this->email, $this->focus) . "<br />
			<br />
			" . loc("register_name") . "
			" . html::wrap_div("bt_forum_register_help", loc("register_name_help")) . "<br />
			" . html::text("reg_name", $this->name) . "<br />
			<br />
			" . loc("register_password") . "<br />
			" . html::password("reg_password1") . "<br />
			<br />
			" . loc("register_password_again") . "<br />
			" . html::password("reg_password2") . "<br />";
		
		return $s;
	}
	
	function verify()
	{
		if ($this->email == "" || $this->p1 == "" || $this->p2 == "")
		{
			$this->error = loc("missing_fields");
			return false;
		}

		if ($this->p1 != $this->p2)
		{
			$this->error = loc("register_password_mismatch");
			return false;
		}
		
		return true;
	}

	function go($perm = user_permission_registered)
	{
		if (!$this->verify())
			return false;
		
		try
		{
			$id = register($this->email, $this->name, $this->p1, $perm);
			
			if (opt()->auto_reg())
				login($id);
			else
				$this->pending = true;
		
			return true;
		}
		catch(user_already_exists $e)
		{
			$this->error = loc("register_already_exists");
			return false;
		}
		catch(bad_password $e)
		{
			$this->error = loc("register_bad_password");
			return false;
		}
	}
}

function output_recover()
{
	echo
		loc("recover_help") . "<br/>
		<br />
		<form method=\"post\">
			" . loc("recover_email") . "<br />
			" . html::text("email", "", true) . "<br />
			<br />
			" . html::submit(loc("recover_submit")) . "
		</form>";
}

function output_reset_password($rec)
{
	echo
		loc("reset_caption") . "<br /><br />
		<form method=\"post\">
		" . loc("reset_email") . "<br />
		" . html::text("email") . "<br />
		<br />
		" . loc("reset_password") . "<br />
		" . html::password("password1") . "<br />
		<br />
		" . loc("reset_password_again") . "<br />
		" . html::password("password2") . "<br />
		<br />
		" . html::submit(loc("reset_submit")) . "
		<br />
		" . html::hidden("h", $rec->uuid()) . "
		" . make_back_inputs() . " 
		</form>";
}

?>