<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

define("accept_single_flag", 1);
define("accept_for_user_flag", 2);
define("accept_all_flag", 3);

class post_not_found extends \Exception {}

class post
{
	private $id = 0;
	private $forum_id = 0;
	private $thread_id = 0;
	private $parent_id = 0;
	private $content = "";
	private $user_id = 0;
	private $author = "";
	private $user = null;
	private $ip_user = null;
	private $post_date = 0;
	private $edit_date = 0;
	private $queued = false;
	private $mod_comment = "";
	private $children = array();
	
	public static function fields($table = "posts",  $prefix = "post_")
	{
		return
			"$table.id {$prefix}id, " .
			"$table.forum {$prefix}forum, " .
			"$table.thread {$prefix}thread, " .
			"$table.parent {$prefix}parent, " .
			"$table.content {$prefix}content, " .
			"$table.user {$prefix}user, " .
			"$table.author {$prefix}author, " .
			"$table.post_date {$prefix}post_date, " .
			"$table.edit_date {$prefix}edit_date, " .
			"$table.queued {$prefix}queued, " .
			"$table.mod_comment {$prefix}mod_comment ";
	}
	
	public static function from_db($row, $prefix = "post_")
	{
		if ($row["{$prefix}id"] === null)
			return null;
	
		$p = new post();
		
		$p->id = (int)$row["{$prefix}id"];
		$p->forum_id = (int)$row["{$prefix}forum"];
		$p->thread_id = (int)$row["{$prefix}thread"];
		$p->parent_id = (int)$row["{$prefix}parent"];
		$p->content = $row["{$prefix}content"];
		$p->user_id = (int)$row["{$prefix}user"];
		$p->author = $row["{$prefix}author"];
		$p->post_date = db()->timestamp($row["{$prefix}post_date"]);
		$p->edit_date = db()->timestamp($row["{$prefix}edit_date"]);
		$p->queued = ($row["{$prefix}queued"] == "0" ? false : true);
		$p->mod_comment = $row["{$prefix}mod_comment"];
		$p->user = user::from_db($row);
		$p->ip_user = ip_user::from_db($row);
		
		return $p;
	}
	
	public function distribute($flat)
	{
		foreach ($flat as $p)
		{
			if ($p->parent_id() == $this->id)
			{
				array_push($this->children, $p);
				$p->distribute($flat);
			}
		}
	}
	
	public function id()
	{
		return $this->id;
	}
	
	public function forum_id()
	{
		return $this->forum_id;
	}
	
	public function forum()
	{
		return get_forum($this->forum_id);
	}
	
	public function thread_id()
	{
		return $this->thread_id;
	}
	
	public function thread()
	{
		return get_thread($this->thread_id);
	}
	
	public function first_post()
	{
		return ($this->parent_id == 0);
	}
	
	public function parent_id()
	{
		return $this->parent_id;
	}
	
	public function content()
	{
		return $this->content;
	}
	
	public function display_content()
	{
		return htmlEscapeAndLinkUrls($this->content);
	}
	
	public function set_content($c, $cm)
	{
		query::update_post($this->id, $c, $cm);
		
		$this->mod_comment = $cm;
		$this->content = $c;
	}
	
	public function quote()
	{
		$lines = explode("\n", $this->content);
		return "> " . implode("\n> ", $lines) . "\n\n";
	}
	
	public function user_id()
	{
		return $this->user_id;
	}
	
	public function user()
	{
		if ($this->user == null)
		{
			if ($this->user_id === 0)
			{
				$this->user = user::anonymous($this->author);
			}
			else
			{
				try
				{
					$this->user = get_user($this->user_id);
				}
				catch(user_not_found $e)
				{
					$this->user = user::anonymous();
				}
			}
		}
		
		return $this->user;
	}

	public function has_ip()
	{
		return ($this->ip_user != null);
	}
	
	public function ip()
	{
		if ($this->ip_user === null)
			return "";
		
		return $this->ip_user->address();
	}
	
	public function post_date()
	{
		return $this->post_date;
	}

	public function queued()
	{
		return $this->queued;
	}
	
	public function mod_comment()
	{
		return $this->mod_comment;
	}
	
	public function child_posts()
	{
		return $this->children;
	}
	
	public function accept($comment)
	{
		accept_impl(accept_single_flag, $this->id, $comment);
	}

	public function editable()
	{
		$u = logged_user();
		
		if ($u->can_edit_all())
			return true;
		
		if ($u->can_edit_own() && $u->id() == $this->user_id)
			return true;
		
		return false;
	}

	public function set_feed_item($i)
	{
		$url =
			opt()->base_url() . "/" .
			make_post_link_impl($this->thread_id, $this->id);
		
		$title =
			$this->user()->profile()->display_name() . " wrote in " .
			$this->thread()->title();
		
		$c =
			"<div " .
				"style=\"white-space: pre-wrap; " .
				"font-family: monospace;\">" .
			$this->display_content() .
			"</div>";
		
		$i->setTitle($title);
		$i->setLink($url);
		$i->setDate($this->post_date());
		$i->setDescription($c);
		$i->setId($url, true);
	}
}

class temporary_post
{
	private $parent_id = 0;
	private $forum_id = 0;
	private $title = "";
	private $author = "";
	private $content = "";
	
	static public function create_reply($parent_id, $author, $content)
	{
		$p = new temporary_post();
		
		$p->parent_id = $parent_id;
		$p->author = $author;
		$p->content = $content;
		
		return $p;
	}
	
	static public function create_new_thread(
		$forum_id, $title, $author, $content)
	{
		$p = new temporary_post();
		
		$p->forum_id = $forum_id;
		$p->title = $title;
		$p->author = $author;
		$p->content = $content;
		
		return $p;
	}
	
	public function reply()
	{
		return ($this->parent_id != 0);
	}
	
	public function new_thread()
	{
		return ($this->forum_id != 0);
	}
	
	public function parent_id()
	{
		return $this->parent_id;
	}
	
	public function forum_id()
	{
		return $this->forum_id;
	}
	
	public function title()
	{
		return $this->title;
	}
	
	public function author()
	{
		return $this->author;
	}
	
	public function content()
	{
		return $this->content;
	}
}


function get_posts($thread_id, $from, $limit)
{
	$ps = query::get_posts($thread_id, $from, $limit);
	
	$flat = array();
	$index = array();
	foreach ($ps as $row)
	{
		$p = post::from_db($row);
		$index[$p->id()] = count($flat);
		array_push($flat, $p);
	}
	
	$root = array();
	foreach ($flat as $p)
	{	
		if (!isset($index[$p->parent_id()]))
		{
			array_push($root, $p);
			$p->distribute($flat);
		}
	}
	
	return $root;
}

function get_single_post($id)
{
	$ps = query::get_single_post($id);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		throw new post_not_found();
	
	return post::from_db($row);
}

function queued_posts_count()
{
	$ps = query::queued_posts_count();
	
	$row = $ps->fetch();
	if ($row === FALSE)
		return 0;
	
	return (int)$row["c"];
}

function queued_posts()
{
	$ps = query::queued_posts();
	
	$r = array();
	foreach ($ps as $row)
		array_push($r, post::from_db($row));
	
	return $r;
}

function accept_impl($flag, $id = 0, $comment = "")
{
	$notify = array();
	
	switch ($flag)
	{
		case accept_single_flag:
		{
			$post = get_single_post($id);
			if ($post === null || !$post->queued())
				return;
			
			array_push($notify, array(
				"post" => $post->id(),
				"thread" => $post->thread_id(),
				"user" => $post->user_id()));
			
			query::accept_single_post($id, $comment);
			break;
		}
		
		case accept_for_user_flag:
		{
			$u = get_user($id);
			if ($u === null || $u->is_anonymous())
				return;

			$ps = query::get_queued_posts_for_user_by_thread($u->id());
			
			foreach ($ps as $row)
			{
				array_push($notify, array(
					"post" => (int)$row["id"],					
					"thread" => (int)$row["thread"],
					"user" => (int)$row["user"]));
			}

			query::accept_posts_for_user($u->id());
			break;
		}
		
		case accept_all_flag:
		{
			$ps = query::get_queued_posts_by_thread();
			
			foreach ($ps as $row)
			{
				array_push($notify, array(
					"post" => (int)$row["id"],
					"thread" => (int)$row["thread"],
					"user" => (int)$row["user"]));
			}
			
			query::accept_all_posts();
			break;
		}
		
		default:
		{
			return;
		}
	}
	
	if (count($notify) == 0)
		return;
	
	foreach ($notify as $n)
	{
		set_thread_changed($n["thread"]);
		notify_subscriptions($n["user"], $n["thread"], $n["post"]);
		set_last_post($n["thread"]);
	}
}

function accept_all_pending()
{
	accept_impl(accept_all_flag);
}

function erase_pending_ip($ip)
{
	query::delete_queued_for_ip($ip);
}

function erase_post($p)
{
	$thread = $p->thread();
	
	if ($thread->reply_count() == 0)
	{
		erase_thread($thread);
		return;
	}
	
	db()->begin();
	
	$parent = $p->parent_id();
	
	query::delete_post($p->id());
	query::reparent_posts($p->id(), $parent);

	set_last_post($thread->id());
	
	db()->commit();
}

function notify_moderators($post_id)
{
	$ps = query::get_unnotified_moderators();
	
	$link = opt()->base_url() . "/" . make_mod_link();
	
	$us = array();
	foreach ($ps as $row)
	{
		$u = user::from_db($row);

		$s = loc("mod_notify_subject", $u->profile()->locale());
		$b = sprintf(
			loc("mod_notify_body", $u->profile()->locale()), $link);
		
		send_email($u->email(), $s, $b);
	}
	
	query::notify_moderators();
}

function add_reply($thread, $parent, $user, $message, $use_trans = true)
{
	// this shouldn't happen
	if (opt()->moderated() && $parent != null && $parent->queued())
		return 0;

	$message = wordwrap($message, max_line_width);

	if ($use_trans)
		db()->begin();
		
	$parent_id = 0;
	if ($parent != null)
		$parent_id = $parent->id();
	
	$author = null;
	if ($user->is_anonymous())
		$author = $user->profile()->name();
	
	$queued = false;
	if (opt()->moderated() && !logged_user()->automod())
		$queued = true;

	query::add_post(
		$thread->forum_id(), $thread->id(), $parent_id, $user->id(),
		$message, $author, $queued);
	
	$id = db()->last_insert_id();
	
	if (opt()->log_ips())
		add_ip_post($id);
	
	if ($user->profile()->auto_subscribe())
		$user->subscribe($thread);

	set_last_post($thread->id());

	if ($use_trans)
		db()->commit();

	if ($queued)
	{
		notify_moderators($id);
	}
	else
	{
		set_thread_changed($thread->id());
		notify_subscriptions($user->id(), $thread->id(), $id);
	}
		
	return $id;
}

?>