<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

class thread_not_found extends \Exception {}

class thread
{
	private $id = 0;
	private $title = "";
	private $poster = 0;
	private $creation_date = 0;
	private $reply_count = 0;
	private $last_post = null;
	private $changed_date = 0;
	private $forum = null;
	private $forum_id = 0;
	private $posts = null;
	private $unread = true;
	
	public static function fields(
		$table = "threads", $prefix = "thread_")
	{
		return
			"$table.id {$prefix}id, " .
			"$table.title {$prefix}title, " .
			"$table.poster {$prefix}poster, " .
			"$table.creation_date {$prefix}creation_date, " .
			"$table.changed_date {$prefix}changed_date, " .
			"$table.forum {$prefix}forum, " .
			"$table.last_post_date {$prefix}last_post_date, " .
			"$table.last_post_id {$prefix}last_post_id ";
	}
	
	public static function from_db($row, $prefix = "thread_")
	{
		$t = new thread();
		
		$t->id = (int)$row["{$prefix}id"];
		$t->title = $row["{$prefix}title"];
		$t->poster = (int)$row["{$prefix}poster"];
		$t->creation_date = db()->timestamp($row["{$prefix}creation_date"]);
		$t->changed_date = db()->timestamp($row["{$prefix}changed_date"]);
		$t->forum_id = (int)$row["{$prefix}forum"];
		$t->last_post = post::from_db($row);
		$t->reply_count = (int)$row["reply_count"];
		
		return $t;
	}
	
	public function id()
	{
		return $this->id;
	}
	
	public function title()
	{
		return make_safe_string($this->title);
	}
	
	public function set_title($t)
	{
		if ($t == "")
			return;
		
		$this->title = $t;
		$this->save();
	}
	
	public function poster()
	{
		return $this->poster;
	}
	
	public function creation_date()
	{
		return $this->creation_date;
	}
	
	public function forum_id()
	{
		return $this->forum_id;
	}
	
	public function forum()
	{
		if ($this->forum == null)
			$this->forum = get_forum($this->forum_id);
		
		return $this->forum;
	}
	
	public function unread()
	{
		return $this->unread;
	}
	
	public function mark()
	{
		if (!$this->unread)
			return;
		
		query::set_mark(logged_user()->id(), db()->now(), $this->id);
		
		$this->unread = false;
	}
	
	// internal
	public function set_unread($b)
	{
		$this->unread = ($b ? true : false);
	}
	
	public function queued()
	{
		if (count($this->posts) > 0)
			return $this->posts[0]->queued();
		else
			return ($this->first_post() == null);
	}
	
	public function first_post()
	{
		$ps = query::thread_first_post($this->id);
		
		$row = $ps->fetch();
		if ($row === FALSE)
			return null;
		
		return post::from_db($row);
	}
	
	public function last_post()
	{
		return $this->last_post;
	}
	
	public function changed_date()
	{
		return $this->changed_date;
	}
	
	public function fetch_posts($from, $limit)
	{
		$this->posts = get_posts($this->id, $from, $limit);
	}
	
	public function root_posts()
	{
		return $this->posts;
	}
	
	public function post_count()
	{
		$ps = query::get_post_count($this->id);
		$row = $ps->fetch();
		
		if ($row === FALSE)
			return 0;
		
		return (int)$row["c"];
	}
	
	public function reply_count()
	{
		return $this->reply_count;
	}
	
	private function save()
	{
		query::update_thread($this->id, $this->title);
	}
		
	function make_feed_head_links()
	{
		$s =
			'<link ' .
				'rel="alternate" ' .
				'type="application/rss+xml" ' .
				'title="This thread (' . $this->title() . ')" ' .
				'href="' . make_feed_link("t=" . $this->id) . '" />
			' . $this->forum()->make_feed_head_links();
		
		return $s;
	}

	public function output_feed()
	{
		$file = make_feed_link("t=" . $this->id);
		$thread_link =
			opt()->base_url() . "/" . make_thread_link_impl($this->id);
		$desc =
			"Messages posted to the baretorrent thread " .
			"'" . $this->title() . "'";
		
		$f = new \RSS2FeedWriter();
		
		$f->setTitle($this->title());
		$f->setLink($thread_link);
		$f->setDescription($desc);
		$f->setSelfLink(opt()->base_url() . "/" . $file);
		
		$ps = query::get_last_thread_posts($this->id, posts_per_feed);
		
		foreach ($ps as $row)
		{
			$post = post::from_db($row);
			
			$i = $f->createNewItem();
			
			$post->set_feed_item($i);
			
			$f->addItem($i);
		}

		header("Content-Type: application/rss+xml");
		echo $f->generateFeed();
	}
}

function get_thread($id)
{
	$ps = query::get_single_thread($id);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		throw new thread_not_found();
		
	return thread::from_db($row);
}

function thread_exists($id)
{
	$ps = query::get_single_thread($id);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		return false;
	
	return true;
}
	
function set_thread_changed($id)
{
	query::set_thread_changed($id, db()->now());
}

function set_last_post($thread_id)
{
	$ps = query::thread_last_post($thread_id);

	$id = 0;
	$post_date = 0;

	$row = $ps->fetch();
	if ($row !== FALSE)
	{
		$id = (int)$row["id"];
		$post_date = db()->timestamp($row["post_date"]);
	}

	$ps = query::set_thread_last_post(
		$thread_id, $id, db()->date_time($post_date));
}

function add_thread($forum, $user, $title, $message)
{
	db()->begin();

	query::add_thread($forum->id(), $title, $user->id());
	
	$tid = db()->last_insert_id();
	$pid = add_reply(get_thread($tid), null, $user, $message, false);

	db()->commit();
	
	return $pid;
}

function erase_thread($t)
{
	db()->begin();
	query::delete_thread($t->id());
	db()->commit();
}


?>