<?php

require_once(__DIR__ . "/version.php");
require_once(__DIR__ . "/user.inc.php");
require_once(__DIR__ . "/utility.php");
require_once(__DIR__ . "/forum/post.inc.php");

function make_nav_menu($c, $path)
{
	$menu = array();
	
	$menu["download"] = $path . "index.html";
	$menu["extensions"] = $path . "ext/";
	$menu["forum"] = $path . "forum/";
	$menu["developers"] = $path . "dev.html";
	$menu["privacy"] = $path . "privacy.html";
	$menu["launchpad"] = "https://launchpad.net/baretorrent/";
	
	$center = array();
	foreach ($menu as $n=>$u)
	{
		if ($n === $c)
			array_push($center, "<span class=\"selected-menu\">$n</span>");
		else
			array_push($center, "<a href=\"$u\">$n</a>");
	}
	
	$s = html::wrap_div("menu_center_top", implode(" | \n", $center));
	
	$s = html::wrap_div("menu_div",
		html::wrap_div("menu_center", $s)) . "\n" .
		"\n\n";
	
	return $s;
}

function make_anonymous_user_menu($path)
{
	$menu = array();
	
	$s = html::a(make_this_link("{$path}login.html"), loc("login_link"));
	array_push($menu, $s);
	
	$s = html::a(make_this_link("{$path}register.html"), loc("register_link"));
	array_push($menu, $s);
	
	/*$alc = available_locales();
	if (count($alc) > 0)
	{
		$s =
			"<form method=\"post\">
			<select name=\"lc\">
			";
		
		foreach ($alc as $lc)
		{
			$v = $lc->ietf();
			$c = $lc->name();
			$sel = "";
			
			if (logged_user()->profile()->locale() === $v)
				$sel = "selected";
			
			$s .= "<option $sel value=\"$v\">$c</option>\n";
		}
		
		$s .=
			"</select>
			" . html::submit(loc("locale_change_submit")) . "
			</form>";
		
		array_push($left, $s);
	}*/
	
	return html::wrap_div("menu_logo_right", implode(" | \n", $menu));
}

function make_logged_user_menu($path)
{
	$u = logged_user();
	$menu = array();
	
	$s = html::a(make_this_link("{$path}logout.html"), loc("logout_link"));
	array_push($menu, $s);
	
	$s = html::a(make_this_link("{$path}profile.html"), loc("profile_link"));
	array_push($menu, $s);
	
	if ($u->can_cp())
	{
		$s = html::a("{$path}cp.html", loc("cp_link"));
		array_push($menu, $s);
		
		if ($u->can_moderate())
		{
			$q = queued_posts_count();
			if ($q > 0)
			{
				$s = html::a("{$path}forum/mod.html", sprintf(loc("mod_link"), $q));
				array_push($menu, $s);
			}
		}
	}

	return html::wrap_div("menu_logo_right", $u->email() . "<br />" . implode(" | \n", $menu));
}

function make_user_menu($path)
{
	if (logged_user()->is_anonymous())
		return make_anonymous_user_menu($path);
	else
		return make_logged_user_menu($path);
}

function output_menu($c, $path)
{
	$nav = make_nav_menu($c, $path);
	$user = make_user_menu($c, $path);
	
	echo "<nav class=\"menu\">$nav</nav>";
}

function output_head($c, $title, $more_head = "")
{
	$path = "";
	if ($c === "forum" || $c === "extensions")
		$path = "../";
	
	$icon = $path . "images/icon_64.png";
	
	?><!doctype html>
<html>

<head>
<meta charset="UTF-8" />
<title><?php echo $title; ?></title>
<link rel="shortcut icon" href="favicon.ico" />
<link rel="stylesheet" href="<?php echo $path; ?>style.css" type="text/css" />

<!--[if lt IE 8]>
<link rel="stylesheet" href="style-ie.css" type="text/css" />
<![endif]-->
	<?php echo $more_head; ?>
</head>
<body>
<div class="title-div">
	<?php echo make_user_menu($path); ?>
	<div class="title-div-inner">
	<a href="<?php echo $path; ?>index.html"><img src="<?php echo $icon; ?>" alt="baretorrent icon" class="title-image" /></a>
	<div class="title-text-div">
		<div class="title-text">baretorrent</div>
		<span class="title-subtext">a simple bittorrent client</span><br />
		<span class="title-version"><?php echo version_string(); ?></span>
	</div>
	</div>
</div>

	<?php
	output_menu($c, $path);
}

function output_footer()
{
	global $g_page_start_time;
	
	$end = gettimeofday(true);
	//echo $end - $g_page_start_time;
	
	debug_dump_queries();
}
?>