<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

require_once(__DIR__ . "/utility.php");

define("locale_directory", "/locales");

$g_current_locale = null;
$g_locales = array();
$g_locale_files = array();

class locale_file
{
	private $ietf;
	private $name;
	private $setter;
	
	public function __construct($ietf, $name, $setter)
	{
		$this->ietf = $ietf;
		$this->name = $name;
		$this->setter = $setter;
	}
	
	public function ietf()
	{
		return $this->ietf;
	}
	
	public function name()
	{
		return $this->name;
	}
	
	public function set()
	{
		$f = $this->setter;
		$f();
	}
}

function set_locale($ietf, $a)
{
	global $g_locales;
	$g_locales[$ietf] = $a;
}

function add_locale($ietf, $localized_name, $setter)
{
	global $g_locale_files;
	$g_locale_files[$ietf] =
		new locale_file($ietf, $localized_name, $setter);
}

function default_locale_ietf()
{
	return "en-US";
}

function make_locale_filename($ietf)
{
	$fn = basename($ietf);
	$p = join_path(
		dirname(__FILE__), locale_directory, ($fn . ".inc.php"));
	
	return $p;
}

function valid_locale($ietf)
{
	return file_exists(make_locale_filename($ietf));
}

function include_locale($ietf)
{
	global $g_locale_files;
	
	if ($ietf === "")
		$ietf = default_locale_ietf();
	
	$p = make_locale_filename($ietf);
	if (file_exists($p))
	{
		include_once($p);
		
		if (isset($g_locale_files[$ietf]))
			$g_locale_files[$ietf]->set();
		else
			$ietf = default_locale_ietf();
	}
	else
	{
		$ietf = default_locale_ietf();
	}
	
	return $ietf;
}

function loc($name, $ietf = null)
{
	global $g_current_locale;
	global $g_locales;
	
	if ($ietf === null)
	{
		if ($g_current_locale === null)
		{
			if (defined("NO_DATABASE"))
				$g_current_locale = default_locale_ietf();
			else
				$g_current_locale = logged_user()->profile()->locale();
			
			$g_current_locale = include_locale($g_current_locale);
		}

		$ietf = $g_current_locale;
	}
	else
	{
		if (!isset($g_locales[$ietf]))
			$ietf = include_locale($ietf);
	}
	
	assert(isset($g_locales[$ietf][$name]));
	return $g_locales[$ietf][$name];
}

function locale_pred($a, $b)
{
	return strcmp($a->name(), $b->name());
}

function available_locales()
{
	global $g_locale_files;
	
	$fs = glob(join_path(
		dirname(__FILE__), locale_directory, "*.inc.php"));

	foreach ($fs as $f)
		include_once($f);
	
	usort($g_locale_files, "locale_pred");
	return $g_locale_files;
}

?>