<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

class query
{
	public static function update_profile(
		$user_id, $name, $tz, $auto_subscribe, $locale)
	{
		$ps = db()->prepare(
			"update profiles set " .
			"name=:n, " .
			"tz=:tz, " .
			"auto_subscribe=:s, " .
			"locale=:lc " .
			"where user=:id");
		
		$ps->execute(array(
			":n" => $name,
			":tz" => $tz,
			":s" => ($auto_subscribe ? "1" : "0"),
			":lc" => $locale,
			":id" => $user_id));
	}
	
	public static function confirm_user($user_id)
	{
		$ps = db()->prepare(
			"update users set confirmed=1 where id=:id");
		$ps->execute(array(":id" => $user_id));
	}

	public static function confirm_all_users()
	{
		db()->execute("update users set confirmed=1");
	}
	
	public static function get_password($user_id)
	{
		$ps = db()->prepare(
			"select password from users where id=:id");
		$ps->execute(array(":id" => $user_id));
		
		return $ps;
	}
	
	public static function update_user(
		$user_id, $permissions, $banned, $automod, $last_login)
	{
		$ps = db()->prepare(
			"update users set " .
			"permissions=:p, banned=:b, automod=:am, last_login=:ll " .
			"where id=:id");
		
		$ps->execute(array(
			":p" => $permissions,
			":b" => ($banned ? "1" : "0"),
			":am" => ($automod ? "1" : "0"),
			":ll" => $last_login,
			":id" => $user_id));
	}
	
	public static function delete_recovery($recovery_id)
	{
		$ps = db()->prepare("delete from recovery where id=:id");
		$ps->execute(array(":id" => $recovery_id));
	}
	
	public static function delete_recovery_for_user($user_id)
	{
		$ps = db()->prepare("delete from recovery where user=:id");
		$ps->execute(array(":id" => $user_id));
	}
	
	public static function delete_recovery_before($date)
	{
		$ps = db()->prepare("delete from recovery where date_sent < :d");
		$ps->execute(array(":d" => $date));
	}
	
	public static function add_recovery($user_id, $uuid)
	{
		$ps = db()->prepare(
			"insert into recovery (uuid, user, date_sent) " .
			"values (:uuid, :u, :d)");
		
		$ps->execute(array(
			":uuid" => $uuid,
			":u" => $user_id,
			":d" => db()->now()));
	}
	
	public static function find_recovery($uuid)
	{
		$ps = db()->prepare("select * from recovery where uuid=:u");
		$ps->execute(array(":u" => $uuid));
		
		return $ps;
	}
	
	public static function set_password($user_id, $encrypted)
	{
		$ps = db()->prepare(
			"update users set password=:p where id=:id");
		$ps->execute(array(
			":p" => $encrypted,
			":id" => $user_id));
	}
	
	public static function get_all_users()
	{
		$ps = db()->prepare("select id, email from users");
		$ps->execute();
		
		return $ps;
	}
	
	public static function get_single_user($user_id)
	{
		$ps = db()->prepare(
			"select " .
				user::fields() . ", " .
				profile::fields() . " " .
			"from users " .
			"left join profiles on users.id=profiles.user " .
			"where users.id=:id");
		
		$ps->execute(array(":id" => $user_id));
		
		return $ps;
	}
	
	public static function find_user($email)
	{
		$ps = db()->prepare(
			"select id from users " .
			"where email=:e limit 1");
		$ps->execute(array(":e" => $email));
		
		return $ps;
	}
	
	private static function make_search_users_params(
		$search_string, $status)
	{
		$r = array("where" => array(), "params" => array());
		
		if ($search_string != "")
		{
			array_push($r["where"], "name like :s or email like :s");
			$r["params"][":s"] = "%" . $search_string . "%";
		}
		
		if ($status != search_status_none)
		{
			if ($status & search_status_banned)
				array_push($r["where"], "banned = 1");
		}
		
		return $r;
	}
	
	public static function search_users_count($search_string, $status)
	{
		$w = query::make_search_users_params($search_string, $status);
		
		$s =
			"select count(users.id) c from users " .
			"left join profiles on users.id=profiles.user ";
		
		if (count($w["where"]) != 0)
			$s .= " where " . implode(" and ", $w["where"]);
		
		$ps = db()->prepare($s);
		$ps->execute($w["params"]);
		
		return $ps;
	}
	
	public static function search_users(
		$search_string, $status, $from, $limit)
	{
		$w = query::make_search_users_params($search_string, $status);

		$s =
			"select " .
				user::fields() . ", " .
				profile::fields() . " " .
			"from users " .
			"left join profiles on users.id=profiles.user ";

		if (count($w["where"]) != 0)
			$s .= " where " . implode(" and ", $w["where"]);

		if ($limit != 0)
			$s .= " limit $from, $limit";
		
		$ps = db()->prepare($s);
		$ps->execute($w["params"]);
		
		return $ps;
	}
	
	public static function add_user(
		$email, $encrypted_password, $permissions, $confirmed, $automod)
	{
		$ps = db()->prepare(
			"insert into users " .
			"(email, password, permissions, confirmed, banned, " .
			"automod, reg_date, last_mark_date, mod_notified) " .
			"values " .
			"(:email, :password, :permissions, :confirmed, 0, " .
			":automod, :reg_date, :last_mark_date, 0)");
		
		$params = array(
			":email" => $email,
			":password" => $encrypted_password,
			":permissions" => $permissions,
			":confirmed" => ($confirmed ? "1" : "0"),
			":automod" => ($automod ? "1" : "0"),
			":reg_date" => db()->now(),
			":last_mark_date" => db()->never());
		
		$ps->execute($params);
	}
	
	public static function add_profile($user_id, $name)
	{
		$ps = db()->prepare(
			"insert into profiles " .
			"(user, name, tz, auto_subscribe, locale) " .
			"values (:id, :n, :tz, :as, :lc)");
		
		$ps->execute(array(
			":id" => $user_id,
			":n" => $name,
			":tz" => "UTC",
			":as" => "1",
			":lc" => default_locale_ietf()));
	}

	public static function delete_user($user_id)
	{
		$ps = db()->prepare("delete from users where id=:id");
		$ps->execute(array(":id" => $user_id));
	}
	
	public static function delete_profile_for_user($user_id)
	{
		$ps = db()->prepare("delete from profiles where user=:id");
		$ps->execute(array(":id" => $user_id));
	}
	
	public static function delete_unbanned_ips_before($date)
	{
		$ps = db()->prepare(
			"delete from ips where date_added < :d and banned=0");
		$ps->execute(array(":d" => $date));
	}
	
	public static function delete_all_ips()
	{
		db()->execute("delete from ips");
	}
	
	public static function get_single_banned_ip($ip_id)
	{
		$ps = db()->prepare("select * from ips where id=:i");
		$ps->execute(array(":i" => $ip_id));
		
		return $ps;
	}
	
	public static function get_all_banned_ips_count()
	{
		$ps = db()->prepare("select count(id) c from ips where banned=1");
		$ps->execute();
		
		return $ps;
	}
	
	public static function get_all_banned_ips($from, $limit)
	{
		$s = "select * from ips where banned=1";
		if ($limit != 0)
			$s .= " limit $from, $limit";
		
		$ps = db()->prepare($s);
		$ps->execute();
		
		return $ps;
	}
	
	public static function find_ip($ip_address)
	{
		$ps = db()->prepare("select * from ips where ip=:i");
		$ps->execute(array(":i" => $ip_address));
		
		return $ps;
	}
	
	public static function add_ip($ip_address)
	{
		$ps = db()->prepare(
			"insert into ips (ip, date_added, banned) " .
			"values (:i, :d, :b)");
		$ps->execute(array(
			":i" => $ip_address,
			":d" => db()->now(),
			":b" => "0"));
	}
	
	public static function update_ip($ip_id, $banned)
	{
		$ps = db()->prepare("update ips set banned=:b where id=:i");
		$ps->execute(array(
			":b" => ($banned ? "1" : "0"),
			":i" => $ip_id));
	}
	
	public static function add_ip_post($ip_id, $post_id)
	{
		$ps = db()->prepare(
			"insert into ip_posts (ip_id, post_id) values (:i, :p)");
		$ps->execute(array(":i" => $ip_id, ":p" => $post_id));
	}
	
	public static function get_unconfirmed_user_ids_before($old)
	{
		$ps = db()->prepare(
			"select id from users where confirmed=0 and reg_date<:d");
		$ps->execute(array(":d" => $old));
		
		return $ps;
	}
	
	public static function delete_registration_for_user($user_id)
	{
		$ps = db()->prepare("delete from registration where user=:id");
		$ps->execute(array(":id" => $user_id));
	}
	
	public static function delete_registration_before($date)
	{
		$ps = db()->prepare("delete from registration where date_sent < :d");
		$ps->execute(array(":d" => $date));
	}
	
	public static function add_registration($user_id, $uuid)
	{
		$ps = db()->prepare(
			"insert into registration (uuid, user, date_sent) " .
			"values (:uuid, :u, :d)");
		
		$ps->execute(array(
			":uuid" => $uuid,
			":u" => $user_id,
			":d" => db()->now()));
	}
	
	public static function find_registration($uuid)
	{
		$ps = db()->prepare("select * from registration where uuid=:u");
		$ps->execute(array(":u" => $uuid));
		
		return $ps;
	}
	
	public static function get_all_options()
	{
		$ps = db()->prepare("select * from options");
		$ps->execute();
		
		return $ps;
	}
	
	public static function find_option($name)
	{
		$ps = db()->prepare("select id from options where name=:n");
		$ps->execute(array(":n" => $name));
		
		return $ps;
	}
	
	public static function add_option($name, $value)
	{
		$ps = db()->prepare(
			"insert into options (name, value) values (:n, :v)");
		$ps->execute(array(":n" => $name, ":v" => $value));
	}
	
	public static function set_options($name_value_map)
	{
		$ps = db()->prepare(
			"update options set value=:v where name=:n");
		
		foreach ($name_value_map as $n=>$v)
			$ps->execute(array(":n" => $n, ":v" => $v));
	}

	public static function thread_count($forum_id)
	{
		$ps = db()->prepare(
			"select count(id) c from threads where forum=:f");
		$ps->execute(array(":f" => $forum_id));
		
		return $ps;
	}
	
	public static function forum_post_count($forum_id)
	{
		$ps = db()->prepare(
			"select count(id) c from posts " .
			"where thread in(" .
			"	select id thread from threads where forum=:f)");
		$ps->execute(array(":f" => $forum_id));
		
		return $ps;
	}
	
	public static function user_post_count($user_id)
	{
		$ps = db()->prepare(
			"select count(id) c from posts where user=:id");
		$ps->execute(array(":id" => $user_id));
		
		return $ps;
	}
	
	public static function get_marks($user_id, $thread_ids)
	{
		$ph = str_repeat("?,", count($thread_ids) - 1) . "?";		
		
		$ps = db()->prepare(
			"select * from marks " .
			"where user=? and thread in ($ph)");
		
		$ps->bind($user_id);
		foreach ($thread_ids as $id)
			$ps->bind($id);
		
		$ps->execute();
		
		return $ps;
	}
	
	public static function add_marks($user_id, $mark_time, $thread_ids)
	{
		$ps = db()->prepare(
			"insert into marks (user, thread, mark_time) " .
			"values (:u, :t, :mt)");
					
		foreach ($thread_ids as $tid)
		{
			$ps->execute(array(
				":u" => $user_id,
				":t" => $tid,
				":mt" => $mark_time));
		}
	}
	
	public static function set_mark($user_id, $mark_time, $thread_id)
	{
		$ps = db()->prepare(
			"update marks set mark_time=:mt " .
			"where user=:u and thread=:t");
		
		$ps->execute(array(
			":mt" => $mark_time,
			":u" => $user_id,
			":t" => $thread_id));
	}
	
	public static function delete_marks_before($user_id, $date)
	{
		$ps = db()->prepare(
			"delete from marks where user=:u and mark_time<=:d");
		$ps->execute(array(
			":u" => $user_id,
			":d" => $date));
	}
	
	private static function get_forum_impl($id)
	{
		$q =
			"select " .
				forum::fields() . ", " .
				post::fields() . ", " .
				user::fields() . ", " .
				profile::fields() . ", " .
				ip_user::fields() . " " .
			"from forums ".
			"left join ( ".
				"select p1.* " .
				"from posts p1 " .
				"left join posts as p2 on " .
					"p1.forum = p2.forum and p1.post_date < p2.post_date " .
				"where p1.queued = 0 and p2.id is null " .
			") as posts on forums.id=posts.forum " .
			"left join users on users.id=posts.user " .
			"left join profiles on profiles.user=posts.user " .
			"left join ip_posts on ip_posts.post_id=posts.id " .
			"left join ips on ips.id = ip_posts.ip_id";
		
		if ($id !== 0)
			$q .= " where forums.id=:id";
		
		$ps = db()->prepare($q);
		
		$params = array();
		if ($id !== 0)
			$params[":id"] = $id;
		
		$ps->execute($params);
		
		return $ps;
	}
	
	public static function get_single_forum($forum_id)
	{
		return query::get_forum_impl($forum_id);
	}
	
	public static function get_all_forums()
	{
		return query::get_forum_impl(0);
	}

	public static function add_forum($name)
	{
		$ps = db()->prepare("insert into forums (name) values (:n)");
		$ps->execute(array(":n" => $name));
	}

	public static function delete_forum($forum_id)
	{
		$ps = db()->prepare("delete from forums where id=:id");
		$ps->execute(array(":id" => $forum_id));

		$ps = db()->prepare(
			"delete from posts where thread in(" .
			"	select id thread from threads where forum=:f)");
		$ps->execute(array(":f" => $forum_id));
	
		$ps = db()->prepare("delete from threads where forum=:f");
		$ps->execute(array(":f" => $forum_id));
	}
	
	public static function thread_first_post($thread_id)
	{
		$ps = db()->prepare(
			"select " . post::fields() . " from posts " .
			"where thread=:tid and parent=0 and queued=0 " .
			"order by post_date limit 1");
		
		$ps->execute(array(":tid" => $thread_id));
		
		return $ps;
	}	
	
	public static function thread_last_post($thread_id)
	{
		$ps = db()->prepare(
			"select id, post_date from posts " .
			"where thread=:tid and queued=0 " .
			"order by post_date desc " .
			"limit 1");
		$ps->execute(array(":tid" => $thread_id));
		
		return $ps;
	}
	
	public static function user_last_post($user_id)
	{
		$ps = db()->prepare(
			"select " . post::fields() . " from posts where user=:id " .
			"order by post_date desc limit 1");
		$ps->execute(array(":id" => $user_id));
		
		return $ps;
	}
	
	public static function set_thread_last_post(
		$thread_id, $post_id, $post_date)
	{
		$ps = db()->prepare(
			"update threads set last_post_date=:pd, last_post_id=:pid " .
			"where id=:tid");

		$ps->execute(array(
			":pd" => $post_date,
			":pid" => $post_id,
			":tid" => $thread_id));
	}
	
	private static function get_threads_impl(
		$forum_id, $thread_id, $from=0, $limit=0)
	{
		$q =
			"select " .
				thread::fields() . ", " .
				"greatest(count(p2.id) - 1, 0) reply_count, " .
				post::fields() . ", " .
				user::fields() . ", " .
				profile::fields() . ", " .
				ip_user::fields() . " " .
			"from threads " .
			"left join posts on posts.id=threads.last_post_id " .
			"left join posts p2 on p2.thread=threads.id " .
			"left join users on users.id = posts.user " .
			"left join profiles on profiles.user = posts.user " .
			"left join ip_posts on ip_posts.post_id = posts.id " .
			"left join ips on ips.id = ip_posts.ip_id " .
			"where ";
		
		$id = 0;
		if ($forum_id === 0)
		{
			$id = $thread_id;
			$q .= "threads.id=:id ";
		}
		else
		{
			$id = $forum_id;
			$q .= "threads.forum=:id and posts.queued=0 ";
		}
		
		$q .=
			"group by threads.id " .
			"order by last_post_date desc";
		
		if ($limit !== 0)
			$q .= " limit $from,$limit";
			
		$ps = db()->prepare($q);
		$ps->execute(array(":id" => $id));
		
		return $ps;
	}
	
	public static function get_all_threads($forum_id, $from, $limit)
	{
		return query::get_threads_impl($forum_id, 0, $from, $limit);
	}
	
	public static function get_single_thread($thread_id)
	{
		return query::get_threads_impl(0, $thread_id);
	}
	
	public static function set_thread_changed($thread_id, $date)
	{
		$ps = db()->prepare(
			"update threads set changed_date=:d where id=:id");
		$ps->execute(array(":d" => $date, ":id" => $thread_id));
	}
	
	public static function update_thread($thread_id, $title)
	{
		$ps = db()->prepare("update threads set title=:t where id=:id");
		$ps->execute(array(":t" => $title, ":id" => $thread_id));
	}
	
	public static function add_thread($forum_id, $title, $user_id)
	{
		$ps = db()->prepare(
			"insert into threads " .
			"(forum, title, poster, creation_date, " .
			"last_post_date, last_post_id, changed_date) " .
			"values (:fid, :t, :p, :cd, :lpd, :lpi, :changed)");
		
		$ps->execute(array(
			":fid" => $forum_id,
			":t" => $title,
			":p" => $user_id,
			":cd" => db()->now(),
			":lpd" => db()->never(),
			":lpi" => 0,
			":changed" => db()->never()));
	}
	
	public static function delete_thread($thread_id)
	{
		$ps = db()->prepare("delete from threads where id=:tid");
		$ps->execute(array(":tid" => $thread_id));
		
		$ps = db()->prepare("delete from posts where thread=:tid");
		$ps->execute(array(":tid" => $thread_id));
	}
	
	public static function set_last_mark_date($user_id, $date)
	{
		$ps = db()->prepare(
			"update users set last_mark_date=:d where id=:id");
		$ps->execute(array(
			":d" => $date,
			":id" => $user_id));
	}
	
	public static function get_subscription($user_id, $thread_id)
	{
		$ps = db()->prepare(
			"select id from subscriptions where user=:u and thread=:t");
		$ps->execute(array(":u" => $user_id, ":t" => $thread_id));
		
		return $ps;
	}
	
	public static function add_subscription($user_id, $thread_id)
	{
		$ps = db()->prepare(
			"insert into subscriptions (user, thread, notified) " .
			"values (:u, :t, 0)");
		$ps->execute(array(":u" => $user_id, ":t" => $thread_id));
	}
	
	public static function delete_subscription($user_id, $thread_id)
	{
		$ps = db()->prepare(
			"delete from subscriptions where user=:u and thread=:t");
		$ps->execute(array(":u" => $user_id, ":t" => $thread_id));
	}
	
	public static function erase_user_pending($user_id)
	{
		$ps = db()->prepare(
			"delete from posts where user=:uid and queued=1");
		$ps->execute(array(":uid" => $user_id));
	}
	
	public static function unnotify_subscription($user_id, $thread_id)
	{
		$ps = db()->prepare(
			"update subscriptions set notified=0 " .
			"where thread=:t and user=:u");
		$ps->execute(array(":t" => $thread_id, ":u" => $user_id));
	}
	
	public static function get_subscriptions_for_thread($thread_id)
	{
		$ps = db()->prepare(
			"select " .
				user::fields() . ", " .
				profile::fields() . " ".
			"from subscriptions " .
			"join users on users.id=subscriptions.user " .
			"join profiles on profiles.user = subscriptions.user " .
			"where thread=:t and notified=0 and users.confirmed=1");
		$ps->execute(array(":t" => $thread_id));
		
		return $ps;
	}
	
	public static function notify_subscriptions(
		$thread_id, $except_user_id)
	{
		$ps = db()->prepare(
			"update subscriptions set notified=1 " .
			"where thread=:t and user != :u");
		$ps->execute(array(":t" => $thread_id, ":u" => $except_user_id));
	}

	public static function get_posts($thread_id, $from=0, $limit=0)
	{
		$s =
			"select " .
				post::fields() . ", " .
				user::fields() . ", " .
				profile::fields() . " " .
			"from posts " .
			"left join users on users.id = posts.user " .
			"left join profiles on profiles.user = posts.user " .
			"where thread=:tid and queued=0";
		
		if ($limit !== 0)
			$s .= " limit $from,$limit";
			
		$ps = db()->prepare($s);
		$ps->execute(array(":tid" => $thread_id));
		
		return $ps;
	}
	
	public static function get_post_count($thread_id)
	{
		$s =
			"select count(id) c from posts " .
			"where thread=:tid and queued=0";
			
		$ps = db()->prepare($s);
		$ps->execute(array(":tid" => $thread_id));
		
		return $ps;
	}
	
	public static function get_single_post($post_id)
	{
		$ps = db()->prepare(
			"select " .
				post::fields() . ", " .
				user::fields() . ", " .
				profile::fields() . ", " .
				ip_user::fields() . " " .
			"from posts " .
			"left join users on users.id = posts.user " .
			"left join profiles on profiles.user = posts.user " .
			"left join ip_posts on ip_posts.post_id = posts.id " .
			"left join ips on ips.id = ip_posts.ip_id " .
			"where posts.id=:id");
		
		$ps->execute(array(":id" => $post_id));
		
		return $ps;
	}
	
	public static function add_post(
		$forum_id, $thread_id, $parent_id, $user_id, $message,
		$author_name, $queued)
	{
		$ps = db()->prepare(
			"insert into posts " .
			"(forum, thread, parent, content, user," .
			" post_date, author, queued, mod_comment) " .
			"values (:fid, :tid, :pid, :m, :u, :pd, :a, :q, '')");
		
		$ps->execute(array(
			":fid" => $forum_id,
			":tid" => $thread_id,
			":pid" => $parent_id,
			":u" => $user_id,
			":m" => $message,
			":pd" => db()->now(),
			":a" => $author_name,
			":q" => ($queued ? "1" : "0")));
	}
	
	public static function delete_post($post_id)
	{
		$ps = db()->prepare("delete from posts where id=:pid");
		$ps->execute(array(":pid" => $post_id));
	}
	
	public static function reparent_posts($old_parent, $new_parent)
	{
		$ps = db()->prepare(
			"update posts set parent=:newpid where parent=:oldpid");
		$ps->execute(array(
			":newpid" => $new_parent, ":oldpid" => $old_parent));
	}
	
	public static function queued_posts_count()
	{
		$ps = db()->prepare("select count(id) c from posts where queued=1");
		$ps->execute();
		
		return $ps;
	}
	
	public static function queued_posts()
	{
		$ps = db()->prepare(
			"select " .
				post::fields() . ", " .
				user::fields() . ", " .
				profile::fields() . ", " .
				ip_user::fields() . " " .
			"from posts " .
			"left join users on users.id = posts.user " .
			"left join profiles on profiles.user = posts.user " .
			"left join ip_posts on ip_posts.post_id = posts.id " .
			"left join ips on ips.id = ip_posts.ip_id " .
			"where queued=1");
		$ps->execute();
		
		return $ps;
	}
	
	public static function delete_queued_for_ip($ip_address)
	{
		$ps = db()->prepare(
			"delete from posts " .
			"where id in (" .
			"	select post_id id from ip_posts " .
			"	join ips on ips.id = ip_posts.ip_id " .
			"	where ip=:i) " .
			"and queued=1");
		$ps->execute(array(":i" => $ip_address));
	}
	
	public static function get_queued_posts_for_user_by_thread($user_id)
	{
		$ps = db()->prepare(
			"select id, thread, user from posts " .
			"where user=:id and queued=1 " .
			"group by thread");
		
		$ps->execute(array(":id" => $user_id));
		
		return $ps;
	}
	
	public static function get_queued_posts_by_thread()
	{
		$ps = db()->prepare(
			"select id, thread, user from posts " .
			"where queued=1 " .
			"group by thread");
		$ps->execute();
		
		return $ps;
	}
	
	public static function accept_single_post($post_id, $mod_comment)
	{
		$ps = db()->prepare(
			"update posts set queued=0, mod_comment=:m " .
			"where id=:id");
		$ps->execute(array(":m" => $mod_comment, ":id" => $post_id));
	}
	
	public static function accept_posts_for_user($user_id)
	{
		$ps = db()->prepare(
			"update posts set queued=0 where user=:id");
		$ps->execute(array(":id" => $user_id));
	}
	
	public static function accept_all_posts()
	{
		db()->execute("update posts set queued=0");
	}
	
	public static function update_post($post_id, $content, $mod_comment)
	{
		$ps = db()->prepare(
			"update posts set content=:c, mod_comment=:cm " .
			"where id=:id");
		$ps->execute(array(
			":c" => $content,
			":cm" => $mod_comment,
			":id" => $post_id));
	}
	
	public static function find_ip_post($post_id)
	{
		$ps = db()->prepare(
			"select * from ip_posts " .
			"join ips on ips.id=ip_posts.ip_id " .
			"where post_id=:id");
		
		$ps->execute(array(":id" => $post_id));
		
		return $ps;
	}
	
	public static function get_last_posts_impl(
		$forum_id, $thread_id, $count)
	{
		$s =
			"select " .
				thread::fields() . ", " .
				post::fields() . ", " .
				user::fields() . ", " .
				profile::fields() . ", " .
				ip_user::fields() . " " .
			"from posts " .
			"left join threads on threads.id = posts.thread " .
			"left join users on users.id=posts.user " .
			"left join profiles on profiles.user=posts.user " .
			"left join ip_posts on ip_posts.post_id=posts.id " .
			"left join ips on ips.id = ip_posts.ip_id " .
			"where ";
		
		$id = 0;
		if ($forum_id === 0)
		{
			$id = $thread_id;
			$s .= "posts.thread = :id";
		}
		else
		{
			$id = $forum_id;
			$s .= "posts.forum = :id";
		}
		
		$s .=
			" and queued=0 " .
			"order by posts.post_date desc " .
			"limit $count";
		
		$ps = db()->prepare($s);
		$ps->execute(array(":id" => $id));
		
		return $ps;
	}
	
	public static function get_last_forum_posts($forum_id, $count)
	{
		return query::get_last_posts_impl($forum_id, 0, $count);
	}
	
	public static function get_last_thread_posts($thread_id, $count)
	{
		return query::get_last_posts_impl(0, $thread_id, $count);
	}
	
	public static function notify_moderators()
	{
		$ps = db()->prepare(
			"update users " .
			"set mod_notified=1 " .
			"where (users.permissions & :p) != 0");
		$ps->execute(array(":p" => user_permission_mod));
	}
	
	public static function unnotify_moderator($user_id)
	{
		$ps = db()->prepare(
			"update users " .
			"set mod_notified=0 " .
			"where id=:id");
		$ps->execute(array(":id" => $user_id));
	}
	
	public static function get_unnotified_moderators()
	{
		$ps = db()->prepare(
			"select " .
				user::fields() . ", " .
				profile::fields() . " " .
			"from users " .
			"join profiles on profiles.user = users.id " .
			"where (users.permissions & :p) != 0 and mod_notified=0");
		
		$ps->execute(array(":p" => user_permission_mod));
		
		return $ps;
	}
	
	
	public static function get_extension($id)
	{
		$ps = db()->prepare(
			"select " .
				extension::fields() . ", " .
				extension_version::fields() . " " .
			"from extensions " .
			"join extension_versions on " .
				"extension_versions.extension=extensions.id " .
			"where extensions.id=:id");
		
		$ps->execute(array(":id" => (int)$id));
		
		return $ps;
	}

	public static function get_extensions()
	{
		$ps = db()->prepare(
			"select " .
				extension::fields() . ", " .
				extension_version::fields() . " " .
			"from extensions " .
			"join extension_versions on " .
				"extension_versions.extension=extensions.id ");
		
		$ps->execute();
		
		return $ps;
	}
	
	public static function find_extension_by_guid($guid)
	{
		$ps = db()->prepare(
			"select " .
				extension::fields() . ", " .
				extension_version::fields() . " " .
			"from extensions " .
			"join extension_versions on " .
				"extension_versions.extension=extensions.id " .
			"where guid=:g " .
			"limit 1");
		
		$ps->execute(array(":g" => $guid));
		
		return $ps;
	}
	
	public static function find_extension_by_name($name)
	{
		$ps = db()->prepare(
			"select " .
				extension::fields() . ", " .
				extension_version::fields() . " " .
			"from extensions " .
			"join extension_versions on " .
				"extension_versions.extension=extensions.id " .
			"where name=:n " .
			"limit 1");
		
		$ps->execute(array(":n" => $name));
		
		return $ps;
	}

	public static function add_extension($guid, $name)
	{
		$ps = db()->prepare(
			"insert into extensions " .
			"(guid, name, author, description, email, website) " .
			"values (:g, :n, :a, :d, :e, :w)");
		
		$ps->execute(array(
			":g" => $guid,
			":n" => $name,
			":a" => "",
			":d" => "",
			":e" => "",
			":w" => ""));
		
		return db()->last_insert_id();
	}
	
	public static function delete_extension($id)
	{
		db()->begin();
		
		$ps = db()->prepare(
			"delete from extensions where id=:id");
		$ps->execute(array(":id" => $id));
		
		$ps = db()->prepare(
			"delete from extension_versions where extension=:id");
		$ps->execute(array(":id" => $id));
		
		$ps = db()->prepare(
			"delete from extension_icons where extension=:id");
		$ps->execute(array(":id" => $id));
		
		db()->commit();
	}
	
	public static function find_extension_version($id, $v)
	{
		$ps = db()->prepare(
			"select " . extension_version::fields() . " " .
			"from extension_versions " .
			"where extension=:e and version=:v " .
			"limit 1");
		
		$ps->execute(array(":e" => $id, ":v" => $v));
		
		return $ps;
	}
	
	public static function add_extension_version(
		$id, $version, $file, $requires)
	{
		$ps = db()->prepare(
			"insert into extension_versions " .
			"(extension, version, file, requires, release_date) " .
			"values (:e, :v, :f, :r, :rd)");
		
		$ps->execute(array(
			":e" => $id,
			":v" => $version,
			":f" => $file,
			":r" => $requires,
			":rd" => db()->now()));
		
		return db()->last_insert_id();
	}
	
	public static function add_extension_icons(
		$id, $small_icon, $medium_icon, $large_icon)
	{
		$ps = db()->prepare(
			"insert into extension_icons " .
			"(extension, small_icon, medium_icon, icon) " .
			"values (:e, :si, :mi, :li)");
		
		$ps->execute(array(
			":e" => $id,
			":si" => $small_icon,
			":mi" => $medium_icon,
			":li" => $large_icon));
	}
	
	public static function extension_has_medium_icon($id)
	{
		$ps = db()->prepare(
			"select count(medium_icon) c from extension_icons " .
			"where extension=:id");
		
		$ps->execute(array(":id" => $id));
		
		$row = $ps->fetch();
		if ($row === FALSE)
			return false;
		
		$c = (int)$row["c"];
		
		return ($c > 0);
	}
	
	public static function extension_medium_icon($id)
	{
		$ps = db()->prepare(
			"select medium_icon from extension_icons " .
			"where extension=:id");
		
		$ps->execute(array(":id" => $id));
		
		return $ps;
	}
	
	public static function extension_permissions($user_id, $ext_id)
	{
		$ps = db()->prepare(
			"select permissions from extension_permissions " .
			"where user=:u and extension=:e");
		$ps->execute(array(":u" => $user_id, ":e" => $ext_id));
		
		return $ps;
	}
}
?>