<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

require_once(__DIR__ . "/locale.php");
require_once(__DIR__ . "/queries.inc.php");
require_once(__DIR__ . "/db.php");
require_once(__DIR__ . "/wiki.php");

define("user_permission_none" , 0);
define("user_permission_post", 1);
define("user_permission_edit_own", 2);
define("user_permission_edit_all", 4);
define("user_permission_erase", 8);
define("user_permission_cp", 16);
define("user_permission_ban", 32);
define("user_permission_mod", 64);
define("user_permission_ext", 128);

define("user_permission_all", 
	user_permission_post | user_permission_edit_own |
	user_permission_edit_all | user_permission_erase |
	user_permission_cp | user_permission_ban | user_permission_mod |
	user_permission_ext);
	
define("user_permission_anonymous", user_permission_post);
define("user_permission_registered",
	user_permission_post | user_permission_edit_own);
define("user_permission_administrator", user_permission_all);

define("search_status_none", 0);
define("search_status_banned", 1);

// 30 minutes
define("recovery_maximum_time", 60 * 30);

// 24 hours
define("registration_maximum_time", 60 * 60 * 24);

// 2 days
define("ips_maximum_time", 60 * 60 * 24 * 2);


class user_not_found extends \Exception {}
class user_already_exists extends \Exception {}
class user_banned extends \Exception {}
class unconfirmed_user extends \Exception {}
class bad_registration extends \Exception {}
class bad_password extends \Exception {}

class ip_user
{
	private $id = 0;
	private $ip = "";
	private $date_added = 0;
	private $banned = false;
	
	static public function fields($table = "ips", $prefix = "ip_")
	{
		return
			"$table.id {$prefix}id, " .
			"$table.ip {$prefix}ip, " .
			"$table.date_added {$prefix}date_added, " .
			"$table.banned {$prefix}banned";
	}
	
	static public function from_db($row, $prefix = "ip_")
	{
		if (!isset($row["{$prefix}id"]))
			return null;
		
		$u = new ip_user();
		
		$u->id = (int)$row["{$prefix}id"];
		$u->ip = $row["{$prefix}ip"];
		$u->date_added = db()->timestamp($row["{$prefix}date_added"]);
		$u->banned = ($row["{$prefix}banned"] === "0" ? false : true);
		
		return $u;
	}
	
	public function id()
	{
		return $this->id;
	}
	
	public function address()
	{
		return $this->ip;
	}
	
	public function date_added()
	{
		return $this->date_added;
	}
}

class profile
{
	private $user = null;
	private $name = "";
	private $tz = "";
	private $auto_subscribe = false;
	private $locale = "";

	public static function fields(
		$table = "profiles", $prefix = "profile_")
	{
		return
			"$table.name {$prefix}name, " .
			"$table.tz {$prefix}tz, " .
			"$table.auto_subscribe {$prefix}auto_subscribe, " .
			"$table.locale {$prefix}locale ";
	}
	
	public static function from_db($user, $row, $prefix = "profile_")
	{
		$p = new profile($user);
		
		$p->name = $row["{$prefix}name"];
		$p->tz = $row["{$prefix}tz"];
		$p->auto_subscribe = 
			($row["{$prefix}auto_subscribe"] == "0" ? false : true);
		$p->locale = $row["{$prefix}locale"];
		
		if ($p->name == "")
			$p->name = null;
		
		if ($p->locale == "")
			$p->locale = default_locale_ietf();
		
		return $p;
	}
	
	public function __construct($user, $name = null)
	{
		$this->user = $user;
		$this->name = $name;
		
		$lc = default_locale_ietf();
		if (isset($_SESSION["locale"]))
		{
			$lc = $_SESSION["locale"];
			if (!valid_locale($lc))
				$lc = default_locale_ietf();
		}
		
		$this->locale = $lc;
	}
	
	public function display_name()
	{
		if ($this->name === null)
		{
			if ($this->user->is_anonymous())
				return loc("user_anonymous");
			else
				return loc("user_unnamed");
		}
		else
		{
			return make_safe_string($this->name);
		}
	}
	
	public function name()
	{
		return make_safe_string($this->name);
	}

	public function set_name($n)
	{
		$this->name = $n;
	}
	
	public function timezone()
	{
		return $this->tz;
	}
	
	public function set_timezone($tz)
	{
		if (valid_timezone($tz))
			$this->tz = $tz;
	}
	
	public function auto_subscribe()
	{
		return $this->auto_subscribe;
	}
	
	public function set_auto_subscribe($v)
	{
		$this->auto_subscribe = ($v ? true : false);
	}
	
	public function locale()
	{
		return $this->locale;
	}
	
	public function set_locale($n)
	{
		if (!valid_locale($n))
			$n = default_locale_ietf();
		
		$this->locale = $n;
	}
	
	public function save()
	{
		query::update_profile(
			$this->user->id(), $this->name, $this->tz,
			$this->auto_subscribe, $this->locale);
	}
}

class user
{
	private $id = 0;
	private $email = "";
	private $name = "";
	private $auto_subscribe = false;
	private $permissions = 0;
	private $last_login = 0;
	private $reg_date = 0;
	private $confirmed = false;
	private $banned = false;
	private $automod = false;
	private $last_mark_date = 0;
	private $profile = null;
	private $ip = null;
	
	public static function fields($table = "users", $prefix = "user_")
	{
		return
			"$table.id {$prefix}id, " .
			"$table.email {$prefix}email, " .
			"$table.permissions {$prefix}permissions, " .
			"$table.last_login {$prefix}last_login, " .
			"$table.reg_date {$prefix}reg_date, " .
			"$table.confirmed {$prefix}confirmed, " .
			"$table.banned {$prefix}banned, " .
			"$table.automod {$prefix}automod, " .
			"$table.last_mark_date {$prefix}last_mark_date ";
	}
	
	public static function from_db($row, $prefix = "user_")
	{
		if (!isset($row["{$prefix}id"]))
			return null;
		
		$u = new user();
		
		$u->id = (int)$row["{$prefix}id"];
		$u->email = $row["{$prefix}email"];
		$u->permissions = (int)$row["{$prefix}permissions"];
		$u->last_login = db()->timestamp($row["{$prefix}last_login"]);
		$u->reg_date = db()->timestamp($row["{$prefix}reg_date"]);
		$u->confirmed = ($row["{$prefix}confirmed"] == "0" ? false : true);
		$u->banned = ($row["{$prefix}banned"] == "0" ? false : true);
		$u->automod = ($row["{$prefix}automod"] == "0" ? false : true);
		$u->last_mark_date = db()->timestamp($row["{$prefix}last_mark_date"]);
		$u->profile = profile::from_db($u, $row);
		$u->ip = ip_user::from_db($row);
		
		return $u;
	}
	
	public static function anonymous($name = null)
	{
		$u = new user();
	
		if ($name == "")
			$name = null;
		
		$u->profile = new profile($u, $name);
		$u->permissions = user_permission_anonymous;
		$u->confirmed = true;
		
		return $u;
	}
	
	public function id()
	{
		return $this->id;
	}
	
	public function me()
	{
		return ($this->id() == logged_user()->id());
	}
	
	public function confirmed()
	{
		return $this->confirmed;
	}
	
	public function confirm()
	{
		query::confirm_user($this->id);
		$this->confirmed = true;
	}
	
	public function last_mark_date()
	{
		return $this->last_mark_date;
	}
	
	public function set_last_mark_date($ts = -1)
	{
		if ($ts === -1)
			$ts = time();
		
		db()->begin();
		
		query::set_last_mark_date($this->id, db()->date_time($ts));
		query::delete_marks_before($this->id, db()->date_time($ts));
		
		db()->commit();
	}
	
	public function check_password($p)
	{
		$ps = query::get_password($this->id);
		
		$row = $ps->fetch();
		if ($row === FALSE)
			throw new user_not_found();
		
		if (password_verify($p, $row["password"]))
			return true;
		else
			return false;
	}
	
	public function email()
	{
		return make_safe_string($this->email);
	}

	public function reg_date()
	{
		return $this->reg_date;
	}
	
	public function profile()
	{
		return $this->profile;
	}
	
	public function subscribed($thread)
	{
		$ps = query::get_subscription($this->id, $thread->id());
		
		$row = $ps->fetch();
		if ($row === FALSE)
			return false;
		
		return true;
	}

	public function subscribe($thread)
	{
		if (!$this->can_subscribe())
			return;
		
		if ($this->subscribed($thread))
			return;

		query::add_subscription($this->id, $thread->id());
	}

	public function unsubscribe($thread)
	{
		if (!$this->can_subscribe())
			return;

		query::delete_subscription($this->id, $thread->id());
	}
	
	public function permissions()
	{
		return $this->permissions;
	}
	
	public function set_permissions($p)
	{
		$p = (int)$p;
		
		if (!valid_permissions($p))
			return;
		
		$this->permissions = $p;
		$this->save();
	}
	
	public function post_count()
	{
		$ps = query::user_post_count($this->id);
		
		$row = $ps->fetch();
		if ($row === FALSE)
			return 0;
		
		return (int)$row["c"];
	}
	
	public function last_login()
	{
		return $this->last_login;
	}
	
	public function last_post()
	{
		$ps = query::user_last_post($this->id);
		
		$row = $ps->fetch();
		if ($row === FALSE)
			return null;
		
		return post::from_db($row);
	}
	
	public function accept_pending()
	{
		accept_impl(accept_for_user_flag, $this->id);
	}
	
	public function erase_pending()
	{
		query::erase_user_pending($this->id);
	}

	function ban($v = true)
	{
		if ($this->is_anonymous() || logged_user()->id() == $this->id)
			return;
		
		$this->banned = $v;
		$this->save();
	}

	function unban()
	{
		$this->ban(false);
	}
	
	public function banned()
	{
		return $this->banned;
	}
	
	public function deletable()
	{
		if ($this->is_anonymous() || $this->id == 1)
			return false;
		
		if ($this->id == logged_user()->id())
			return false;
		
		return true;
	}
	
	public function is_anonymous()
	{
		return ($this->id == 0);
	}
	
	public function automod()
	{
		return $this->automod;
	}
	
	public function set_automod($b)
	{
		$this->automod = ($b ? true : false);
		$this->save();
	}
	
	public function set_last_login($t)
	{
		$this->last_login = (int)$t;
		$this->save();
	}
		
	private function save()
	{
		query::update_user(
			$this->id, $this->permissions, $this->banned,
			$this->automod, db()->date_time($this->last_login));
	}

	
	public function can_post()
	{
		return ($this->permissions & user_permission_post) != 0;
	}
	
	public function can_edit_own()
	{
		return ($this->permissions & user_permission_edit_own) != 0;
	}
	
	public function can_edit_all()
	{
		return ($this->permissions & user_permission_edit_all) != 0;
	}
	
	public function can_erase()
	{
		return ($this->permissions & user_permission_erase) != 0;
	}
	
	public function can_cp()
	{
		return ($this->permissions & user_permission_cp) != 0;
	}
	
	public function can_see_ips()
	{
		return $this->can_cp();
	}
	
	public function can_ban()
	{
		return ($this->permissions & user_permission_ban) != 0;
	}
	
	public function can_moderate()
	{
		return ($this->permissions & user_permission_mod) != 0;
	}
	
	public function can_subscribe()
	{
		return !$this->is_anonymous();
	}
	
	public function can_add_ext()
	{
		return !$this->is_anonymous();
	}
	
	public function can_delete_ext()
	{
		return ($this->permissions & user_permission_ext);
	}
	
	public function can_edit_ext($e)
	{
		if ($this->permissions & user_permission_ext)
			return true;
		
		$p = extension_permissions($this->id, $e->id());
		return ($p & ext_permission_edit);
	}
}

class recovery
{
	private $id = 0;
	private $uuid = "";
	private $user_id = 0;
	private $time = 0;
	
	public static function from_db($row)
	{
		$r = new recovery();
		
		$r->id = (int)$row["id"];
		$r->uuid = $row["uuid"];
		$r->user_id = (int)$row["user"];
		$r->time = db()->timestamp($row["date_sent"]);
		
		return $r;
	}
	
	public function uuid()
	{
		return $this->uuid;
	}
	
	public function user_id()
	{
		return $this->user_id;
	}
	
	public function time()
	{
		return $this->time;
	}
	
	public function erase()
	{
		query::delete_recovery($this->id);
	}
	
	public function reset($cleartext)
	{
		query::set_password(
			$this->user_id, make_user_password($cleartext));
		
		$this->erase();
	}
}

function get_user($id)
{
	$ps = query::get_single_user($id);

	$row = $ps->fetch();
	if ($row === FALSE)
		throw new user_not_found();
	
	return user::from_db($row);
}

function find_user($email)
{
	$ps = query::find_user($email);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		return null;
	
	return get_user((int)$row["id"]);
}

function search_users($search, $status, $from, $limit)
{
	$r = array("count" => 0, "users" => array());

	// count
	$ps = query::search_users_count($search, $status);
	$row = $ps->fetch();
	if ($row === FALSE)
		return $r;
		
	$r["count"] = (int)$row["c"];
	if ($r["count"] <= 0)
		return $r;
	
	// rows
	$ps = query::search_users($search, $status, $from, $limit);
	foreach ($ps as $row)
		array_push($r["users"], user::from_db($row));
	
	return $r;
}

function cleanup_ips()
{
	$old = db()->date_time(time() - ips_maximum_time);
	query::delete_unbanned_ips_before($old);
}

function clear_all_ips()
{
	query::delete_all_ips();
}

function get_banned_ip($id)
{
	$ps = query::get_single_banned_ip($id);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		throw new user_not_found();
	
	return ip_user::from_db($row);
}

function unnotify_subscription($user, $thread)
{
	query::unnotify_subscription($user->id(), $thread->id());
}

function notify_subscriptions($poster_id, $thread_id, $post_id)
{
	$post_url = opt()->base_url() . "/" . make_post_link_impl($thread_id, $post_id);	
	$unsub_url = opt()->base_url() . "/thread.html?s=0&id=$thread_id";
	
	$ps = query::get_subscriptions_for_thread($thread_id);
	foreach ($ps as $row)
	{
		$u = user::from_db($row);
		if ($u->id() == $poster_id)
			continue;
		
		$subject = loc("subscription_subject", $u->profile()->locale());
		$body = sprintf(
			loc("subscription_body", $u->profile()->locale()),
			$post_url, $unsub_url);

		send_email($u->email(), $subject, $body);
	}

	query::notify_subscriptions($thread_id, $poster_id);
}

function banned_ips_list($from, $limit)
{
	$r = array("count" => 0, "ips" => array());
	
	// count
	$ps = query::get_all_banned_ips_count();
	
	$row = $ps->fetch();
	if ($row === FALSE)
		return $r;
	
	$r["count"] = (int)$row["c"];
	if ($r["count"] <= 0)
		return $r;

	// ips
	$ps = query::get_all_banned_ips($from, $limit);
	foreach ($ps as $row)
		array_push($r["ips"], ip_user::from_db($row));
	
	return $r;
}


function make_user_password($cleartext)
{
	$hash = password_hash(
		$cleartext, PASSWORD_DEFAULT, array("cost" => 12));

	if ($hash == null)
		throw new bad_password();
		
	return $hash;
}

function register(
	$email, $name, $password, $perm = user_permission_registered)
{
	if (find_user($email) != null)
		throw new user_already_exists();
	
	$automod = (($perm & user_permission_cp) != 0);

	db()->begin();
	
	query::add_user(
		$email, make_user_password($password),
		$perm, opt()->auto_reg(), $automod);
	
	$id = db()->last_insert_id();
	
	query::add_profile($id, $name);
	
	db()->commit();
	
	if (!opt()->auto_reg())
		send_confirm(get_user($id));
	
	wiki::add_user($email);
	
	return $id;
}

function logout()
{
	session_start();
	session_regenerate_id(true);
	unset($_SESSION["user"]);
	session_write_close();
}

function login($id, $lifetime = 0)
{
	logout();
	
	$u = get_user($id);
	
	if ($u->banned())
		throw new user_banned();
	
	if (!$u->confirmed())
		throw new unconfirmed_user();
	
			
	session_start();

	session_regenerate_id(true);
	setcookie(session_name(), session_id(), time() + $lifetime);

	$_SESSION["user"] = $id;
	session_write_close();
	
	$u->set_last_login(time());
}

function try_login($email, $password, $lifetime = 0)
{
	logout();
	
	$u = find_user($email);
	if ($u === null)
		return false;
	
	if (!$u->check_password($password))
		return false;

	login($u->id());
	return true;
}

function logged_user()
{
	static $u = null;
	
	if ($u === null)
	{
		session_start();
	
		try
		{
			if (!isset($_SESSION["user"]))
			{
				$u = user::anonymous();
			}
			else
			{
				$id = (int)$_SESSION["user"];
				$_SESSION["user"] = $id;
				
				$u = get_user($id);
			}

			session_write_close();
		}
		catch(Exception $e)
		{
			session_write_close();
			throw new user_not_found();
		}
	}

	return $u;
}

function add_ip_if_needed($ip)
{
	$ps = query::find_ip($ip);
	
	$row = $ps->fetch();
	if ($row !== FALSE)
		return (int)$row["id"];
	
	query::add_ip($ip);
	
	return db()->last_insert_id();
}

function add_ip_post($post_id)
{
	cleanup_ips();
	
	$ip_id = add_ip_if_needed(current_ip());
	
	query::add_ip_post($ip_id, $post_id);
}

function ban_ip($ip, $v = true)
{
	cleanup_ips();
	
	if ($ip == current_ip())
		return;
	
	$ip_id = add_ip_if_needed($ip);

	query::update_ip($ip_id, $v);
}

function unban_ip($ip)
{
	ban_ip($ip, false);
}

function ip_banned()
{
	$ps = query::find_ip(current_ip());
	
	$row = $ps->fetch();
	if ($row === FALSE)
		return false;

	if ($row["banned"] == "0")
		return false;
		
	return true;
}

function delete_user_impl($user_id)
{
	query::delete_user($user_id);
	query::delete_profile_for_user($user_id);
}

function delete_user($u)
{
	if (!$u->deletable())
		return;
	
	db()->begin();
	delete_user_impl($u->id());
	db()->commit();
}

function check_user($p = user_permission_none)
{
	if (ip_banned())
	{
		header("HTTP/1.0 403 Forbidden");
		exit;
	}
		
	$u = logged_user();

	if (!$u->banned() && $u->confirmed())
	{
		if (($u->permissions() & $p) == $p)
			return;
	}

	logout();
	header("Location: index.html");
	exit;
}

function check_user_logged()
{
	$u = logged_user();
	
	if ($u->is_anonymous())
	{
		header("Location: index.html");
		exit;
	}
}

function clear_confirm_for($u)
{
	query::delete_registration_for_user($u->id());
}

function cleanup_unconfirmed()
{
	$old = db()->date_time(time() - confirmation_maximum_time);
}

function cleanup_confirm()
{
	db()->begin();

	$old = db()->date_time(time() - registration_maximum_time);
	query::delete_registration_before($old);
	
	$ps = query::get_unconfirmed_user_ids_before($old);
	foreach ($ps as $row)
	{
		$id = (int)$row["id"];
		delete_user_impl($id);
	}

	db()->commit();
}

function send_confirm($u)
{
	cleanup_confirm();
	clear_confirm_for($u);
	
	$uuid = make_uuid();

	query::add_registration($u->id(), $uuid);
	
	$url = opt()->base_url() . "/register.html?h=$uuid";

	$s = loc("register_email_subject");
	$b = sprintf(loc("register_email_body"), $url);
	
	send_email($u->email(), $s, $b);
}

function confirm_registration($h)
{
	cleanup_confirm();
	
	$ps = query::find_registration($h);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		return false;
	
	try
	{
		$uid = (int)$row["user"];
		$u = get_user($uid);
		
		$u->confirm();
		clear_confirm_for($u);
		
		login($u->id());
		
		return true;
	}
	catch(user_not_found $e)
	{
		return false;
	}
}

function confirm_all_registrations()
{
	query::confirm_all_users();

	// registration codes are not deleted in case some users haven't
	// confirmed yet
}

function clear_recovery_for($u)
{
	query::delete_recovery_for_user($u->id());
}

function cleanup_recovery()
{
	$old = db()->date_time(time() - recovery_maximum_time);
	query::delete_recovery_before($old);
}

function recover($u)
{
	cleanup_recovery();
	clear_recovery_for($u);
	
	$uuid = make_uuid();

	query::add_recovery($u->id(), $uuid);
	
	$url = opt()->base_url() . "/reset.html?h=$uuid";

	$s = loc("recover_email_subject");
	$b = sprintf(loc("recover_email_body"), $url);
	
	send_email($u->email(), $s, $b);
}

function get_recovery($uuid)
{
	$ps = query::find_recovery($uuid);
	
	$row = $ps->fetch();
	if ($row === FALSE)
		return null;
	
	$r = recovery::from_db($row);
	
	$d = time() - $r->time();
	if ($d < 0 || $d > recovery_maximum_time)
	{
		$r->erase();
		return null;
	}
	
	return $r;
}

function valid_permissions($p)
{
	if (($p & ~user_permission_all) === 0)
		return true;
	else
		return false;
}

if (!defined("NO_DATABASE"))
{
	try
	{
		logged_user();
	}
	catch(Exception $e)
	{
		logout();
		header("location: index.html");
		die;
	}
}

?>