<?php
if (!defined("bt_include_is_in_script"))
{
	header("Location: ../index.html");
	exit;
}

require_once(__DIR__ . "/libs/class.phpmailer.php");

class bad_parameters extends \Exception {}
class bad_back_parameters extends \Exception {}
class email_failed extends \Exception {}

class bad_7z_file extends \Exception {}
class internal_7z_error extends \Exception {}

$g_this_link = array("name" => "", "params" => array());

define("never_timestamp", 0);
define("max_line_width", 72);

class html
{
	public static function a($u, $c)
	{
		return "<a href=\"" . htmlspecialchars($u) . "\">$c</a>";
	}
	
	public static function radio($name, $value, $label, $checked = false)
	{
		$c = ($checked ? "checked" : "");
		$attrs = "$c type=\"radio\" name=\"$name\" value=\"$value\"";
		
		return "<label><input $attrs />$label</label>";
	}
	
	public static function submit($value, $name="submit")
	{
		$attrs = "type=\"submit\" name=\"$name\" value=\"$value\"";
		return "<input $attrs />";
	}
	
	public static function textarea($name, $cols, $rows, $content = "", $focus = false)
	{
		$af = ($focus ? "autofocus" : "");
		$attrs = "cols=\"$cols\" rows=\"$rows\" name=\"$name\" $af";
		return "<textarea $attrs>$content</textarea>";
	}
	
	public static function checkbox($name, $value, $label, $checked = false)
	{
		$c = ($checked ? "checked" : "");
		$attrs = "$c type=\"checkbox\" name=\"$name\" value=\"$value\"";
		
		return "<label><input $attrs />$label</label>";
	}
	
	public static function text($name, $value = "", $focus = false, $size = 0)
	{
		if ($size == 0)
			$size = 40;
		
		$af = ($focus ? "autofocus" : "");
		$s = ($size == 0 ? "" : "size=\"$size\"");
		
		return "<input $af $s name=\"$name\" value=\"$value\" />";
	}
	
	public static function file($name)
	{
		return "<input name=\"$name\" type=\"file\" />";
	}
	
	public static function hidden($n, $v)
	{
		return "<input type=\"hidden\" name=\"$n\" value=\"$v\" />";
	}
	
	public static function password($name)
	{
		return "<input type=\"password\" size=\"40\" name=\"$name\" />";
	}
	
	public static function wrap_div($cl, $content)
	{
		return "<div class=\"$cl\">$content</div>";
	}
	
	public static function wrap_span($cl, $content)
	{
		return "<span class=\"$cl\">$content</span>";
	}
	
	public static function anchor($n)
	{
		return "<a name=\"$n\"></a>";
	}
}

function safe_get($n, $def = "")
{
	if (isset($_GET[$n]))
		return $_GET[$n];
	else
		return $def;
}

function safe_post($n, $def = "")
{
	if (isset($_POST[$n]))
		return $_POST[$n];
	else
		return $def;
}

function get_backtrace()
{
	ob_start();
	debug_print_backtrace();
	$bt = ob_get_contents();
	ob_end_clean();
	
	return $bt;
}

function fatal_error($e)
{
	//$bt = get_backtrace();
	//echo "<br /><div class=\"bt_forum_fatal_error\">$e<br /><br />$bt</div>";
	trigger_error($e, E_ERROR);
	exit;
}

function make_root_path()
{
	// todo
	$f = $_SERVER["PHP_SELF"];

	if (strpos($f, "/forum/") === FALSE)
		return "";
	else
		return "../";
}

function make_root_link_impl()
{
	return make_root_path() . "index.html";
}

function date_to_string($ts)
{
	$f = \IntlDateFormatter::create(
		logged_user()->profile()->locale(),
		\IntlDateFormatter::FULL, \IntlDateFormatter::SHORT,
		logged_user()->profile()->timezone());
	
	return $f->format($ts);
}

function simple_date_to_string($ts)
{
	if ($ts == 0)
		return "";
	
	$d = time() - $ts;
	
	if ($d < 0)
		return "";
	
	if ($d < 60)
	{
		$v = $d;
		
		if ($v <= 1)
			return sprintf(loc("time_second_ago"), $v);
		else
			return sprintf(loc("time_seconds_ago"), $v);
	}
	else if ($d < (60 * 60))
	{
		$v = (int)($d / 60);
		
		if ($v <= 1) 
			return sprintf(loc("time_minute_ago"), $v);
		else
			return sprintf(loc("time_minutes_ago"), $v);
	}
	else if ($d < (60 * 60 * 24))
	{
		$v = (int)($d / 60 / 60);
		
		if ($v <= 1)
			return sprintf(loc("time_hour_ago"), $v);
		else
			return sprintf(loc("time_hours_ago"), $v);
	}
	else if ($d < (60 * 60 * 24 * 30))
	{
		$v = (int)($d / 60 / 60 / 24);
		
		if ($v == 1)
			return loc("time_yesterday");
		else
			return sprintf(loc("time_days_ago"), $v);
	}
	else if ($d < (60 * 60 * 24 * 365))
	{
		$v = (int)($d / 60 / 60 / 24 / 30);
		
		if ($v <= 1)
			return sprintf(loc("time_month_ago"), $v);
		else
			return sprintf(loc("time_months_ago"), $v);
	}
	else
	{
		$v = (int)($d / 60 / 60 / 24 / 365);
		
		if ($v <= 1)
			return sprintf(loc("time_year_ago"), $v);
		else
			return sprintf(loc("time_years_ago"), $v);
	}
}

function odd_even_class($i)
{
	if (($i & 1) == 0)
		return "bt_even_row";
	else
		return "bt_odd_row";
}

function clamp($v, $min, $max)
{
	if ($v < $min)
		return $min;
	else if ($v > $max)
		return $max;
	else
		return $v;
}

function send_email($email, $subject, $body)
{
	try
	{
		$m = new \PHPMailer(true);
		
		$m->IsSMTP();
		$m->Host = "smtp.teksavvy.com";
		$m->From = "noreply@baretorrent.org";
		$m->FromName = "baretorrent mailer";
		$m->AddAddress("" . $email);
		$m->WordWrap = 0;
		$m->Subject = "" . $subject;
		$m->Body = "" . $body;
		$m->CharSet = "utf-8";
		
		$m->Send();
	}
	catch(Exception $e)
	{
		trigger_error($e->errorMessage(), E_WARNING);
	}
}

function make_uuid()
{
	// from http://www.php.net/manual/en/function.uniqid.php#94959
	
    return sprintf('%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
        // 32 bits for "time_low"
        mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ),

        // 16 bits for "time_mid"
        mt_rand( 0, 0xffff ),

        // 16 bits for "time_hi_and_version",
        // four most significant bits holds version number 4
        mt_rand( 0, 0x0fff ) | 0x4000,

        // 16 bits, 8 bits for "clk_seq_hi_res",
        // 8 bits for "clk_seq_low",
        // two most significant bits holds zero and one for variant DCE1.1
        mt_rand( 0, 0x3fff ) | 0x8000,

        // 48 bits for "node"
        mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff ), mt_rand( 0, 0xffff )
    );
}

function make_safe_string($s)
{
	return htmlentities($s);
}

function make_simple_date($ts)
{
	$short = simple_date_to_string($ts);
	if ($short == "")
		return "";
	
	$long = date_to_string($ts);
	
	return "<span title=\"$long\">$short</span>";
}

function current_ip()
{
	return $_SERVER["REMOTE_ADDR"];
}

function valid_timezone($tz)
{
	$a = timezone_identifiers_list();
	
	if (array_search($tz, $a) === FALSE)
		return false;
	
	return true;
}

function join_path()
{
	$n = func_num_args();
	
	$s = "";
	for ($i=0; $i<$n; ++$i)
	{
		if ($s !== "")
			$s .= DIRECTORY_SEPARATOR;
		
		$s .= func_get_arg($i);
	}
	
	return $s;
}

function make_link_to_self($ps)
{
	$params = $_GET;

	foreach ($ps as $pn=>$pv)
		$params[$pn] = $pv;
	
	$pstring = "";
	foreach ($params as $pn=>$pv)
	{
		if ($pstring != "")
			$pstring .= "&";
			
		$pstring .= $pn . "=" . $pv;
	}

	return $_SERVER["PHP_SELF"] . "?" . $pstring;
}

function set_this_link($name, $params = array())
{
	global $g_this_link;
	$g_this_link["name"] = $name;
	$g_this_link["params"] = $params;
}

function make_this_link($base)
{
	global $g_this_link;
	
	$n = $g_this_link["name"];
	$ps = $g_this_link["params"];
	
	if ($n == "")
		return $base;
	
	$s = "b=$n";	

	foreach ($ps as $pn=>$pv)
		$s .= "&$pn=$pv";
	
	if (strpos($base, "?") === FALSE)
		$s = "?" . $s;
	else
		$s = "&" . $s;
	
	return $base . $s;
}

function make_this_inputs()
{
	global $g_this_link;
	
	$n = $g_this_link["name"];
	$ps = $g_this_link["params"];
	
	if ($n == "")
		return "";
		
	$s = "<input type=\"hidden\" name=\"b\" value=\"$n\" />";
	foreach ($ps as $pn=>$pv)
		$s .= "<input type=\"hidden\" name=\"$pn\" value=\"$pv\" />";
	
	return $s;
}

function one_of_params($names, $use_get)
{
	foreach ($names as $n)
	{
		if ($use_get)
		{
			if (isset($_GET[$n]))
				return array($n => $_GET[$n]);
		}
		else
		{
			if (isset($_POST[$n]))
				return array($n => $_POST[$n]);
		}
	}
	
	throw new bad_back_parameters();
}

function all_of_params($names, $use_get)
{
	$a = array();
	
	foreach ($names as $n)
	{
		if ($use_get)
		{
			if (isset($_GET[$n]))
				$a[$n] = $_GET[$n];
		}
		else
		{
			if (isset($_POST[$n]))
				$a[$n] = $_POST[$n];
		}
	}
	
	return $a;
}

function get_back_params()
{
	$r = array("name" => "", "params" => array());
	
	$use_get = true;
	
	$n = safe_get("b");
	if ($n == "")
	{
		$use_get = false;
		$n = safe_post("b");
		
		if ($n == "")
			return $r;
	}
	
	try
	{
		switch ($n)
		{
			case "index":
			{
				$r["name"] = "index";
				break;
			}
			
			case "ext":
			{
				$r["name"] = "ext";
				break;
			}
			
			case "forumindex":
			{
				$r["name"] = "forumindex";
				break;
			}
				
			case "dev":
			{
				$r["name"] = "dev";
				break;
			}
			
			case "privacy":
			{
				$r["name"] = "privacy";
				break;
			}
			
			case "forum":
			{
				$r["name"] = "forum";
				$r["params"] = one_of_params(array("id"), $use_get);
				break;
			}
			
			case "thread":
			{
				$r["name"] = "thread";
				$r["params"] = one_of_params(array("id"), $use_get);
				break;
			}
			
			case "post":
			{
				$r["name"] = "post";
				$r["params"] = one_of_params(array("p", "t", "f", "e"), $use_get);
				break;
			}
			
			case "erase":
			{
				$r["name"] = "erase";
				$r["params"] = one_of_params(array("p", "t"), $use_get);
				break;
			}
			
			case "cp_users":
			{
				$r["name"] = "cp_users";
				$r["params"] = all_of_params(array("t", "s", "pc", "id"), $use_get);
				break;
			}
		}
		
		return $r;
	}
	catch(bad_back_parameters $e)
	{
		return array("name" => "", "params" => array());
	}
}

function forward_back_link()
{
	$b = get_back_params();
	
	$n = $b["name"];
	$ps = $b["params"];

	set_this_link($n, $ps);
}

function make_back_inputs()
{
	$b = get_back_params();
		
	$n = $b["name"];
	$ps = $b["params"];
	
	if ($n == "")
		return "";
		
	$s = "<input type=\"hidden\" name=\"b\" value=\"$n\" />\n";
	foreach ($ps as $pn=>$pv)
		$s .= "<input type=\"hidden\" name=\"$pn\" value=\"$pv\" />\n";
	
	return $s;
}

function goto_back_link()
{
	$b = get_back_params();
	
	$n = $b["name"];
	$ps = $b["params"];

	$url = make_root_link_impl();

	switch ($n)
	{
		case "index":
		{
			$url = make_root_link_impl();
			break;
		}
		
		case "ext":
		{
			$url = make_root_path() . "ext";
			break;
		}
		
		case "forumindex":
		{
			$url = make_forum_root_link_impl();
			break;
		}
		
		case "dev":
		{
			$url = make_root_path() . "dev.html";
			break;
		}
		
		case "privacy":
		{
			$url = make_root_path() . "privacy.html";
			break;
		}
			
		case "forum":
		{
			$url = make_forum_link_impl($ps["id"]);
			break;
		}
		
		case "thread":
		{
			$url = make_thread_link_impl($ps["id"]);
			break;
		}
		
		case "post":
		{
			if (isset($ps["p"]))
				$url = "post.html?p=" . $ps["p"];
			else if (isset($ps["t"]))
				$url = "post.html?t=" . $ps["t"];
			else if (isset($ps["f"]))
				$url = "post.html?f=" . $ps["f"];
			else if (isset($ps["e"]))
				$url = "post.html?e=" . $ps["e"];
			
			break;
		}
		
		case "erase":
		{
			if (isset($ps["p"]))
				$url = "erase.html?p=" . $ps["p"];
			else if (isset($ps["t"]))
				$url = "erase.html?t=" . $ps["t"];
			
			break;
		}
		
		case "cp_users":
		{
			$valid = array("t", "s", "pc");
			
			$pstring = "";
			foreach ($valid as $v)
			{
				if (isset($ps[$v]))
				{
					if ($pstring != "")
						$pstring .= "&";
					
					$pstring .= $v . "=" . $ps[$v];
				}
			}
			
			$url = "users.html";
			if ($pstring != "")
				$url .= "?" . $pstring;
			
			break;
		}
	}

	header("location: $url");
	exit;
}

function get_7z_file($zip_file, $filename)
{
	$desc = array(
		0 => array("pipe", "r"),
		1 => array("pipe", "w"),
		2 => array("pipe", "w"));
	
	$cmd = "7z x -so \"$zip_file\" \"$filename\"";
	
	$pipes = null;
	$p = proc_open($cmd, $desc, $pipes);
	if (!is_resource($p))
		throw new internal_7z_error();
	
	stream_set_blocking($pipes[2], 0);
	
	$stdout = stream_get_contents($pipes[1]);
	$stderr = stream_get_contents($pipes[2]);
	
	$es = proc_close($p);
	
	if (function_exists("pcntl_wexitstatus"))
		$es = pcntl_wexitstatus($es);
	
	if ($es != 0)
		throw new bad_7z_file();

	return $stdout;
}

function get_optional_7z_file($zip_file, $filename)
{
	try
	{
		return get_7z_file($zip_file, $filename);
	}
	catch(bad_7z_file $e)
	{
		return "";
	}
}

?>