<?php

class wiki_failed extends Exception {}
class wiki_login_failed extends wiki_failed {}
class wiki_edit_token_failed extends wiki_failed {}
class wiki_edit_failed extends wiki_failed {}
class wiki_bad_filename extends wiki_failed {}
class wiki_undo_failed extends wiki_failed {}

class wiki
{
	static public function make_username($email)
	{
		$ss = array("#", "<", ">", "[", "]", "|", "{", "}", "O", "U", "@");
		return str_replace($ss, " ", $email);
	}
	
	static public function add_user($email)
	{	
		$c = curl_init();
		
		$ps = array(
			"name" => wiki::make_username($email),
			"password" => wiki::password());
			
		$s = wiki::post_api($c, "createaccount", $ps);
		if ($s === FALSE)
			return false;
		
		$token = wiki::parse_token($s, "createaccount");
		if ($token === FALSE)
			return false;
		
		$ps["token"] = $token;
		$s = wiki::post_api($c, "createaccount", $ps);
		
		if (!wiki::success($s, "createaccount"))
			return false;
		
		return true;
	}

	static public function markup($title)
	{
		$c = curl_init();
		
		$ps = array(
			"titles" => $title,
			"prop" => "revisions",
			"rvprop" => "content");
		
		$s = wiki::get_api($c, "query", $ps);
		if ($s === FALSE)
			return false;

		$doc = new DOMDocument();
		$doc->loadXML($s);
		
		$es = $doc->getElementsByTagName("rev");
		if ($es->length == 0)
			return FALSE;
		
		return $es->item(0)->nodeValue;
	}

	static public function save($page, $content, $summary = "")
	{
		$c = curl_init();
		
		wiki::login($c);
		
		$et = wiki::edit_token($c, $page);
		if ($et === FALSE)
			throw new wiki_edit_token_failed;
		
		$ps = array(
			"title" => $page,
			"text" => $content,
			"summary" => $summary,
			"starttimestamp" => $et["timestamp"],
			"md5" => md5($content),
			"token" => $et["token"]);
		
		$s = wiki::post_api($c, "edit", $ps);
		if (!wiki::success($s, "edit"))
			throw new wiki_edit_failed;
	}
	
	static public function history($page)
	{
		$c = curl_init();
		
		$ps = array(
			"prop" => "revisions",
			"titles" => $page,
			"rvprop" => "ids|timestamp|user|parsedcomment",
			"rvlimit" => 100);
		
		$s = wiki::get_api($c, "query", $ps);
		if ($s === FALSE)
			return FALSE;

		$doc = new DOMDocument();
		$doc->loadXML($s);
		
		$es = $doc->getElementsByTagName("revisions");
		if ($es->length == 0)
			return FALSE;
		
		$rs = $es->item(0);

		$entries = array();
		foreach ($rs->childNodes as $r)
		{
			$time = $r->getAttribute("timestamp");
			$user = $r->getAttribute("user");
			$comment = $r->getAttribute("parsedcomment");
			$revid = $r->getAttribute("revid");

			array_push($entries, array(
				"revid" => $revid,
				"time" => strtotime($time),
				"user" => $user,
				"comment" => $comment));
		}
		
		return $entries;
	}
	
	static public function undo($page, $rev)
	{
		$c = curl_init();
		
		wiki::login($c);
		
		$et = wiki::edit_token($c, $page);
		if ($et === FALSE)
			throw new wiki_edit_token_failed;
		
		$ps = array(
			"title" => $page,
			"undo" => $rev,
			"summary" => $summary,
			"starttimestamp" => $et["timestamp"]);
			//"token" => $et["token"]);
		
		$s = wiki::post_api($c, "edit", $ps);
		
		echo $s;exit;
		

		
		if ($s === FALSE)
			throw new wiki_failed;
		
		echo $s;exit;
		
		if (wiki::result($s, "edit") == "undo-failure")
			throw new wiki_undo_failed;
		
		if (!wiki::success($s, "edit"))
			throw new wiki_edit_failed;
	}
	
	static public function upload($filename, $tmp_name)
	{
		$c = curl_init();
		
		wiki::login($c);
		
		$et = wiki::edit_token($c);
		if ($et === FALSE)
			return false;
		
		$ps = array(
			"filename" => $filename,
			"file" => "@$tmp_name",
			"token" => $et["token"]);
		
		$s = wiki::post_api($c, "upload", $ps, "multipart/form-data");
		if ($s === FALSE)
			return new wiki_failed;
			
		if (wiki::result($s, "upload") == "badfilename")
			throw new wiki_bad_filename;
		
		if (!wiki::success($s, "upload"))
			return new wiki_failed;
	}
	
	static public function parse($content)
	{
		$c = curl_init();
		
		$ps = array(
			"text" => $content,
			"disablepp" => "1");
		
		$s = wiki::post_api($c, "parse", $ps);
		if ($s === FALSE)
			return FALSE;
		
		$doc = new DOMDocument();
		$doc->loadXML($s);
		
		$es = $doc->getElementsByTagName("text");
		if ($es->length == 0)
			return FALSE;
		
		return $es->item(0)->nodeValue;
	}
	
	static public function diff_content($page, $content)
	{
		$c = curl_init();
		
		$ps = array(
			"prop" => "revisions",
			"titles" => $page,
			"rvdiffto" => "cur",
			"rvdifftotext" => $content);
		
		$s = wiki::post_api($c, "query", $ps);
		if ($s === FALSE)
			return FALSE;
		
		$doc = new DOMDocument();
		$doc->loadXML($s);
		
		$es = $doc->getElementsByTagName("diff");
		if ($es->length == 0)
			return FALSE;
		
		return "
			<table class=\"diff diff-contentalign-left\">
				<col class='diff-marker' />
				<col class='diff-content' />
				<col class='diff-marker' />
				<col class='diff-content' />
				". $es->item(0)->nodeValue . "
			</table>";
	}
	
	static public function diff($page, $older, $newer = "")
	{
		if ($newer == "")
			$newer = "prev";
		
		$c = curl_init();
		
		$ps = array(
			"prop" => "revisions",
			"titles" => $page,
			"rvstartid" => $older,
			"rvdiffto" => $newer);
		
		$s = wiki::post_api($c, "query", $ps);
		if ($s === FALSE)
			return FALSE;
		
		$doc = new DOMDocument();
		$doc->loadXML($s);
		
		$es = $doc->getElementsByTagName("diff");
		if ($es->length == 0)
			return FALSE;
		
		return "
			<table class=\"diff diff-contentalign-left\">
				<col class='diff-marker' />
				<col class='diff-content' />
				<col class='diff-marker' />
				<col class='diff-content' />
				". $es->item(0)->nodeValue . "
			</table>";
	}
	
	static public function page($name, $rev = "")
	{
		$c = curl_init();
		
		$ps = array(
			"page" => $name,
			"format" => "xml",
			"disablepp" => "1");
		
		if ($rev != "")
			$ps["oldid"] = $rev;
		
		$s = wiki::get_api($c, "parse", $ps);
		
		$doc = new DOMDocument;
		$doc->loadXML($s);
		
		$es = $doc->getElementsByTagName("text");
		if ($es->length == 0)
			return "";
		
		$e = $es->item(0);
		return $e->nodeValue;
	}
	
	
	static private function password()
	{
		return "abcdef12345";
	}

	static private function index_url()
	{
		return "http://127.0.0.1/bt/wiki/index.php";
	}
	
	static private function api_url()
	{
		return "http://127.0.0.1/bt/wiki/api.php";
	}

	static private function api_impl($c, $action, $ps, $post, $ct)
	{
		$ps["action"] = $action;
		$ps["format"] = "xml";

		$url = wiki::api_url();
		
		curl_setopt($c, CURLOPT_HTTPHEADER, array());
		curl_setopt($c, CURLOPT_POST, false);
		curl_setopt($c, CURLOPT_FOLLOWLOCATION, true);
	
		if ($post)
		{
			curl_setopt($c, CURLOPT_POST, true);
			curl_setopt($c, CURLOPT_POSTFIELDS, $ps);
			
			if ($ct != "")
			{
				curl_setopt(
					$c, CURLOPT_HTTPHEADER, array("Content-type: $ct"));
			}
		}
		else
		{
			$url .= "?";
			
			foreach ($ps as $n=>$v)
				$url .= "$n=$v&";
		}
		
		curl_setopt($c, CURLOPT_URL, $url);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($c, CURLOPT_COOKIEFILE, "");
		
		$s = curl_exec($c);
		if ($s === FALSE)
			return FALSE;

		return $s;
	}

	static private function post_api($c, $action, $ps=array(), $ct="")
	{
		return wiki::api_impl($c, $action, $ps, true, $ct);
	}

	static private function get_api($c, $action, $ps = array())
	{
		return wiki::api_impl($c, $action, $ps, false, "");
	}

	static private function parse_token($xml, $tag)
	{
		$doc = new DOMDocument();
		$doc->loadXML($xml);
		
		$token = "";
		
		$es = $doc->getElementsByTagName($tag);
		for ($i=0; $i<$es->length; ++$i)
		{
			$e = $es->item($i);
			
			if (strcasecmp($e->getAttribute("result"), "needtoken") != 0)
				continue;
			
			return $e->getAttribute("token");
		}
		
		return FALSE;
	}

	static private function edit_token($c, $page = "")
	{
		if ($page == "")
			$page = "Main_Page";
		
		$ps = array(
			"prop" => "info",
			"intoken" => "edit",
			"titles" => $page);
		
		$s = wiki::get_api($c, "query", $ps);
		if ($s === FALSE)
			return FALSE;
		
		$doc = new DOMDocument();
		$doc->loadXML($s);
		
		$es = $doc->getElementsByTagName("page");
		if ($es->length == 0)
			return FALSE;
			
		$token = $es->item(0)->getAttribute("edittoken");
		if ($token == "")
			return FALSE;
		
		$ts = $es->item(0)->getAttribute("starttimestamp");
		if ($ts == "")
			return FALSE;
		
		return array("token"=>$token, "timestamp"=>$ts);
	}

	static private function success($xml, $tag)
	{
		$doc = new DOMDocument();
		$doc->loadXML($xml);
		
		$es = $doc->getElementsByTagName($tag);
		for ($i=0; $i<$es->length; ++$i)
		{
			$e = $es->item($i);
			
			if (strcasecmp($e->getAttribute("result"), "success") == 0)
				return true;
		}
		
		return false;
	}
	
	static private function result($xml, $tag)
	{
		$doc = new DOMDocument();
		$doc->loadXML($xml);
		
		$es = $doc->getElementsByTagName($tag);
		if ($es->length == 0)
			return "";
		
		$e = $es->item(0);
		
		if (strcasecmp($e->getAttribute("result"), "success") == 0)
			return "";
		
		if (strcasecmp($e->getAttribute("result"), "warning") == 0)
		{
			$ws = $e->getElementsByTagName("warnings");
			if ($ws->length == 0)
				return "";
			
			$w = $ws->item(0);
			
			if ($w->attributes->length == 0)
				return "";
			
			return $w->attributes->item(0)->nodeName;
		}
		
		return "";
	}

	static private function login($c)
	{
		$u = logged_user();
		if ($u->is_anonymous())
			throw new wiki_login_failed;
		
		$ps = array(
			"lgname" => wiki::make_username($u->email()),
			"lgpassword" => wiki::password());
		
		$s = wiki::post_api($c, "login", $ps);
		if ($s === FALSE)
			throw new wiki_login_failed;
		
		$token = wiki::parse_token($s, "login");
		if ($token === FALSE)
			throw new wiki_login_failed;
		
		$ps["lgtoken"] = $token;
		$s = wiki::post_api($c, "login", $ps);
		
		if (!wiki::success($s, "login"))
			throw new wiki_login_failed;
	}

	static private function logout()
	{
		$c = curl_init();
		$s = wiki::get_api($c, "logout");
	}
}
?>
