// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_APP_INCLUDED
#define BARETORRENT_APP_INCLUDED

#include "platform.h"
#include "widgets.h"

namespace baretorrent
{

class torrent;
class main_window;
class manager;

// this is the entry point for wxWidgets and an instance is created on
// the stack in start(); it creates the main window and handles the
// manager
//
class app : public wxApp
{
public:
	typedef sig::signal<void ()> update_handler;
	typedef sig::signal<void (bool)> update_done_handler;
	typedef sig::signal<void (baretorrent_version)> version_handler;
	typedef sig::signal<void (const utf8_string&)> association_handler;
	

	update_handler checking_update;
	update_done_handler update_done;
	version_handler update_available;
	association_handler missing_association;


	// nothing much
	//
	app();
	~app();


	void add_torrents(const std::vector<utf8_string>& v);

	// the manager is created on the stack in start() and given here
	//
	void set(baretorrent::manager& m);

	// creates the main window hidden
	//
	void create_window();

	// shows the main window
	//
	void show_window();

	// called from wxWidgets when the application starts, initializes
	// some wxWidgets stuff like the bitmap handlers and adds an
	// exception to the system firewall (if any)
	//
	virtual bool OnInit();

	// closes the main window, interrupts the manager and quits as soon
	// as possible, which may not be that soon since libtorrent has to
	// clean up
	//
	void quit();

	// whether quit() has been called
	//
	bool quitting() const;

	// main window
	//
	main_window& window();

	// returns a pointer to the manager, may be null just after
	// startup
	//
	baretorrent::manager* manager_ptr();

	// returns the manager
	//
	baretorrent::manager& manager();


	// called on startup and from the main window when the options have
	// changed
	//
	void reload_settings();


	// disables update checking for this session
	//
	void disable_updates();

#if BARETORRENT_APPLE
	virtual void MacOpenFiles(const wxArrayString& files);
	virtual void MacOpenURL(const wxString& url);
#endif

private:
	// platform-specific initialization and cleanup (raii)
	platform::platform pf_;

	// main window
	main_window* window_;

	// ticks the manager (and therefore libtorrent)
	timer timer_;

	// saves the options
	timer options_timer_;

	// checks for updates
	boost::thread update_thread_;

	// current update delay; if this didn't change, the timer isn't
	// restarted
	int update_delay_;

	// whether to stop the update process; this is picked up by curl
	// if stuck on the network
	bool update_stop_;

	// whether updates are disabled, set when the user clicks the
	// 'not now' button
	bool update_disabled_;

	// whether quit() has been called
	bool quitting_;

	// the torrent manager which interacts with libtorrent
	baretorrent::manager* manager_;

	std::vector<utf8_string> new_;


	bool update_cb(double);

	// fired by the manager timer
	//
	bool on_timer();

	// fired by options_timer_; saves the options
	//
	bool on_options_timer();

	// sleeps for a while and calls check_update()
	//
	void check_update_thread();

	// checks if there's a new version
	//
	bool check_update();

	// checks file and scheme associations
	//
	void check_associations();
};

app& get_app();

// convenience, forwards to app::quit()
//
void quit();

}	// namespace baretorrent

#endif	// BARETORRENT_APP_INCLUDED
