// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_CONFIG_INCLUDED
#define BARETORRENT_CONFIG_INCLUDED

namespace baretorrent
{

// platform
#define BARETORRENT_WINDOWS 0
#define BARETORRENT_APPLE 0
#define BARETORRENT_LINUX 0
#define BARETORRENT_POSIX 0

// windows features
#define BARETORRENT_VISTA_PLUS 0

// compiler
#define BARETORRENT_VISUALC 0
#define BARETORRENT_GCC 0
#define BARETORRENT_HAS_PCH 0

// whether the compiler currently has any kind of debugging flag
#define BARETORRENT_DEBUG 0
#define BARETORRENT_USE_DEBUG_PYTHON 0

// native character encoding of the platform
#define BARETORRENT_TARGET_IS_UTF8 0
#define BARETORRENT_TARGET_IS_UTF16 0

// architecture
#define BARETORRENT_X86 0
#define BARETORRENT_X64 0

// features
#define BARETORRENT_HAS_NULLPTR 1

// compiler compatibility
#define BARETORRENT_FILE_MACRO
#define BARETORRENT_LINE_MACRO
#define BARETORRENT_FUNCTION_MACRO
#define BARETORRENT_DATE_MACRO
#define BARETORRENT_TIME_MACRO
#define BARETORRENT_UNUSED

// toolkit
#define BARETORRENT_GTK 0
#define BARETORRENT_AQUA 0

// gcc-specific features
#define BARETORRENT_GCC_HAS_DIAGNOSTIC 0

// separator between paths in environment variables (";" on windows,
// ":" on posix)
#define BARETORRENT_PATH_ENV_SEPARATOR

// separator between path components (\ on windows, / on posix)
#define BARETORRENT_DIRECTORY_SEPARATOR

// exports or imports names from the dll
#define BARETORRENT_EXPORT

// platform check
#if defined(_WIN32)
	#undef BARETORRENT_WINDOWS
	#define BARETORRENT_WINDOWS 1

	#ifndef _USING_V110_SDK71_
		#undef BARETORRENT_VISTA_PLUS
		#define BARETORRENT_VISTA_PLUS 1
	#endif

#elif defined(__APPLE__)
	#undef BARETORRENT_APPLE
	#define BARETORRENT_APPLE 1

	#undef BARETORRENT_POSIX
	#define BARETORRENT_POSIX 1

	#undef BARETORRENT_AQUA
	#define BARETORRENT_AQUA 1

#elif defined(__linux__)
	#undef BARETORRENT_LINUX
	#define BARETORRENT_LINUX 1

	#undef BARETORRENT_POSIX
	#define BARETORRENT_POSIX 1

	#ifdef __WXGTK__
		#undef BARETORRENT_GTK
		#define BARETORRENT_GTK 1
	#endif

#else
	#error unrecognized platform

#endif


// compiler check
#if defined(_MSC_VER) || defined(RC_INVOKED)
	// _MSC_VER is not defined when compiling resources, but RC_INVOKED
	// is

	#undef BARETORRENT_VISUALC
	#define BARETORRENT_VISUALC 1

	#undef BARETORRENT_HAS_PCH
	#define BARETORRENT_HAS_PCH 1

	// visual c++ defines _DEBUG by default when compiled in debug mode
	#if defined(_DEBUG)
		#undef BARETORRENT_DEBUG
		#define BARETORRENT_DEBUG 1
	#endif

	#ifdef _M_X64	
		#undef BARETORRENT_X64
		#define BARETORRENT_X64 1
	#else
		#undef BARETORRENT_X86
		#define BARETORRENT_X86 1
	#endif

	// visual c++ outputs bogus messages about functions being deprecated
	#ifndef _CRT_SECURE_NO_WARNINGS
		#define _CRT_SECURE_NO_WARNINGS
	#endif

	#ifndef _SCL_SECURE_NO_WARNINGS
		#define _SCL_SECURE_NO_WARNINGS
	#endif

	#undef BARETORRENT_FILE_MACRO
	#define BARETORRENT_FILE_MACRO __FILE__

	#undef BARETORRENT_LINE_MACRO
	#define BARETORRENT_LINE_MACRO __LINE__
	
	#undef BARETORRENT_FUNCTION_MACRO
	#define BARETORRENT_FUNCTION_MACRO __FUNCTION__

	#undef BARETORRENT_DATE_MACRO
	#define BARETORRENT_DATE_MACRO __DATE__

	#undef BARETORRENT_TIME_MACRO
	#define BARETORRENT_TIME_MACRO __TIME__

	#undef BARETORRENT_UNUSED
	#define BARETORRENT_UNUSED(x) __pragma(warning(suppress: 4100)) x

	#undef BARETORRENT_EXPORT
	#define BARETORRENT_EXPORT __declspec(dllexport)

	#undef BARETORRENT_DIRECTORY_SEPARATOR
	#define BARETORRENT_DIRECTORY_SEPARATOR "\\"

#elif defined(__GNUC__)
	#undef BARETORRENT_GCC
	#define BARETORRENT_GCC 1

	// gcc defines NDEBUG when compiled without -g
	#if !defined(NDEBUG)
		#undef BARETORRENT_DEBUG
		#define BARETORRENT_DEBUG 1
	#endif

	#if __GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ <= 6)
		#undef BARETORRENT_HAS_NULLPTR
		#define BARETORRENT_HAS_NULLPTR 0
	#elif _cplusplus < 201103L
		#undef BARETORRENT_HAS_NULLPTR
		#define BARETORRENT_HAS_NULLPTR 0
	#endif

	#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
		#undef BARETORRENT_GCC_HAS_DIAGNOSTIC
		#define BARETORRENT_GCC_HAS_DIAGNOSTIC 1
	#endif

	#ifdef __x86_64__
		#undef BARETORRENT_X64
		#define BARETORRENT_X64 1
	#else
		#undef BARETORRENT_X86
		#define BARETORRENT_X86 1
	#endif

	#undef BARETORRENT_FILE_MACRO
	#define BARETORRENT_FILE_MACRO __FILE__

	#undef BARETORRENT_LINE_MACRO
	#define BARETORRENT_LINE_MACRO __LINE__
	
	#undef BARETORRENT_FUNCTION_MACRO
	#define BARETORRENT_FUNCTION_MACRO __PRETTY_FUNCTION__

	#undef BARETORRENT_DATE_MACRO
	#define BARETORRENT_DATE_MACRO __DATE__

	#undef BARETORRENT_TIME_MACRO
	#define BARETORRENT_TIME_MACRO __TIME__

	#undef BARETORRENT_UNUSED
	#define BARETORRENT_UNUSED(x) x __attribute__((unused))

	#undef BARETORRENT_DIRECTORY_SEPARATOR
	#define BARETORRENT_DIRECTORY_SEPARATOR "/"

#else
	#error unrecognized compiler

#endif

#if BARETORRENT_DEBUG
	// this is used just in case there's a problem because it requires
	// changing target names to baretorrent_d.pyd, etc.
	#undef BARETORRENT_USE_DEBUG_PYTHON
	#define BARETORRENT_USE_DEBUG_PYTHON 1
#else
	#ifndef NDEBUG
		#define NDEBUG
	#endif
#endif

// encoding check: windows is utf16, but all other platforms are
// assumed to be utf8
#if BARETORRENT_WINDOWS
	#undef BARETORRENT_TARGET_IS_UTF16
	#define BARETORRENT_TARGET_IS_UTF16 1

#else
	#undef BARETORRENT_TARGET_IS_UTF8
	#define BARETORRENT_TARGET_IS_UTF8 1

#endif

// path separator
#if BARETORRENT_POSIX
	#undef BARETORRENT_PATH_ENV_SEPARATOR
	#define BARETORRENT_PATH_ENV_SEPARATOR ":"
#else
	#undef BARETORRENT_PATH_ENV_SEPARATOR
	#define BARETORRENT_PATH_ENV_SEPARATOR ";"
#endif

// native path character type
#if BARETORRENT_POSIX
	typedef char path_char_t;
#else
	typedef wchar_t path_char_t;
#endif

// gcc has this on linux
#ifdef linux
#undef linux
#endif

struct config
{
	static bool windows()
	{
		return (BARETORRENT_WINDOWS == 0 ? false : true);
	}

	static bool apple()
	{
		return (BARETORRENT_APPLE == 0 ? false : true);
	}

	static bool linux()
	{
		return (BARETORRENT_LINUX == 0 ? false : true);
	}

	static bool posix()
	{
		return (BARETORRENT_POSIX == 0 ? false : true);
	}


	static bool debug()
	{
		return (BARETORRENT_DEBUG == 0 ? false : true);
	}

	static bool target_is_utf8()
	{
		return (BARETORRENT_TARGET_IS_UTF8 == 0 ? false : true);
	}

	static bool target_is_utf16()
	{
		return (BARETORRENT_TARGET_IS_UTF16 == 0 ? false : true);
	}

	static bool x86()
	{
		return (BARETORRENT_X86 == 0 ? false : true);
	}

	static bool x64()
	{
		return (BARETORRENT_X64 == 0 ? false : true);
	}
};

#if BARETORRENT_HAS_NULLPTR == 0
	#define nullptr 0
#endif

// todo: wxWidgets doesn't define this on apple and linux with the
// versions I'm currently using
#if !BARETORRENT_WINDOWS && BARETORRENT_X86
	typedef long wxIntPtr;
#endif

// python fails because environ doesn't exist on apple; this is defined
// in pch.cpp
#if BARETORRENT_APPLE
	extern char** environ;
#endif

}	// namespace baretorrent

#endif	// BARETORRENT_CONFIG_INCLUDED
