// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_CONNECTIONS_PANEL_INCLUDED
#define BARETORRENT_CONNECTIONS_PANEL_INCLUDED

#include "utility.h"
#include "custom_list.h"

namespace baretorrent
{

// this is the panel on the main window that displays the trackers and
// peers
//
class connections_panel : public wxPanel
{
public:
	// creates the panel and its widgets
	//
	connections_panel(wxWindow* parent);

	// cleans up the user data in the lists
	//
	~connections_panel();
	
	// empties all the widgets
	//
	void clear();

	// updates the widgets for the given torrents
	//
	void update(const torrents& ts);

private:
	DECLARE_EVENT_TABLE()

	// widgets
	custom_list<bt::announce_entry>* trackers_list_;
	wxStaticText* trackers_caption_;
	custom_list<bt::peer_info>* peers_list_;
	wxStaticText* peers_caption_;
	wxMenu* trackers_popup_;


	// runs update_thread()
	boost::thread thread_;

	// protects trackers_ and peers_ since they're used in both
	// the thread and the update timer
	boost::mutex info_mutex_;

	// a copy of the trackers list from libtorrent fetched from the
	// thread
	std::vector<bt::announce_entry> trackers_;

	// a copy of the peers list from libtorrent fetched from the thread
	std::vector<bt::peer_info> peers_;

	// update timer
	timer timer_;

	// set to true when the selection first changes so they "updating" labels
	// can be set; set to false as soon as the update thread gets values
	bool initial_;

	// set to true when the lists are showing the "updating" labels so they
	// can be cleared before updating
	bool showing_initial_;


	// protects current_ since it's used in various places, such as
	// timers and the update thread
	boost::recursive_mutex current_mutex_;

	// currently displayed torrents
	torrents current_;


	// creates the trackers list
	//
	wxWindow* create_trackers(wxWindow* parent);

	// creates the peers list
	//
	wxWindow* create_peers(wxWindow* parent);


	// retrieves the peers and trackers so they can be updated from the
	// timer
	void update_thread();

	// updates the various widgets based on the selection; calls
	// update_impl()
	//
	bool on_timer();

	// updates the widgets based on the selection; may call
	// multiple_selection() or single_torrent()
	//
	void update_impl(const torrents& ts);

	// clears the list and adds the "updating" labels; called when the
	// selection changes
	//
	void initial_update();

	// sets up the widgets for multiple torrents
	//
	void multiple_selection();

	// sets up the widgets for a single torrent; may call no_metadata()
	//
	void single_torrent();

	// updates the trackers list for a single torrent with metadata
	//
	void update_trackers();

	// updates the peers list for a single torrent with metadata
	//
	void update_peers();

	// updates a single tracker in the list
	//
	void update_tracker(const bt::announce_entry& e, int i);

	// updates a single peer in the list
	//
	void update_peer(const bt::peer_info& p, int i);


	// fired when the tracker menu pops up, sets the state of menu items
	//
	void on_tracker_popup_update(wxUpdateUIEvent& e);

	// asks the user for tracker urls
	//
	void on_add_tracker(wxCommandEvent&);

	// removes the selected trackers
	//
	void on_remove_tracker(wxCommandEvent&);
};

}	// namespace baretorrent

#endif	// BARETORRENT_CONNECTIONS_PANEL_INCLUDED
