// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_DIALOG_INCLUDED
#define BARETORRENT_DIALOG_INCLUDED

#include "path.h"
#include "options.h"
#include "custom_list.h"

// this contains some simple dialogs and panels

namespace baretorrent
{

class manager;

// takes a list of torrents and adds them
//
class add_dialog : public wxDialog
{
public:
	add_dialog(wxWindow* parent);

	// returns the list of filenames and/or urls
	//
	std::vector<utf8_string> what() const;

private:
	DECLARE_EVENT_TABLE()

	wxTextCtrl* text_;
	std::vector<utf8_string> what_;

	void on_browse(wxCommandEvent&);
	void on_close(wxCommandEvent&);
};

class queue_dialog : public wxFrame
{
public:
	queue_dialog(wxWindow* parent, manager& m);

private:
	wxStaticText* label_;
	wxGauge* progress_;

	void on_update_event(wxCommandEvent& e);
	void on_update(std::size_t c, std::size_t t);

	void update(std::size_t c, std::size_t t);
};

// confirms torrent removal
//
class remove_dialog : public wxDialog
{
public:
	remove_dialog(wxWindow* parent);

	bool keep_files() const;

private:
	DECLARE_EVENT_TABLE()

	wxCheckBox* del_;
	bool keep_;

	void on_close(wxCommandEvent&);
};

// shown when the user adds bad torrents
//
class bad_torrents_dialog : public wxDialog
{
public:
	bad_torrents_dialog(
		wxWindow* parent, const std::vector<utf8_string>& v);
};

// adds trackers
//
class add_trackers_dialog : public wxDialog
{
public:
	add_trackers_dialog(wxWindow* parent);

	std::vector<utf8_string> trackers() const;

private:
	DECLARE_EVENT_TABLE()

	wxTextCtrl* text_;
	std::vector<utf8_string> what_;

	void on_close(wxCommandEvent&);
};

// about dialog
//
class about_dialog : public wxDialog
{
public:
	about_dialog(wxWindow* parent);

private:
	wxSizer* make_versions();
};


class change_option_dialog : public wxDialog
{
public:
	change_option_dialog(
		wxWindow* w, const option& o);

	utf8_string value() const;

private:
	DECLARE_EVENT_TABLE()

	wxTextCtrl* text_;
	option opt_;

	void on_reset(wxCommandEvent&);
	void on_ok(wxCommandEvent&);
};

class options_dialog;

// a panel that has a list of options, a search box and a description
// when options are selected; this works with either the global or
// extension options
//
class option_list_widget : public wxPanel
{
public:
	typedef sig::signal<void (std::string, option)> handler;

	handler changed;


	// creates the panel; if guid is not empty, this is for an extension
	//
	option_list_widget(wxWindow* parent, const std::string& guid);

	// clears the list
	//
	~option_list_widget();


	// rebuilds the list with the given search string
	//
	void rebuild(const utf8_string& search = utf8_string());

private:
	DECLARE_EVENT_TABLE()

	// guid of the extension
	std::string guid_;

	// options list
	custom_list<option>* list_;

	// description of the selected option
	wxTextCtrl* description_;

	// search box to filter options
	wxTextCtrl* search_;

	// options that changed
	std::map<std::string, option_list> new_;


	// fired when the user double clicks an item in the advanced list;
	// shows the text entry dialog
	//
	void on_activated(option* o);

	// fired when the user selects an item in the advanced list; sets
	// the description
	//
	void on_selection(option* o);

	// fired when user types in the search box
	//
	void on_search_changed(wxCommandEvent&);

	// returns the option selected in the advanced list; may be
	// null
	//
	const option* selected_option() const;

	// updates the given list item; called on rebuild and after an
	// option is changed by the user
	//
	void update_impl(int i, const option& o);
};


class power_dialog : public wxDialog
{
public:
	power_dialog(wxWindow* parent, power_actions p);
	virtual int ShowModal();

private:
	wxStaticText* label_;
	boost::format message_;

	timer timer_;
	boost::posix_time::ptime end_;

	bool on_timer();
	void set_label();
};

}	// namespace baretorrent

#endif	// BARETORRENT_DIALOG_INCLUDED
