// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#include "pch.h"
#include "../pch.h"
#include "../path.h"

namespace baretorrent
{
namespace ext
{

// utf8_path -> python
//
struct utf8_path_to_python
{
	typedef bpl::copy_const_reference::apply<
		const std::string&>::type converter;

	utf8_path_to_python()
	{
		bpl::to_python_converter<
			utf8_path, utf8_path_to_python, true>();
	}

	static PyObject* convert(const utf8_path& p)
	{
		converter c;
		return c(p.as_utf8_string().std_string());
	}
	
	static const PyTypeObject* get_pytype()
	{
		converter c;
		return c.get_pytype();
	}
};

// python -> utf8_path
//
struct utf8_path_from_python_str
{
	utf8_path_from_python_str()
	{
		bpl::converter::registry::push_back(
			&convertible, &construct, bpl::type_id<utf8_path>());
	}

	static void* convertible(PyObject* obj)
	{
		try
		{
			bpl::object s(bpl::handle<>(bpl::borrowed(obj)));
			
			if (bpl::len(s) > 1)
			{
				if (!bpl::extract<std::string>(s).check())
					return nullptr;
			}

			return obj;
		}
		catch (bpl::error_already_set&)
		{
			bpl::handle_exception();
			PyErr_Clear();
			return nullptr;
		}
	}

	static void construct(
		PyObject* obj,
		bpl::converter::rvalue_from_python_stage1_data* data)
	{
		bpl::object s(bpl::handle<>(bpl::borrowed(obj)));

		typedef bpl::converter::rvalue_from_python_storage<utf8_path>
			storage_t;

		storage_t* storage = reinterpret_cast<storage_t*>(data);
		void* bytes = storage->storage.bytes;

		const utf8_string ss((bpl::extract<std::string>(s)()));

		new (bytes) utf8_path(ss);
		data->convertible = bytes;
	}
};


// utf8_string -> python
//
struct utf8_string_to_python
{
	typedef bpl::copy_const_reference::apply<
		const std::string&>::type converter;

	utf8_string_to_python()
	{
		bpl::to_python_converter<
			utf8_string, utf8_string_to_python, true>();
	}

	static PyObject* convert(const utf8_string& p)
	{
		converter c;
		return c(p.std_string());
	}
	
	static const PyTypeObject* get_pytype()
	{
		converter c;
		return c.get_pytype();
	}
};

// python -> utf8_path
//
struct utf8_string_from_python_str
{
	utf8_string_from_python_str()
	{
		bpl::converter::registry::push_back(
			&convertible, &construct, bpl::type_id<utf8_string>());
	}

	static void* convertible(PyObject* obj)
	{
		try
		{
			bpl::object s(bpl::handle<>(bpl::borrowed(obj)));
			
			if (bpl::len(s) > 1)
			{
				if (!bpl::extract<std::string>(s).check())
					return nullptr;
			}

			return obj;
		}
		catch (bpl::error_already_set&)
		{
			bpl::handle_exception();
			PyErr_Clear();
			return nullptr;
		}
	}

	static void construct(
		PyObject* obj,
		bpl::converter::rvalue_from_python_stage1_data* data)
	{
		bpl::object s(bpl::handle<>(bpl::borrowed(obj)));

		typedef bpl::converter::rvalue_from_python_storage<utf8_string>
			storage_t;

		storage_t* storage = reinterpret_cast<storage_t*>(data);
		void* bytes = storage->storage.bytes;

		new (bytes) utf8_string((bpl::extract<std::string>(s)()));
		data->convertible = bytes;
	}
};


void export_converters()
{
	// utf8_string
	bpl::to_python_converter<utf8_string, utf8_string_to_python>();
	bpl::implicitly_convertible<std::string, utf8_string>();

	// utf8_path
	bpl::to_python_converter<utf8_path, utf8_path_to_python>();
	bpl::implicitly_convertible<utf8_string, utf8_path>();

	// vectors
	bpl::class_<std::vector<int64_t> >("vector_of_int64s")
		.def(bpl::vector_indexing_suite<std::vector<int64_t> >());

	bpl::class_<std::vector<int> >("vector_of_ints")
		.def(bpl::vector_indexing_suite<std::vector<int> >());
}

}	// namespace ext
}	// namespace baretorrent
