// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#include "pch.h"
#include "../pch.h"
#include "ext.h"
#include "converters.h"
#include "../app.h"
#include "../torrent.h"
#include "../manager.h"
#include "../main_window.h"

namespace baretorrent
{
namespace ext
{

// gcc has this on linux
#ifdef linux
#undef linux
#endif

struct panel
{
	wxWindow* p;
	std::string class_name;
	ext::instance_ptr i;

	panel(wxWindow* p, const std::string& n, ext::instance_ptr i)
		: p(p), class_name(n), i(i)
	{
	}
};

typedef std::vector<panel> panels;
static panels g_panels;

void export_misc()
{
	bpl::def("quit", &quit);

	bpl::class_<baretorrent_version, boost::noncopyable>(
		"baretorrent_version", bpl::no_init)
		.def_readonly("major", &baretorrent_version::major)
		.def_readonly("minor", &baretorrent_version::minor)
		.def_readonly("maintenance", &baretorrent_version::maintenance)
		.def_readonly("tag", &baretorrent_version::tag)
		.def_readonly("build_date", &baretorrent_version::build_date)
		.def_readonly("string", &baretorrent_version::string);

	bpl::def("version", &version);
	bpl::def("make_loc_version", &make_loc_version);
}

void do_log(const std::string& what)
{
	log(calling_instance()) << what << "\n";
}

void do_log_error(const std::string& what)
{
	log_error(calling_instance()) << what << "\n";
}

void do_log_v(const std::string& what)
{
	log_v(calling_instance()) << what << "\n";
}

void do_log_vv(const std::string& what)
{
	log_vv(calling_instance()) << what << "\n";
}

void export_log()
{
	bpl::def("log", &do_log);
	bpl::def("log_error", &do_log_error);
	bpl::def("log_v", &do_log_v);
	bpl::def("log_vv", &do_log_vv);
}

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(
	torrent_start_function, start, 0, 1)

void export_torrent()
{
	const utf8_path& (torrent::*save_path)() const =
		&torrent::save_path;

	std::vector<int> (torrent::*torrent_priorities_getter)() const =
		&torrent::file_priorities;

	void (torrent::*torrent_priorities_setter)
		(const std::vector<int>&) = &torrent::file_priorities;

	std::vector<bt::announce_entry>
		(torrent::*torrent_trackers_getter)() const =
			&torrent::trackers;

	void (torrent::*torrent_trackers_setter)(
		const std::vector<bt::announce_entry>&) =
			&torrent::trackers;

	bpl::def("status_to_loc", &status_to_loc);

	bpl::scope s = bpl::class_<torrent, boost::noncopyable>(
			"torrent", bpl::no_init)
		// info
		.def("total_size", &torrent::total_size)
		.def("handle", &torrent::handle)
		// save_torrent_file
		// save_resume
		// save_state_file
		.def("downloaded", &torrent::downloaded)
		.def("added", &torrent::added)
		.def("errored", &torrent::errored)
		.def("error", &torrent::error)
		.def("start", &torrent::start,
			torrent_start_function(bpl::args("force")))
		.def("pause", &torrent::pause)
		.def("remove", &torrent::remove)
		.def("up", &torrent::up)
		.def("down", &torrent::down)
		.def("top", &torrent::top)
		.def("bottom", &torrent::bottom)
		.def("name", &torrent::name)
		.def("hash", &torrent::hash)
		.def("url", &torrent::url)
		.def("status", &torrent::status)
		.def("sha1_hash", &torrent::sha1_hash)
		.def("save_path", save_path,
			bpl::return_value_policy<bpl::copy_const_reference>())
		.def("position", &torrent::position)
		.def("file_progress", &torrent::file_progress)
		.def("file_priorities", torrent_priorities_setter)
		.def("file_priorities", torrent_priorities_getter)
		.def("trackers", torrent_trackers_setter)
		.def("trackers", torrent_trackers_getter)
		.def("add_tracker", &torrent::add_tracker)
		.def("peers", &torrent::peers)
		.def("torrent_status", &torrent::torrent_status,
			bpl::return_value_policy<bpl::copy_const_reference>())
		.def("eta", &torrent::eta);

	bpl::enum_<torrent::priorities>("priorities")
		.value("skip", torrent::skip_priority)
		.value("normal", torrent::normal_priority)
		.value("high", torrent::high_priority);

	bpl::enum_<torrent::statuses>("statuses")
		.value("none", torrent::no_status)
		.value("paused", torrent::paused_status)
		.value("stopped", torrent::stopped_status)
		.value("checking", torrent::checking_status)
		.value("resuming", torrent::resuming_status)
		.value("starting", torrent::starting_status)
		.value("queued", torrent::queued_status)
		.value("metadata", torrent::metadata_status)
		.value("stalled", torrent::stalled_status)
		.value("downloading", torrent::downloading_status)
		.value("seeding", torrent::seeding_status)
		.value("moving", torrent::moving_status)
		.value("removing", torrent::removing_status)
		.value("first_error", torrent::first_error_status)
		.value("bad_save_path", torrent::bad_save_path_status)
		.value("resume_failed", torrent::resume_failed_status)
		.value("move_failed", torrent::move_failed_status)
		.value("generic_error", torrent::generic_error_status);

	bpl::register_ptr_to_python<boost::shared_ptr<torrent> >();
}

manager* get_manager()
{
	app* a = dynamic_cast<app*>(wxTheApp);
	if (!a)
		return nullptr;
	
	return a->manager_ptr();
}

void export_manager()
{
	torrent_ptr (manager::*manager_get)(std::size_t i) =
		&manager::get;

	bpl::class_<manager, boost::noncopyable>("manager", bpl::no_init)
		.def("size", &manager::size)
		.def("empty", &manager::empty)
		.def("get", manager_get)
		.def("save_resume", &manager::save_resume)
		.def("add_torrent", &manager::add_torrent)
		.def("remove_torrent", &manager::remove_torrent)
		.def("session", &manager::session,
			bpl::return_value_policy<bpl::reference_existing_object>())
		.def("download_rate", &manager::download_rate)
		.def("upload_rate", &manager::upload_rate);
		//.def("connection", &manager::connection);

	bpl::def("manager", &get_manager,
		bpl::return_value_policy<bpl::reference_existing_object>());
}

void export_config()
{
	bpl::class_<config, boost::noncopyable>("config", bpl::no_init)
		.def("windows", &config::windows).staticmethod("windows")
		.def("apple", &config::apple).staticmethod("apple")
		.def("linux", &config::linux).staticmethod("linux")
		.def("posix", &config::posix).staticmethod("posix")
		.def("debug", &config::debug).staticmethod("debug")
		.def("target_is_utf8", &config::target_is_utf8)
			.staticmethod("target_is_utf8")
		.def("target_is_utf16", &config::target_is_utf16)
			.staticmethod("target_is_utf16")
		.def("x86", &config::x86).staticmethod("x86")
		.def("x64", &config::x64).staticmethod("x64");
}

void set_opt(const std::string& name, const utf8_string& value)
{
	instance_ptr i = calling_instance();
	if (!i)
		return;

	ext_opt(i->guid(), name, value);
}

utf8_string get_opt(const std::string& name)
{
	instance_ptr i = calling_instance();
	if (!i)
		return utf8_string();

	return ext_opt(i->guid(), name);
}

void add_opt(const std::string& name, const utf8_string& def)
{
	instance_ptr i = calling_instance();
	if (!i)
		return;

	add_ext_opt(i->guid(), name, def);
}

class dummy_paths_namespace {};

void export_utility()
{
	bpl::def("urlencode", &urlencode);
	bpl::def("urldecode", &urldecode);

	bpl::def("make_loc_size", &make_loc_size);
	bpl::def("make_loc_duration", &make_loc_duration);
	bpl::def("make_loc_duration_abbr", &make_loc_duration_abbr);
	bpl::def("make_loc_rate", &make_loc_rate);
	bpl::def("make_loc_time", &make_loc_time);
	bpl::def("make_loc_priority", &make_loc_priority);

	bpl::def("directory_separator", &directory_separator);

	bpl::def("natural_compare", &natural_compare);
	bpl::def("name_from_url", &name_from_url);
	
	bpl::def("set_opt", &set_opt);
	bpl::def("get_opt", &get_opt);
	bpl::def("add_opt", &add_opt);

	bpl::scope s = bpl::class_<
		dummy_paths_namespace, boost::noncopyable>(
			"paths", bpl::no_init)
		.def("conf", &paths::conf)
		//.def("conf_filename", &paths::conf_filename)
		.def("locales", &paths::locales)
		.def("locale_suffix", &paths::locale_suffix)
		.def("incomplete_suffix", &paths::incomplete_suffix)
		.def("torrent_file_suffix", &paths::torrent_file_suffix)
		.def("resume_file_suffix", &paths::resume_file_suffix)
		.def("state_file_suffix", paths::state_file_suffix)
		.def("session_filename", &paths::session_filename)
		.def("queue_filename", &paths::queue_filename);
}

std::string do_loc_function(
	const std::string& name,
	const std::string& a1="", const std::string& a2="",
	const std::string& a3="", const std::string& a4="")
{
	if (!has_loc(name))
	{
		log_error()
			<< "an extension is attempting to get the value of locale "
			<< "string '" << name << "', but that string doesn't "
			<< "exist; returning an empty string\n";

		return "";
	}

	try
	{
		return (locf(name)
			% utf8_string(a1) % utf8_string(a2)
			% utf8_string(a3) % utf8_string(a4)).str();
	}
	catch(std::exception& e)
	{
		log_error()
			<< "an extension is attempting to get the value of locale "
			<< "string '" << name << "', but is passing the wrong "
			<< "arguments: " << e.what() << "; returning an empty "
			<< "string\n";

		return "";
	}
}

BOOST_PYTHON_FUNCTION_OVERLOADS(do_loc_overloads, do_loc_function, 1, 5)

void export_locale()
{
	bpl::def("loc", &do_loc_function, do_loc_overloads());
	bpl::def("loc_error", &loc_error);
}
	
class dummy_platform_namespace {};

void export_platform()
{
	using namespace platform;

	bpl::scope c = bpl::class_<
		dummy_platform_namespace, boost::noncopyable>(
			"platform", bpl::no_init)
		.def("open_directory", &open_directory)
		.def("download_directory", &download_directory)
		.def("conf_directory", &conf_directory)
		.def("conf_directory_name" ,&conf_directory_name)

		.def("add_firewall_exception", &add_firewall_exception)
		.def("compile_target", &compile_target)
		.def("name", &name);
}

void add_status_impl(instance_ptr i, const std::string& f, bool greyed)
{
	const utf8_path p = i->path() / utf8_string(f);
	if (!p.exists())
	{
		log_error(i) << "icon " << p << " not found\n";
		return;
	}

	log(i) << "adding icon " << p << "\n";
	get_app().window().status().add_icon(i, p, greyed);
}

void remove_status_impl(instance_ptr i)
{
	get_app().window().status().remove_icon(i);
}

void add_status(const std::string& f, bool greyed)
{
	instance_ptr i = calling_instance();
	if (!i)
		return;

	get_app().window().async_call(
		boost::bind(add_status_impl, i, f, greyed));
}

void remove_status()
{
	instance_ptr i = calling_instance();
	if (!i)
		return;

	get_app().window().async_call(boost::bind(remove_status_impl, i));
}

void add_panel_impl(instance_ptr i, const std::string& class_name)
{
	python_lock lock;

	class failed {};

	try
	{
		log(i) << "is adding panel '" << class_name << "'\n";

		wxNotebook* tabs = get_app().window().tabs();

		bpl::object o = i->create_object(
			utf8_string(class_name), wx_to_bpl(tabs));

		if (!o || !o.ptr())
			throw failed();

		wxPanel* p = 0;
		bool r = wxPyConvertWrappedPtr(
			o.ptr(), reinterpret_cast<void**>(&p),
			wxString("wxWindow"));

		if (!r || !p)
		{
			log_error(i)
				<< "class '" << class_name << "' is not a wx.Window\n";

			throw failed();
		}

		utf8_string name = i->name();
		if (has_attr(o, utf8_string("tab_name")))
		{
			bpl::object no = o.attr("tab_name")();
			if (!no)
			{
				log_error(i) << "tab_name() failed\n";
			}
			else
			{
				const utf8_string temp = extract_string(no);
				if (temp.empty())
				{
					log_error(i)
						<< "tab_name() did not return a valid "
						<< "string\n";
				}
				else
				{
					name = temp;
				}
			}
		}

		tabs->InsertPage(
			tabs->GetPageCount(), p, to_wxstring(name));

		g_panels.push_back(panel(p, class_name, i));

		log(i) << "added a new panel '" << name << "'\n";
	}
	catch(bpl::error_already_set&)
	{
		log_error(i) << python_error() << "\n";
		PyErr_Clear();
	}
	catch(failed&)
	{
		log_error(i) << "failed to add panel\n";
	}
}

void add_panel(const std::string& class_name)
{
	instance_ptr i = calling_instance();
	if (!i)
		return;

	get_app().window().async_call(boost::bind(
		add_panel_impl, i, class_name));
}


void remove_panel_impl(instance_ptr i, const std::string& class_name)
{
	python_lock lock;

	log(i) << "removing panel '" << class_name << "'\n";

	panels::iterator itor;
	for (itor=g_panels.begin(); itor!=g_panels.end(); ++itor)
	{
		const panel& p = *itor;

		if (p.i == i && p.class_name == class_name)
		{
			wxNotebook* tabs = get_app().window().tabs();
			
			const int n = tabs->FindPage(p.p);
			if (n == wxNOT_FOUND)
				log_error(i) << "panel not found\n";
			else
				tabs->DeletePage(n);

			g_panels.erase(itor);
			break;
		}
	}
}

void remove_panel(const std::string& class_name)
{
	instance_ptr i = calling_instance();
	if (!i)
		return;

	get_app().window().async_call(boost::bind(
		remove_panel_impl, i, class_name));
}

void export_ui()
{
	bpl::def("add_status", add_status);
	bpl::def("remove_status", remove_status);
	bpl::def("add_panel", add_panel);
	bpl::def("remove_panel", remove_panel);
}

}	// namespace ext
}	// namespace baretorrent
