// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

// this is adapted from libtorrent; original copyright notice:
//
// Copyright Daniel Wallin 2006. Use, modification and distribution is
// subject to the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include "pch.h"
#include "../../pch.h"
#include "../python_wrapper.h"
#include "../../log.h"

#pragma warning(push, 1)

namespace libtorrent
{
bool operator==(const bt::announce_entry&, const bt::announce_entry&)
{
	// dummy: vector_indexing_suite requires operator==
	return false;
}

bool operator==(const bt::peer_info&, const bt::peer_info&)
{
	// dummy: vector_indexing_suite requires operator==
	return false;
}
}	// namespace libtorrent

void bind_utility();
void bind_fingerprint();
void bind_big_number();
void bind_session();
void bind_entry();
void bind_torrent_info();
void bind_unicode_string_conversion();
void bind_torrent_handle();
void bind_torrent_status();
void bind_session_settings();
void bind_version();
void bind_alert();
void bind_datetime();
void bind_torrent();
void bind_peer_info();
void bind_ip_filter();
void bind_magnet_uri();
void bind_converters();
void bind_create_torrent();
void bind_error_code();

void from_libtorrent()
{
    bind_error_code();
    bind_utility();
    bind_fingerprint();
    bind_big_number();
    bind_entry();
    bind_session();
    bind_torrent_info();
#if TORRENT_USE_WSTRING
    bind_unicode_string_conversion();
#endif
    bind_torrent_handle();
    bind_torrent_status();
    bind_session_settings();
    bind_version();
    bind_alert();
    bind_datetime();
    bind_torrent();
    bind_peer_info();
    bind_ip_filter();
    bind_magnet_uri();
    bind_converters();
    bind_create_torrent();
}

BARETORRENT_EXPORT void export_libtorrent_module()
{
	baretorrent::ext::python_lock lock;

	try
	{
		from_libtorrent();

		bpl::class_<std::vector<bt::announce_entry> >(
			"vector_of_bt_announce_entries")
				.def(bpl::vector_indexing_suite<
					std::vector<bt::announce_entry> >());

		bpl::class_<std::vector<bt::peer_info> >(
			"vector_of_bt_peer_infos")
				.def(bpl::vector_indexing_suite<
					std::vector<bt::peer_info> >());
	}
	catch(bpl::error_already_set&)
	{
		baretorrent::log_error()
			<< "could not inialize libtorrent python module, "
			<< baretorrent::ext::python_error() << "\n";

		throw;
	}
}
