// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_INFO_PANEL_INCLUDED
#define BARETORRENT_INFO_PANEL_INCLUDED

#include "manager.h"

namespace baretorrent
{

// this is the panel on the main window that displays information about
// the selected torrent
//
// each widget is paired with a "setter", which is a function that
// generates the text going on the widget when updating
//
class info_panel : public wxPanel
{
public:
	// creates the panel and its widgets
	//
	info_panel(wxWindow* parent);


	// empties all the widgets
	//
	void clear();

	// updates the widgets for the given torrents
	//
	void update(const torrents& ts);

private:
	// pairs a widget with a setter
	//
	struct setter
	{
		typedef boost::function <
			utf8_string (
				const torrent_ptr,
				const bt::torrent_status&,
				wxWindow*)>
					fun;
		
		wxStaticText* widget;
		fun f;
		
		setter(wxStaticText* w, fun f)
			: widget(w), f(f)
		{
		}
	};

	typedef std::vector<setter> setters;

	// download progress bar
	wxGauge* progress_;

	// torrent name; this is needed since different values are set
	// depending on multiple selection or metadata
	wxStaticText* name_;

	// list of widgets and setters
	setters setters_;


	// creates the various widgets
	//
	wxSizer* create_widgets();

	// makes a label widget with the given locale name
	//
	wxStaticText* make_text(const std::string& name, bool bold=false);

	// sets up the widgets for a single torrent
	//
	void single_torrent(const torrent_ptr t);

	// sets up the widgets for multiple torrents
	//
	void multiple_selection();
};

}	// namespace baretorrent

#endif	// BARETORRENT_INFO_PANEL_INCLUDED
