// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#include "pch.h"
#include "locale.h"
#include "utility.h"
#include "manager.h"
#include "options.h"

#if BARETORRENT_GTK
#include <glib.h>
#endif

namespace baretorrent
{

typedef std::map<std::string, boost::format> locale_map;

// current locale
static locale_map g_map;

// these are the default english strings
//
void load_english()
{
	// adds the name/value pair, asserts if that name already exists
#define s(n, v)										\
		bt_assert(g_map.find(n) == g_map.end());	\
		g_map[n] = boost::format(v)

	s("locale.name", "English");
	s("torrent.name.caption", "Name");

	s("torrent.progress.caption", "Progress");
	s("torrent.progress", "%1%%%");
		
	s("torrent.download_rate.caption", "DL");
	s("torrent.upload_rate.caption", "UL");

	s("torrent.add", "Add torrent");
	s("torrent.add.info",
			"This is a list of files, URLs or magnet links separated "
			"by newlines");
	s("torrent.add.invalid_files", "Invalid torrent files");
	s("torrent.add.invalid_files.title", "Error");
	s("torrent.add.path_error",
		"Save path is inaccessible, check the options");
	s("torrent.add.path_error.title", "Error");

	s("torrent.adding", "Adding %1%");
	s("torrent.duplicate", "Duplicate %1%");
	s("torrent.removed", "Removed %1%");
	s("torrent.moving", "Moving %1% to %2%");
	s("torrent.move_failed", "Failed to move %1% (%2%)");

	s("torrent.remove", "Remove torrent");
	s("torrent.remove.confirm",
		"Are you sure you want to remove this torrent?");
	s("torrent.remove.delete_files", "Also delete the files from disk");

	s("torrent.save_in", "Save in:");

	s("torrent.resume", "Resume");
	s("torrent.pause", "Pause");
	s("torrent.down", "Decrease priority");
	s("torrent.up", "Increase priority");
	s("torrent.top", "Top priority");
	s("torrent.bottom", "Bottom priority");

	s("torrent.status.caption", "Status");
	s("torrent.status.paused", "paused");
	s("torrent.status.stopped", "stopped");
	s("torrent.status.checking", "checking");
	s("torrent.status.starting", "starting");
	s("torrent.status.queued", "queued");
	s("torrent.status.metadata", "metadata");
	s("torrent.status.stalled", "stalled");
	s("torrent.status.downloading", "downloading");
	s("torrent.status.seeding", "seeding");
	s("torrent.status.moving", "moving");
	s("torrent.status.removing", "removing");
	s("torrent.status.resuming", "resuming");
	s("torrent.status.bad_save_path", "bad save path");
	s("torrent.status.generic_error", "error");
	s("torrent.status.resume_failed", "resume failed");
	s("torrent.status.move_failed_status", "move failed");

	s("torrent.file.priority", "Priority");
	s("torrent.file.skip", "skip");
	s("torrent.file.low", "low");
	s("torrent.file.normal", "normal");
	s("torrent.file.high", "high");
	s("torrent.file.progress.caption", "%%");
	s("torrent.file.progress", "%1%%%");

	s("torrent.position.caption", "#");
	s("torrent.hash.caption", "Hash");
	s("torrent.error.caption", "Error");
	s("torrent.time.active.caption", "Time active");
	s("torrent.time.finished.caption", "Time finished");
	s("torrent.time.seeding.caption", "Time seeding");
	s("torrent.time.added.caption", "Added on");
	s("torrent.time.completed.caption", "Completed on");
	s("torrent.time.last_seen", "Last seen complete on");
	s("torrent.total_size.caption", "Size");
	s("torrent.wanted_size.caption", "Selected size");
	s("torrent.full_copies.caption", "Full copies");
	s("torrent.peers.caption", "Peers");
	s("torrent.seeders.caption", "Seeders");
	s("torrent.leechers.caption", "Leechers");
	s("torrent.block_size.caption", "Block size");
	s("torrent.connections.caption", "Connections");
	s("torrent.upload_slots.caption", "Upload slots");
	s("torrent.max_connections.caption", "Max connections");
	s("torrent.queued_leechers.caption", "Queued leechers");
	s("torrent.queued_seeders.caption", "Queued seeders");
	
	s("torrent.trackers.caption", "Trackers");
	s("torrent.trackers.url", "URL");
	s("torrent.trackers.message", "Message");
	s("torrent.trackers.ok", "ok");
	s("torrent.trackers.updating", "updating...");
	s("torrent.trackers.start_sent", "start sent...");
	s("torrent.trackers.complete_sent", "complete sent...");
	s("torrent.peers.ip", "IP");

	s("torrent.download.payload_rate.caption", "Payload rate");
	s("torrent.download.rate.caption", "Rate");
	s("torrent.download.payload_session.caption", "Session");
	s("torrent.download.session.caption", "Session");
	s("torrent.download.all_time.caption", "All time");
	s("torrent.download.wasted.caption", "Wasted");
	s("torrent.download.time_since.caption", "Time since download");

	s("torrent.upload.payload_rate.caption", "Payload rate");
	s("torrent.upload.rate.caption", "Rate");
	s("torrent.upload.payload_session.caption", "Session");
	s("torrent.upload.session.caption", "Session");
	s("torrent.upload.all_time.caption", "All time");
	s("torrent.upload.wasted.caption", "Wasted");
	s("torrent.upload.time_since.caption", "Time since upload");
	s("torrent.upload.ratio.caption", "Ratio");

	s("torrent.eta.caption", "ETA");


	s("status.resuming_session", "Resuming session...");
	s("status.ready", "Ready");
	s("status.bad_ext",
		"It looks like extension %1% is misbehaving. Do you want to "
		"disable it?");
	s("status.remaining", "%1% remaining");
	s("status.checking_update", "Checking for updates...");
	s("status.checking_update_failed", "Could not check for updates");
	s("status.adding", "Adding... (%1%/%2%)");

	s("search.close", "Close");
	s("search.label", "Find:");

	s("queue_dialog.title", "Loading torrents");
	s("queue_dialog.start_label", "Loading...");
	s("queue_dialog.label", "Loading %1%/%2%");

	s("general", "General");
	s("swarm", "Swarm");
	s("network", "Network");
	s("download", "Download");
	s("upload", "Upload");
	s("times", "Times");
	s("never", "never");
	s("files", "Files");
	s("file", "File");
	s("multiple_selection", "multiple selection");
	s("multiple_paths", "multiple directories");
	s("updating", "updating");
	s("no_metadata", "no metadata yet");
	s("not_available", "N/A");
	s("unlimited", "unlimited");
	s("transfer_rate", "%1%/s");
	s("close", "Close");
	s("search", "Search");
	s("yes", "Yes");
	s("no", "No");
	s("cancel", "Cancel");
	s("ok", "OK");
	s("reset", "Reset");

	s("update.available",
		"A new version of baretorrent is available (%1%)");
	s("update.no", "Not now");

	s("assoc.label_other",
		"Make <b>baretorrent</b> the default torrent program "
		"instead of %1%.");
	s("assoc.label", "Make baretorrent the default torrent program.");
	s("assoc.yes", "Yes");
	s("assoc.no", "No");

	s("power.shutdown.label",
		"The computer will be shut down once the downloads are "
		"completed.");
	s("power.shutdown.title", "Shutting down");
	s("power.shutdown.warning",
		"The computer will be shut down in %1%.");
	s("power.suspend.label",
		"The computer will be put to sleep once the downloads are "
		"completed");
	s("power.suspend.title", "Suspending");
	s("power.suspend.warning",
		"The computer will be put to sleep in %1%.");
	s("power.hibernate.title", "Hibernating");
	s("power.hibernate.label",
		"The computer will hibernate once the downloads are "
		"completed");
	s("power.hibernate.warning",
		"The computer will hibernate in %1%.");
	s("power.cancel", "Cancel");

	s("pcon.title", "Python console");

	s("priority", "Priority");
	s("priority.skip", "Skip");
	s("priority.normal", "Normal");
	s("priority.high", "High");

	s("byte", "%1% bytes");
	s("byte.abbr", "%1% B");
	s("kibi", "%1% kibibytes");
	s("kibi.abbr", "%1% KiB");
	s("mebi", "%1% mebibytes");
	s("mebi.abbr", "%1% MiB");
	s("gibi", "%1% gibibytes");
	s("gibi.abbr", "%1% GiB");
	s("tebi", "%1% tebibytes");
	s("tebi.abbr", "%1% TiB");

	s("time.second", "%1% second");
	s("time.seconds", "%1% seconds");
	s("time.minute", "%1% minute");
	s("time.minutes", "%1% minutes");
	s("time.hour", "%1% hour");
	s("time.hours", "%1% hours");
	s("time.day", "%1% day");
	s("time.days", "%1% days");
	s("time.full", "%%c");

	s("time.second.abbr", "%1% s");
	s("time.minute.abbr", "%1% m");
	s("time.hour.abbr", "%1% h");
	s("time.day.abbr", "%1% d");
		
	s("connections", "Connections");
	s("connections.trackers", "Trackers (%1%)");
	s("connections.trackers.add", "Add tracker");
	s("connections.trackers.remove", "Remove tracker");
	s("connections.tracker.add.caption",
		"This is a list of tracker URLs separated by newlines");
	s("connections.tracker.add.title", "Add a tracker");
	s("connections.peers", "Peers (%1%)");
	s("connections.dht_pex_lsd", "DHT/PEX/LSD");
	s("connections.no_remote", "No incoming connections");

	s("category.all", "All torrents");
	s("category.downloading", "Downloading");
	s("category.uploading", "Seeding");
	s("category.active", "Active torrents");
	s("category.inactive", "Inactive torrents");

	s("proxy.socks4", "SOCKS4");
	s("proxy.socks5", "SOCKS5");
	s("proxy.http", "HTTP");

	s("options.title", "Preferences");

	s("options.general", "General");
	s("options.language", "Language (a restart is required)");
	s("options.minimized", "Start minimized");
	s("options.notification", "Minimize to notification area");
	s("options.update", "Check for updates");

	s("options.torrents", "Torrents");

	s("options.add_top", "Add torrents on top");
	s("options.append_extension",
		"Append %1% extension to incomplete files");

	s("options.paths", "Paths");
	s("options.save_complete_in", "Save completed in");
	s("options.save_incomplete_in", "Keep incomplete in");
	s("options.save_complete_torrent_file_in",
		"Save completed torrent file in");
	s("options.save_incomplete_torrent_file_in",
		"Keep incomplete torrent file in");
	s("options.changing_paths",
		"Changing paths will only apply to new torrents");
	s("options.auto_load", "Automatically load torrent files from");


	s("options.network", "Network");

	s("options.queue", "Queue");
	s("options.downloads_queued", "Active downloads");
	s("options.upload_queued", "Active uploads");
	s("options.stop_seed", "Stop seeding after reaching ratio (%%)");

	s("options.speeds", "Speeds");
	s("options.max_download_rate", "Maximum download rate (KiB/s)");
	s("options.max_upload_rate", "Maximum upload rate (KiB/s)");

	s("options.connections", "Connections");
	s("options.incoming_port", "Listening port");
	s("options.firewall_exception", "Add exception to system firewall");
	s("options.num_connections", "Number of connections");
	s("options.connections_per_torrent", "Connections per torrent");
	
	s("options.proxy", "Proxy");
	s("options.proxy_enabled", "Proxy");
	s("options.proxy.host", "Host (:port)");
	s("options.proxy.type", "Type");
	s("options.proxy.username", "Username");
	s("options.proxy.password", "Password");
	

	s("options.extensions", "Extensions");
	s("options.extensions.name", "Name");
	s("options.extensions.version", "Version");
	s("options.extensions.author", "Author");
	s("options.extensions.description", "Description");
	s("options.extensions.by", "By %1%");
	s("options.extensions.disable", "Disable");
	s("options.extensions.enable", "Enable");
	s("options.extensions.uninstall", "Uninstall");
	s("options.extensions.options", "Options");
	s("options.extensions.options.title", "%1%");
	s("options.extensions.failed",
		"The extension failed to load. Contact the author with the "
		"following information.");
	s("options.extensions.failed.title", "Extension error");
	s("options.extensions.uninstall.caption",
		"This will uninstall the %1% extension.");
	s("options.extensions.uninstall.title", "Uninstall extension");
	s("options.extensions.install.bad",
		"This is not a valid extension.");
	s("options.extensions.install.failed",
		"Installation of the extension failed. Contact the author.");
	s("options.extensions.install.failed.title",
		"Installation failed");
	s("options.extensions.install.require",
		"This extension requires at least version %1% of baretorrent (you have %2%)");
	s("options.extensions.reload", "Reload");

	s("extension.install.title", "Install extension");
	s("extension.install", "Install");
	s("extension.update", "Update");
	s("extension.already_installed",
		"This extension is already installed");
	s("extension.can_update",
		"An earlier version of this extension is already installed");
	s("extension.dev", "This is a dev extension");
	s("extension.reload_failed",
		"There was an error while reloading the extension");
	s("extension.reload_failed.title", "Reloading error");
	s("extension.error.title", "Extension error");
	s("extension.install_success", "Installed");
	s("extension.install_error", "Installation failed");

	s("ext_browser.title", "Extension browser");
	s("ext_browser.search_label", "Search extensions");
	s("ext_browser.search", "Search");

	s("options.advanced", "Advanced");
	s("options.advanced.name", "Name");
	s("options.advanced.value", "Value");
	s("options.advanced.warning",
		"Close this window after any modifications.");
	s("options.advanced.change.message", "Enter the new value");
	s("options.advanced.change.caption", "Change option");

	s("window.title", "baretorrent (DL: %1%; UL: %2%)");
	s("window.title.no_python", "extensions disabled");
	s("window.notification_title", program_name().std_string());
	s("window.open", "Open");

	s("browse", "...");
	s("browse.open_file", "Open file");
	s("browse.open_directory", "Open directory");

	s("wildcard.torrents",
		"Torrent files (*.torrent)|*.torrent|All files (*.*)|*");
	s("wildcard.extensions",
		"Extensions (*.btx)|*.btx|All files (*.*)|*");

	s("pcon.extensions", "Extensions");

	s("menu.file", "&File");
	s("menu.file.add", "&Add...");
	s("menu.file.add.help", "Adds one or more torrents");
	s("menu.file.quit", "&Exit");
	s("menu.file.quit.help", "Exits baretorrent");

	s("menu.view", "&View");
	s("menu.view.all", "&All torrents");
	s("menu.view.all.help", "Displays all torrents");
	s("menu.view.downloading", "&Downloading");
	s("menu.view.downloading.help",
		"Displays torrents that are downloading");
	s("menu.view.uploading", "&Seeding");
	s("menu.view.uploading.help",
		"Displays torrents that are seeding");
	s("menu.view.active", "A&ctive");
	s("menu.view.active.help",
		"Displays torrents that are either downloading or seeding");
	s("menu.view.inactive", "&Inactive");
	s("menu.view.inactive.help", "Displays inactive torrents");
	s("menu.view.search", "&Find");
	s("menu.view.search.help", "Finds torrents in the list");

	s("menu.torrent", "&Torrent");
	s("menu.torrent.resume", "&Resume");
	s("menu.torrent.resume.help", "Resumes the selected torrents");
	s("menu.torrent.pause", "&Pause");
	s("menu.torrent.pause.help", "Pauses the selected torrents");
	s("menu.torrent.up", "&Increase Priority");
	s("menu.torrent.up.help",
		"Increases the priority of the selected torrents");
	s("menu.torrent.down", "&Decrease Priority");
	s("menu.torrent.down.help",
		"Decreases the priority of the selected torrents");
	s("menu.torrent.top", "&Top Priority");
	s("menu.torrent.top.help",
		"Moves the selected torrents to the top");
	s("menu.torrent.bottom", "&Bottom Priority");
	s("menu.torrent.bottom.help",
		"Moves the selected torrents to the bottom");
	s("menu.torrent.remove", "R&emove...");
	s("menu.torrent.remove.help", "Removes the selected torrents");
	s("menu.torrent.open_directory", "Open containing directory");
	s("menu.torrent.open_directory.help",
		"Opens the directory the files are saved");
	s("menu.torrent.recheck", "Recheck");
	s("menu.torrent.recheck.help", "Verifies the selected torrent");
	s("menu.torrent.move_storage", "Change directory...");
	s("menu.torrent.move_storage.help",
		"Changes the download directory of all the selected torrents");

	s("menu.tools", "T&ools");
	s("menu.tools.install_extension", "Install extension...");
	s("menu.tools.install_extension.help", "Installs a new extension");
	s("menu.tools.pcon", "Python console");
	s("menu.tools.pcon.help", "Toggles the python console");
	s("menu.tools.options", "&Preferences...");
	s("menu.tools.options.help", "Displays the preferences");
	s("menu.tools.after_downloads", "After downloads");
	s("menu.tools.after_downloads.help",
		"Behaviour when all torrents are downloaded");
	s("menu.tools.after_downloads.shutdown", "Shutdown");
	s("menu.tools.after_downloads.shutdown.help",
		"Shuts down this machine");
	s("menu.tools.after_downloads.suspend", "Sleep");
	s("menu.tools.after_downloads.suspend.help",
		"Puts the computer in low power mode");
	s("menu.tools.after_downloads.hibernate", "Hibernate");
	s("menu.tools.after_downloads.hibernate.help",
		"Hibernates this machine");
	s("menu.tools.after_downloads.linux_pm",
		"Install the pm-utils and upower packages");
	s("menu.tools.after_downloads.linux_pm.help",
		"Install the pm-utils and upower packages");

	s("menu.help", "&Help");
	s("menu.help.log", "&Log");
	s("menu.help.log.help", "Toggles the log window");
	s("menu.help.about", "&About...");
	s("menu.help.about.help", "Shows the program version");
	
	s("dialog.error", "Error: %1%");
	s("dialog.error_title", "Error");

	s("log.title", "Log");
	s("log.time", "Time");
	s("log.text", "Message");
	s("log.level", "Level:");
	s("log.level.error", "Errors");
	s("log.level.normal", "Normal");
	s("log.level.verbose", "Verbose");
	s("log.level.very_verbose", "Very verbose");
	s("log.level.ext", "Extensions only");
	s("log.extensions", "Extensions");
	s("log.extensions.name", "Name");
	s("log.clear", "Clear log");
	s("log.save", "Save log...");
	s("log.save.title", "Save log file");

	s("about.title", "About baretorrent");
	s("about.version", "%1%, built %2%");
	s("about.target", "Compiled on %1%");
	s("about.platform", "Running on %1%");


	s("option.bt.update",
		"Automatically checks for updates; 0 or 1");
	s("option.bt.update_delay",
		"Number of seconds between updates");
	s("option.bt.update_on_startup",
		"Checks for updates on startup if bt.update is set; 0 or 1");
	s("option.bt.update_url",
		"URL to connect to when checking for updates");

	s("option.bt.assoc",
		"Checks whether baretorrent is the default torrent program on "
		"startup");

	s("option.bt.ui.language",
		"The filename without extension in the locale directory");
	s("option.bt.ui.minimized",
		"Whether to start minimized; 0 or 1");
	s("option.bt.ui.minimize_notification",
		"Whether to minimize to the notification area; 0 or 1");
	s("option.bt.ui.main_window.bounds",
		"Main window bounds; whitespace-separated \"x y w y\"");
	s("option.bt.ui.main_window.maximized",
		"Whether the main window was maximized when closed; 0 or 1");
	s("option.bt.ui.window_splitter",
		"Position of the splitter on the main window; integer");
	s("option.bt.ui.connections_splitter",
		"Position of the splitter on the connections panel; integer");
	s("option.bt.ui.main_list.widths",
		"List of column widths in the torrent list; whitespace-"
		"separated list of integers");
	s("option.bt.ui.main_list.sort",
		"Two integers for the index of the sort column (positive "
		"integer) and direction (-1 or 1) in the torrent list");
	s("option.bt.ui.peers_list.sort",
		"Two integers for the index of the sort column (positive "
		"integer) and direction (-1 or 1) in the peers list");
	s("option.bt.ui.peers_list.widths",
		"List of column widths in the peers list; whitespace-separated "
		"list of integers");
	s("option.bt.ui.trackers_list.sort",
		"Two integers for the index of the sort column (positive "
		"integer) and direction (-1 or 1) in the trackers list");
	s("option.bt.ui.trackers_list.widths",
		"List of column widths in the trackers list; whitespace-"
		"separated list of integers");
	s("option.bt.ui.ext_list.sort",
		"Two integers for the index of the sort column (positive "
		"integer) and direction (-1 or 1) in the extensions list");
	s("option.bt.ui.ext_list.widths",
		"List of column widths in the extensions list; whitespace-"
		"separated list of integers");
	s("option.bt.ui.ext_splitter",
		"Position of the splitter in the extensions tab; integer");
	s("option.bt.ui.advanced_list.sort",
		"Two integers for the index of the sort column (positive "
		"integer) and direction (-1 or 1) in the advanced options "
		"list");
	s("option.bt.ui.advanced_list.widths",
		"List of column widths in the advanced list; whitespace-"
		"separated list of integers");
	s("option.bt.ui.advanced_splitter",
		"Position of the splitter in the advanced tab; integer");
	s("option.bt.ui.extension_time",
		"Time in milliseconds after which a call into an extension is "
		"cancelled");
	s("option.bt.ui.add_from_clipboard",
		"Pastes the content of the clipboard if it makes sense when "
		"adding torrents; 0 or 1");
	s("option.bt.ui.log", "Whether the log window is visible; 0 or 1");
	s("option.bt.ui.log.split",
		"Position of the splitter in the log window; integer");
	s("option.bt.ui.log_ext.sort",
		"Two integers for the index of the sort column (positive "
		"integer) and direction (-1 or 1) in the extensions list of "
		"the log window");
	s("option.bt.ui.log_ext.widths",
		"List of column widths in the extensions list of the log "
		"window; whitespace-separated list of integers");
	s("option.bt.ui.log_list.sort",
		"Two integers for the index of the sort column (positive "
		"integer) and direction (-1 or 1) in the message list of the "
		"log window");
	s("option.bt.ui.log_list.widths",
		"List of column widths in the message list of the log window; "
		"whitespace-separated list of integers");
	s("option.bt.ui.log_window.bounds",
		"Log window bounds; whitespace-separated \"x y w y\"");
	s("option.bt.ui.log_window.maximized",
		"Whether the log window was maximized when closed; 0 or 1");
	s("option.bt.ui.remove_confirm",
		"Whether to show the confirmation dialog when deleting "
		"torrents; 0 or 1. If 0, bt.ui.remove_keeps_files is used "
		"to determine whether files are kept or not.");
	s("option.bt.ui.remove_keeps_files",
		"The default value of the checkbox in the confirmation "
		"dialog when removing torrents. If the dialog is disabled "
		"(see bt.ui.remove_confirm), determines whether files are "
		"kept or not when removing torrents. 0 will remove files and "
		"1 will keep them.");
	s("option.pcon.enabled",
		"Enables the python console; 0 or 1");
	s("option.pcon.visible",
		"Whether the python console is visible; 0 or 1");
	s("option.pcon.max_history",
		"Maximum number of commands in the history; -1 for infinite");
	
	s("option.bt.torrent.add_top",
		"Whether to add new torrents on top; 0 or 1");
	s("option.bt.torrent.append_extension",
		"Whether a suffix is added to incomplete files; 0 or 1");
	s("option.bt.torrent.auto_load",
		"The directory from which to auto-load torrent files");
	s("option.bt.torrent.auto_load_enabled",
		"Whether auto-load is enabled; 0 or 1");
	
	s("option.bt.torrent.active_downloads",
		"Maximum number of torrents downloading; positive integer");
	s("option.bt.torrent.active_downloads_enabled",
		"Whether the maximum downloading torrents is enabled; 0 or 1");
	s("option.bt.torrent.active_uploads",
		"Maximum number of torrents seeding; positive integer");
	s("option.bt.torrent.active_uploads_enabled",
		"Whether the maximum seeding torrents is enabled; 0 or 1");
	s("option.bt.torrent.seed_ratio",
		"Ratio at which a torrent stops seeding; integer (200 would "
		"upload what was downloaded twice)");
	s("option.bt.torrent.max_download_enabled",
		"Whether the maximum download rate is enabled; 0 or 1");
	s("option.bt.torrent.max_download",
		"Maximum download rate in kibibytes per second; integer");
	s("option.bt.torrent.max_upload_enabled",
		"Whether the maximum upload rate is enabled; 0 or 1");
	s("option.bt.torrent.max_upload",
		"Maximum upload rate in kibibytes per second; integer");
	s("option.bt.torrent.default_trackers",
		"A list of trackers to automatically add to new torrents; "
		"whitespace-separated list of URLs");
	s("option.bt.torrent.auto_active_limit",
		"Whether bt.session.active_limit is automatically set using "
		"bt.session.active_uploads + bt.session.active_downloads; "
		"0 or 1");
	s("option.bt.torrent.stalled_rate",
		"Speed in bytes/s below which a torrent is considered stalled; "
		"this only affects the user interface");
	s("option.bt.torrent.storage_mode",
		"Whether to preallocate files or use sparse files; one of "
		"'allocate' or 'sparse'");
	
	s("option.bt.network.listen",
		"Two integers giving the inclusive range of ports to try or "
		"an empty value to use a random port");

	s("option.bt.system.firewall_exception",
		"Whether to add the program to the system firewall (if "
		"present); 0 or 1 -- **not implemented**");
	s("option.bt.network.num_connections",
		"Maximum global number of connections; positive integer");
	s("option.bt.network.connections_per_torrent",
		"Maximum number of connections per torrent; positive integer");
	s("option.bt.network.connections_per_torrent_enabled",
		"Whether the maximum number of connections per torrent is "
		"enabled; 0 or 1");
	s("option.bt.network.proxy_enabled",
		"Whether the proxy is enabled; 0 or 1");
	s("option.bt.network.proxy", "Proxy URL");
	s("option.bt.network.proxy_type", "Proxy type; one of \"SOCKS4\", "
		"\"SOCKS5\", \"HTTP\" or an empty value");
 
	s("option.bt.network.proxy_username", "Username for the proxy");
	s("option.bt.network.proxy_password",
		"Password for the proxy, saved in clear text");
		
	s("option.bt.path.complete_save_path",
		"Path where torrents are saved; if "
		"bt.path.incomplete_save_path is set and enabled, path where "
		"completed torrents are saved");
	s("option.bt.path.incomplete_save_path",
		"Path where incomplete torrents are saved");
	s("option.bt.path.complete_torrent_file_path",
		"Path where a copy of the torrent file is saved for the user; "
		"if bt.path.incomplete_torrent_file_path is not empty, path "
		"where a copy of a completed torrent file is saved; disabled "
		"if empty");
	s("option.bt.path.incomplete_torrent_file_path",
		"Path where incomplete torrent files are saved for the user; "
		"disabled if empty");
		
	s("option.bt.path.incomplete_save_path_enabled",
		"Whether bt.path.incomplete_save_path is enabled");
	s("option.bt.path.complete_torrent_file_path_enabled",
		"Whether bt.path.complete_torrent_file_path is enabled");
	s("option.bt.path.incomplete_torrent_file_path_enabled",
		"Whether bt.path.incomplete_torrent_file_path is enabled");
	s("option.bt.paths.conf_filename",
		"Filename without path of the internal configuration file");
	s("option.bt.paths.backup_conf_suffix",
		"Suffix added to the configuration file when doing a backup on "
		"startup");
	s("option.bt.paths.bad_conf_suffix",
		"Suffix added to the configuration file when it cannot be "
		"parsed correctly");
	s("option.bt.paths.locales",
		"Path where locale files are stored");
	s("option.bt.paths.locale_suffix",
		"Suffix of the locale files added to the names");
	s("option.bt.paths.incomplete_suffix",
		"Suffix of the incomplete torrent files if "
		"bt.torrent.append_extension is enabled");
	s("option.bt.paths.torrent_file_suffix",
		"Suffix of the torrent files saved for the user");
	s("option.bt.paths.resume_file_suffix",
		"Suffix of the internal resume files");
	s("option.bt.paths.state_file_suffix",
		"Suffix of the internal state files");
	s("option.bt.paths.session_filename",
		"Suffix of the internal session file");
	s("option.bt.paths.queue_filename",
		"Filename of the queue file");

	s("option.bt.pe.out_enc_policy",
		"Encryption policy for outgoing connections; one of "
		"\"forced\", \"enabled\" or \"disabled\" "
		"(http://www.rasterbar.com/products/libtorrent/manual.html#pe-settings)");
	s("option.bt.pe.in_enc_policy",
		"Encryption policy for incoming connections; one of "
		"\"forced\", \"enabled\" or \"disabled\" "
		"(http://www.rasterbar.com/products/libtorrent/manual.html#pe-settings)");
	s("option.bt.pe.allowed_enc_level",
		"Encryption level of the connections; one of \"plaintext\", "
		"\"rc4\" or \"both\" (http://www.rasterbar.com/products/libtorrent/manual.html#pe-settings)");
	s("option.bt.pe.prefer_rc4",
		"Whether rc4 encryption is preferred "
		"(http://www.rasterbar.com/products/libtorrent/manual.html#pe-settings)");

	s("option.bt.power_timeout",
		"Time in seconds that the shutdown warning is displayed before "
		"actually shutting down the machine");

	s("option.bt.version",
		"This is updated when baretorrent is first started with "
		"another version. It is used to update or change options if "
		"necessary during the update process.");
	s("option.bt.peer_id",
		"The peer id sent to trackers; this uniquely identifies the "
		"client and version. Either \"BE\" for baretorrent's id and "
		"version or an empty string for libtorrent's default. "
		"Changing this requires a restart.");
	
	s("error.none", "no error");
	s("error.db",
		"There was an error accessing the configuration database");

	// libtorrent error codes
	s("bt.error.1",
		"torrent file collides with file from another torrent");
	s("bt.error.2", "hash check failed");
	s("bt.error.3", "torrent file is not a dictionary");
	s("bt.error.4",
		"missing or invalid 'info' section in torrent file");
	s("bt.error.5", "'info' entry is not a dictionary");
	s("bt.error.6",
		"invalid or missing 'piece length' entry in torrent file");
	s("bt.error.7", "missing name in torrent file");
	s("bt.error.8",
		"invalid 'name' of torrent (possible exploit attempt)");
	s("bt.error.9", "invalid length of torrent");
	s("bt.error.10", "failed to parse files from torrent file");
	s("bt.error.11",
		"invalid or missing 'pieces' entry in torrent file");
	s("bt.error.12",
		"incorrect number of piece hashes in torrent file");
	s("bt.error.13", "too many pieces in torrent");
	s("bt.error.14", "invalid metadata received from swarm");
	s("bt.error.15", "invalid bencoding");
	s("bt.error.16", "no files in torrent");
	s("bt.error.17", "invalid escaped string");
	s("bt.error.18", "session is closing");
	s("bt.error.19", "torrent already exists in session");
	s("bt.error.20", "invalid torrent handle used");
	s("bt.error.21", "invalid type requested from entry");
	s("bt.error.22", "missing info-hash from URI");
	s("bt.error.23", "file too short");
	s("bt.error.24", "unsupported URL protocol");
	s("bt.error.25", "failed to parse URL");
	s("bt.error.26", "peer sent 0 length piece");
	s("bt.error.27", "parse failed");
	s("bt.error.28", "invalid file format tag");
	s("bt.error.29", "missing info-hash");
	s("bt.error.30", "mismatching info-hash");
	s("bt.error.31", "invalid hostname");
	s("bt.error.32", "invalid port");
	s("bt.error.33", "port blocked by port-filter");
	s("bt.error.34", "expected closing ] for address");
	s("bt.error.35", "destructing torrent");
	s("bt.error.36", "timed out");
	s("bt.error.37", "upload to upload connection");
	s("bt.error.38", "uninteresting upload-only peer");
	s("bt.error.39", "invalid info-hash");
	s("bt.error.40", "torrent paused");
	s("bt.error.41",
		"'have'-message with higher index than the number of pieces");
	s("bt.error.42", "bitfield of invalid size");
	s("bt.error.43", "too many piece requests while choked");
	s("bt.error.44", "invalid piece packet");
	s("bt.error.45", "out of memory");
	s("bt.error.46", "torrent aborted");
	s("bt.error.47", "connected to ourselves");
	s("bt.error.48", "invalid piece size");
	s("bt.error.49", "timed out: no interest");
	s("bt.error.50", "timed out: inactivity");
	s("bt.error.51", "timed out: no handshake");
	s("bt.error.52", "timed out: no request");
	s("bt.error.53", "invalid choke message");
	s("bt.error.54", "invalid unchoke message");
	s("bt.error.55", "invalid interested message");
	s("bt.error.56", "invalid not-interested message");
	s("bt.error.57", "invalid request message");
	s("bt.error.58", "invalid hash list");
	s("bt.error.59", "invalid hash piece message");
	s("bt.error.60", "invalid cancel message");
	s("bt.error.61", "invalid dht-port message");
	s("bt.error.62", "invalid suggest piece message");
	s("bt.error.63", "invalid have-all message");
	s("bt.error.64", "invalid have-none message");
	s("bt.error.65", "invalid reject message");
	s("bt.error.66", "invalid allow-fast message");
	s("bt.error.67", "invalid extended message");
	s("bt.error.68", "invalid message");
	s("bt.error.69", "sync hash not found");
	s("bt.error.70", "unable to verify encryption constant");
	s("bt.error.71", "plaintext mode not provided");
	s("bt.error.72", "rc4 mode not provided");
	s("bt.error.73", "unsupported encryption mode");
	s("bt.error.74", "peer selected unsupported encryption mode");
	s("bt.error.75", "invalid encryption pad size");
	s("bt.error.76", "invalid encryption handshake");
	s("bt.error.77", "incoming encrypted connections disabled");
	s("bt.error.78", "incoming regular connections disabled");
	s("bt.error.79", "duplicate peer-id");
	s("bt.error.80", "torrent removed");
	s("bt.error.81", "packet too large");
	s("bt.error.82", "error 82");
	s("bt.error.83", "HTTP error");
	s("bt.error.84", "missing location header");
	s("bt.error.85", "invalid redirection");
	s("bt.error.86", "redirecting");
	s("bt.error.87", "invalid HTTP range");
	s("bt.error.88", "missing content-length");
	s("bt.error.89", "banned by IP filter");
	s("bt.error.90", "too many connections");
	s("bt.error.91", "peer banned");
	s("bt.error.92", "stopping torrent");
	s("bt.error.93", "too many corrupt pieces");
	s("bt.error.94", "torrent is not ready to accept peers");
	s("bt.error.95", "peer is not properly constructed");
	s("bt.error.96", "session is closing");
	s("bt.error.97", "optimistic disconnect");
	s("bt.error.98", "torrent finished");
	s("bt.error.99", "no router found");
	s("bt.error.100", "metadata too large");
	s("bt.error.101", "invalid metadata request");
	s("bt.error.102", "invalid metadata size");
	s("bt.error.103", "invalid metadata offset");
	s("bt.error.104", "invalid metadata message");
	s("bt.error.105", "pex message too large");
	s("bt.error.106", "invalid pex message");
	s("bt.error.107", "invalid lt_tracker message");
	s("bt.error.108",
		"pex messages sent too frequent (possible attack)");
	s("bt.error.109", "torrent has no metadata");
	s("bt.error.110", "invalid dont-have message");
	s("bt.error.111", "SSL connection required");
	s("bt.error.112", "invalid SSL certificate");

	// natpmp errors
	s("bt.error.120", "unsupported protocol version");
	s("bt.error.121",
		"not authorized to create port map "
		"(enable NAT-PMP on your router)");
	s("bt.error.122", "network failure");
	s("bt.error.123", "out of resources");
	s("bt.error.124", "unsupported opcode");

	// fastresume errors
	s("bt.error.130", "missing or invalid 'file sizes' entry");
	s("bt.error.131", "no files in resume data");
	s("bt.error.132", "missing 'slots' and 'pieces' entry");
	s("bt.error.133", "mismatching number of files");
	s("bt.error.134", "mismatching file size");
	s("bt.error.135", "mismatching file timestamp");
	s("bt.error.136", "not a dictionary");
	s("bt.error.137", "invalid 'blocks per piece' entry");
	s("bt.error.138", "missing slots list");
	s("bt.error.139", "file has more slots than torrent");
	s("bt.error.140", "invalid entry type in slot list");
	s("bt.error.141", "invalid piece index in slot list");
	s("bt.error.142", "pieces needs to be reordered");

	// HTTP errors
	s("bt.error.150", "Invalid HTTP header");
	s("bt.error.151", "missing Location header in HTTP redirect");
	s("bt.error.152", "failed to decompress HTTP response");

	// i2p errors
	s("bt.error.160", "no i2p router is set up");

	// tracker errors
	s("bt.error.170", "scrape not available on tracker");
	s("bt.error.171", "invalid tracker response");
	s("bt.error.172", "invalid peer dictionary entry");
	s("bt.error.173", "tracker sent a failure message");
	s("bt.error.174", "missing or invalid 'files' entry");
	s("bt.error.175", "missing or invalid 'hash' entry");
	s("bt.error.176", "missing or invalid 'peers' and 'peers6' entry");
	s("bt.error.177", "udp tracker response packet has invalid size");
	s("bt.error.178", "invalid transaction id in udp tracker response");
	s("bt.error.179", "invalid action field in udp tracker response");

	// bdecode errors
	s("bt.error.190", "expected string in bencoded string");
	s("bt.error.191", "expected colon in bencoded string");
	s("bt.error.192", "unexpected end of file in bencoded string");
	s("bt.error.193",
		"expected value (list, dict, int or string) "
		"in bencoded string");
	s("bt.error.194", "bencoded nesting depth exceeded");
	s("bt.error.195", "bencoded item count limit exceeded");

	// upnp
	s("bt.error.402", "invalid argument");
	s("bt.error.501", "the request failed");
	s("bt.error.714", "value not in array");
	s("bt.error.715", "source ip cannot be wildcarded");
	s("bt.error.716", "external port cannot be wildcarded");
	s("bt.error.717", "error 717");
	s("bt.error.718", "port mapping conflict");
	s("bt.error.724", "internal port must match external");
	s("bt.error.725", "only permanent leases supported");
	s("bt.error.726", "remote host must be wildcard");
	s("bt.error.727", "external port must be wildcard");
#undef s
}

std::size_t iconv_wrapper(
	iconv_t cd, const char** inbuf, size_t* inbytesleft,
	char** outbuf, size_t* outbytesleft)
{
#if BARETORRENT_WINDOWS
	return iconv(cd, inbuf, inbytesleft, outbuf, outbytesleft);
#else
	return iconv(
		cd, const_cast<char**>(inbuf), inbytesleft,
		outbuf, outbytesleft);
#endif
}

bool has_loc(const std::string& name)
{
	return (g_map.find(name) != g_map.end());
}

boost::format locf(const std::string& name)
{
	locale_map::iterator itor = g_map.find(name);
	bt_assert(itor != g_map.end());
	
	boost::format f;
	if (itor == g_map.end())
	{
		// this will fail if the format was expected to have a
		// positional fields, but hey
		log_error() << "unknown locale string '" << name << "'\n";
		f = boost::format("*" + name + "*");
	}

	f = itor->second;

	f.exceptions(
		boost::io::all_error_bits ^ boost::io::too_many_args_bit);

	return f;
}

utf8_string loc(const std::string& name)
{
	return utf8_string(locf(name).str());
}

wxString loc_wx(const std::string& name)
{
	const boost::format f = locf(name);
	const std::string s = f.str();

	return to_wxstring(utf8_string(s));
}

utf8_string loc_error(boost::system::error_code e)
{
	utf8_string s;

	if (e.category() == bt::get_libtorrent_category())
	{
		// the libtorrent errors are localized by baretorrent

		std::ostringstream oss;
		oss << "bt.error." << e.value();

		locale_map::iterator itor = g_map.find(oss.str());
		
		if (itor == g_map.end())
		{
			log_error()
				<< "invalid error code " << e.value()
				<< " (" << e.message() << ") from libtorrent\n";

#if BARETORRENT_WINDOWS
			const byte_array ba = convert(
				e.message().c_str(), e.message().size(),
				current_locale_encoding, utf8_encoding);

			if (!ba.empty())
				s = utf8_string(ba.data());
#else
			s = utf8_string(e.message());
#endif
		}
		else
		{
			s = utf8_string(itor->second.str());
		}
	}
	else
	{
		// boost uses the platform to localize messages
#if BARETORRENT_WINDOWS
		const byte_array ba = convert(
			e.message().c_str(), e.message().size(),
			current_locale_encoding, utf8_encoding);

		if (!ba.empty())
			s = utf8_string(ba.data());
#else
		s = utf8_string(e.message());
#endif
	}

	return s;
}

const char* iconv_enc(encodings e)
{
	switch (e)
	{
		case utf8_encoding:
			return "UTF-8";

		case utf16_encoding:
			return "UTF-16LE";

		case utf32_encoding:
			return "UTF-32LE";

		case current_locale_encoding:
			return "";

		default:
			bt_assert(false);
			return nullptr;
	}
}

class scoped_iconv_handle : boost::noncopyable
{
public:
	scoped_iconv_handle(iconv_t i)
		: i_(i)
	{
	}

	~scoped_iconv_handle()
	{
		if (i_ != iconv_t(-1))
			iconv_close(i_);
	}

	operator iconv_t()
	{
		return i_;
	}

private:
	iconv_t i_;
};

byte_array convert(
	const void* s, std::size_t size, encodings from, encodings to)
{
	const char* fromcode = iconv_enc(from);
	const char* tocode = iconv_enc(to);

	if (!fromcode || !tocode)
		return byte_array();

	scoped_iconv_handle cd(iconv_open(tocode, fromcode));
	if (cd == (iconv_t)(-1))
	{
		log_error()
			<< "could not open iconv for " << fromcode << "->"
			<< tocode << "\n";

		return byte_array();
	}

	std::size_t inbytesleft = size;
	std::size_t outbytesleft = size * 4;
	std::size_t outsize = outbytesleft;
	boost::shared_array<char> out;

	const char* inbuf = reinterpret_cast<const char*>(s);

	byte_array r;

	for (int i=0; i<3; ++i)
	{
		const std::size_t bad = static_cast<std::size_t>(-1);

		out.reset(new char[outsize + 4]);
		std::memset(out.get(), 0, outsize + 4);

		char* outbuf = out.get();
	
		if (iconv_wrapper(
			cd, &inbuf, &inbytesleft,
		       	&outbuf, &outbytesleft) == bad)
		{
			const int e = errno;

			if (e == E2BIG)
			{
				outsize *= 2;
				continue;
			}

			log_error()
				<< "iconv failed for " << fromcode << "->"
				<< tocode << ", errno=" << e << "\n";

			return byte_array();
		}

		const std::size_t ts = outsize - outbytesleft + 4;
		r = byte_array(
			reinterpret_cast<const unsigned char*>(out.get()), ts);

		break;
	}

	return r;
}

utf8_string utf16_to_utf8(const utf16_string& s)
{
	const byte_array ba = convert(
		s.c_str(), s.size_in_bytes(),
		utf16_encoding, utf8_encoding);

	if (ba.empty())
		return utf8_string();

	return utf8_string(ba.data());
}

utf16_string utf8_to_utf16(const utf8_string& s)
{
	const byte_array ba = convert(
		s.c_str(), s.size_in_bytes(),
		utf8_encoding, utf16_encoding);

	if (ba.empty())
		return utf16_string();

	return utf16_string(reinterpret_cast<const wchar_t*>(ba.data()));
}

BARETORRENT_EXPORT wxString to_wxstring(const utf8_string& s)
{
#if BARETORRENT_GTK
	if (!g_utf8_validate(s.c_str(), -1, 0))
		return wxString("?????");
#endif

	return wxString::FromUTF8(s.c_str());
}

BARETORRENT_EXPORT wxString to_wxstring(const boost::format& s)
{
	return to_wxstring(utf8_string(s.str()));
}

BARETORRENT_EXPORT utf8_string from_wxstring(const wxString& s)
{
	return utf8_string(s.utf8_str().data());
}


locale_map parse_locale(const utf8_path& p, bool only_name = false)
{
	std::ifstream in(p.native().c_str());
	in.exceptions(std::ios::badbit);

	locale_map lm;

	int n = 1;
	for (;;)
	{
		std::string line_temp;
		std::getline(in, line_temp);
		if (!in)
			break;

		utf8_string line(line_temp);

		boost::trim(line);
		if (line.empty() || line[0] == '#')
			continue;
			
		// each line should "name = value", whitespace is not
		// significant

		utf8_string::size_type p = line.find(utf8_string("="));
		if (p == utf8_string::npos)
		{
			log_error() << "bad line " << n << "\n";
			continue;
		}

		std::string name(line.substr(0, p).std_string());
		utf8_string value(line.substr(p+1));

		boost::trim(name);
		boost::trim(value);

		if (!name.empty() && !value.empty())
		{
			if (lm.find(name) != lm.end())
			{
				log_error()
					<< "duplicate string name " << name
					<< " on line " << n << "\n";
			}
			else
			{
				lm[name] = boost::format(value.std_string());
			}
		}

		if (only_name && name == "locale.name")
			break;

		++n;
	}

	return lm;
}

bool check_locale_file(const utf8_path& p, std::vector<locale_file>& v)
{
	// ignoring directories
	if (!p.is_regular_file())
		return true;

	locale_map lm = parse_locale(p, true);
	if (lm.find("locale.name") == lm.end())
	{
		log_error() << "locale " << p << " missing locale.name\n";
		return true;
	}

	v.push_back(locale_file(utf8_string(lm["locale.name"].str()), p));

	return true;
}

std::vector<locale_file> locale_files()
{
	std::vector<locale_file> v;

	v.push_back(locale_file(utf8_string("English"), utf8_path()));

	try
	{
		const utf8_path root = absolute(paths::locales());
		walk_directory(root, boost::bind(
			&check_locale_file, _1, boost::ref(v)));
	}
	catch(path_exception&)
	{
		log_error()
			<< "couldn't check locale files in "
			<< paths::locales() << "\n";
	}

	return v;
}

bool locale_comp(
	const locale_map::value_type& left,
	const locale_map::value_type& right)
{
	return left.first < right.first;
}

void load_locale()
{
	load_english();

	// empty language string is english
	if (opt("bt.ui.language").empty())
		return;

	try
	{
		const utf8_string fn =
			opt("bt.ui.language") + paths::locale_suffix();

		const utf8_path file = paths::locales() / fn;

		log() << "loading locale " << file << "\n";
		
		locale_map lm = parse_locale(file);

		locale_map missing, extra;

		// gets the missing locale strings and the ones that are not
		// in the default english list

		std::set_difference(
			g_map.begin(), g_map.end(),
			lm.begin(), lm.end(),
			std::insert_iterator<locale_map>(missing, missing.end()),
			locale_comp);

		std::set_difference(
			lm.begin(), lm.end(),
			g_map.begin(), g_map.end(),
			std::insert_iterator<locale_map>(extra, extra.end()),
			locale_comp);

		if (!missing.empty())
		{
			log_error() << "missing names in this locale:\n";
			foreach (const locale_map::value_type& v, missing)
				log_error() << "  " << v.first << "\n";
		}

		if (!extra.empty())
		{
			log_error() << "extra names in this locale:\n";
			foreach (const locale_map::value_type& v, extra)
				log_error() << "  " << v.first << "\n";
		}

		// replacing the english strings by the one that were translated
		foreach (const locale_map::value_type& v, lm)
			g_map[v.first] = v.second;
	}
	catch(std::exception& e)
	{
		log_error() << "can't load locale, " << e.what() << "\n";
	}
}

}	// namespace baretorrent
