// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_LOCALE_INCLUDED
#define BARETORRENT_LOCALE_INCLUDED

#include "path.h"

// these are facilities for localizing strings

namespace baretorrent
{

struct locale_file
{
	utf8_string name;
	utf8_path path;
	utf8_string ietf;

	locale_file(const utf8_string& n, const utf8_path& p)
		: name(n), path(p)
	{
		ietf = path.stem();
	}
};


// returns whether the given name is in the locale file
//
bool has_loc(const std::string& name);

// returns the localized string for the given name
//
BARETORRENT_EXPORT utf8_string loc(const std::string& name);
BARETORRENT_EXPORT boost::format locf(const std::string& name);

// returns the localized string for the given name as a wxString
//
BARETORRENT_EXPORT wxString loc_wx(const std::string& name);

// converts a string using the given encodings
//
BARETORRENT_EXPORT byte_array convert(
	const void* s, std::size_t size, encodings from, encodings to);

// converts a utf16 string to utf8
//
BARETORRENT_EXPORT utf8_string utf16_to_utf8(const utf16_string& s);

// converts a utf8 string to utf16
//
BARETORRENT_EXPORT utf16_string utf8_to_utf16(const utf8_string& s);

// converts a utf8 string to a wxString
//
BARETORRENT_EXPORT wxString to_wxstring(const utf8_string& s);
BARETORRENT_EXPORT wxString to_wxstring(const boost::format& s);

// converts a wxString to a utf8_string
//
BARETORRENT_EXPORT utf8_string from_wxstring(const wxString& s);

// returns a localized string for the given error code
//
utf8_string loc_error(boost::system::error_code i);

// returns the list of locale files found in the locale directory
//
std::vector<locale_file> locale_files();

// loads the currently selected locale file
//
void load_locale();

}	// namespace baretorrent

#endif	// BARETORRENT_LOCALE_INCLUDED
