// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_PCH_INCLUDED
#define BARETORRENT_PCH_INCLUDED

#include "config.h"
#include "push_warnings.h"

#if BARETORRENT_WINDOWS
	#define WIN32
	#define WIN32_LEAN_AND_MEAN

	#if BARETORRENT_VISTA_PLUS
		#define WINVER _WIN32_WINNT_VISTA
		#define	_WIN32_WINNT _WIN32_WINNT_VISTA
		#define NTDDI_VERSION NTDDI_VISTA
	#else
		#define WINVER _WIN32_WINNT_WINXP
		#define	_WIN32_WINNT _WIN32_WINNT_WINXP
		#define NTDDI_VERSION NTDDI_WINXP
	#endif

	#include <WinSock2.h>
	#include <CommCtrl.h>
	#include <netfw.h>
	#include <objbase.h>
	#include <Shlobj.h>
	#include <Shellapi.h>
	#undef min
	#undef max
#endif

// boost
#include <boost/bind.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/regex.hpp>
#include <boost/thread.hpp>
#include <boost/date_time.hpp>
#include <boost/format.hpp>
#include <boost/foreach.hpp>
#include <boost/signals2.hpp>
#include <boost/smart_ptr.hpp>
#include <boost/program_options.hpp>
#include <boost/filesystem.hpp>
#include <boost/preprocessor/stringize.hpp>
#include <boost/interprocess/sync/interprocess_mutex.hpp>
#include <boost/interprocess/sync/scoped_lock.hpp>
#include <boost/interprocess/sync/file_lock.hpp>
#include <boost/interprocess/mapped_region.hpp>
#include <boost/interprocess/shared_memory_object.hpp>
#include <boost/mpl/at.hpp>

// bpl doesn't compile in mm files
#ifndef BARETORRENT_SKIP_PYTHON

#define BOOST_LINKING_PYTHON

#if BARETORRENT_USE_DEBUG_PYTHON
#define BOOST_DEBUG_PYTHON
#endif

#include <boost/python.hpp>
#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/python/make_function.hpp>
#include <boost/python/def_visitor.hpp>
#include <boost/python/signature.hpp>

namespace bpl = boost::python;
namespace sig = boost::signals2;

#endif	// BARETORRENT_SKIP_PYTHON

#include <boost/uuid/uuid.hpp>
#include <boost/uuid/uuid_generators.hpp>
#include <boost/uuid/uuid_io.hpp>
#include <boost/preprocessor/stringize.hpp>

// wxWidgets
#define wxUSE_UNICODE 1

// todo: this doesn't seem to be defined on visual c++
#ifndef WXUSINGDLL
#define WXUSINGDLL
#endif

#if !BARETORRENT_DEBUG
#ifndef wxDEBUG_LEVEL
#define wxDEBUG_LEVEL 0
#endif
#endif

#include <wx/wx.h>
#include <wx/listctrl.h>
#include <wx/splitter.h>
#include <wx/gbsizer.h>
#include <wx/mstream.h>
#include <wx/notebook.h>
#include <wx/treectrl.h>
#include <wx/spinctrl.h>
#include <wx/taskbar.h>
#include <wx/hyperlink.h>
#include <wx/clipbrd.h>
#include <wx/url.h>
#include <wx/dnd.h>
#include <wx/combo.h>
#include <phoenix/wxpy_api.h>

#define TORRENT_NO_DEPRECATE
#define TORRENT_USE_OPENSSL
#define TORRENT_DISABLE_GEO_IP
#define TORRENT_DISABLE_RESOLVE_COUNTRIES

#define BOOST_ASIO_SEPARATE_COMPILATION
#define TORRENT_USE_WSTRING 1

#if BARETORRENT_DEBUG
#define TORRENT_DEBUG
#endif

#include <libtorrent/config.hpp>
#include <libtorrent/lazy_entry.hpp>
#include <libtorrent/torrent_info.hpp>
#include <libtorrent/create_torrent.hpp>
#include <libtorrent/session.hpp>
#include <libtorrent/alert_types.hpp>
#include <libtorrent/peer_info.hpp>
#include <libtorrent/extensions.hpp>
#include <libtorrent/peer_connection.hpp>
#include <libtorrent/extensions/lt_trackers.hpp>
#include <libtorrent/extensions/ut_pex.hpp>
#include <libtorrent/extensions/metadata_transfer.hpp>
#include <libtorrent/extensions/ut_metadata.hpp>
#include <libtorrent/extensions/smart_ban.hpp>
#include <libtorrent/torrent.hpp>
#include <libtorrent/storage.hpp>
#include <libtorrent/ip_filter.hpp>
#include <libtorrent/disk_io_thread.hpp>
#include <libtorrent/magnet_uri.hpp>
#include <libtorrent/settings.hpp>
#include <libtorrent/aux_/session_impl.hpp>

// curl
#define CURL_STATICLIB
#include <curl/curl.h>

// libxml
#define LIBXML_STATIC
#include <libxml/parser.h>
#include <libxml/xmlwriter.h>

// sqlite
#include "db/sqlite3.h"

// iconv
#define USING_STATIC_LIBICONV 1
#include <iconv.h>

// openssl
#include <openssl/opensslv.h>
#include <openssl/bio.h>
#include <openssl/evp.h>

// standard
#include <memory>
#include <functional>
#include <vector>
#include <string>
#include <map>
#include <exception>

#include "version.h"

#if BARETORRENT_GCC
#undef major
#undef minor
#endif

// gcc on linux defines that
#ifdef linux
#undef linux
#endif

namespace bt = libtorrent;
namespace po = boost::program_options;
namespace ipc = boost::interprocess;

#define foreach BOOST_FOREACH

typedef boost::posix_time::ptime ptime;

#include "pop_warnings.h"

#endif	// BARETORRENT_PCH_INCLUDED
