// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_PYTHON_CONSOLE_INCLUDED
#define BARETORRENT_PYTHON_CONSOLE_INCLUDED

#include "widgets.h"

namespace baretorrent
{

class python_console : public custom_dialog
{
public:
	python_console(wxWindow* parent);
	~python_console();

private:
	static std::size_t no_history;

	struct ext_info : public wxClientData
	{
		ext::instance_ptr i;

		ext_info(ext::instance_ptr i)
			: i(i)
		{
		}
	};

	wxComboBox* extensions_;

	multiline_text* out_;
	wxTextCtrl* in_;
	wxStaticText* indicator_;
	bpl::object console_;
	bool more_;
	ext::instance_ptr ext_;
	signal_connections gcons_, econs_;

	std::deque<utf8_string> history_;
	std::size_t current_history_;

	void on_log(const log_t::entry& e);
	void on_log_impl(const log_t::entry& e);
	void on_input(wxCommandEvent&);
	void on_input_key(wxKeyEvent& e);
	void on_ext_selected(wxCommandEvent&);
	void on_ext_loaded();
	void clear_input();

	void refresh_extensions();
	void set_ext(ext::instance_ptr i);
	void create_interactive_console(ext::instance_ptr i);
	void add_history(const utf8_string& s);
	void output_command(const utf8_string& s);
	bool execute_builtin(utf8_string s);
	void execute(const utf8_string &s);
	bool handle_result(
		bpl::object r, const utf8_string& out, const utf8_string& err);
};

}	// namespace baretorrent

#endif	// BARETORRENT_PYTHON_CONSOLE_INCLUDED
