// This file is part of baretorrent and is released in the public
// domain. Hopefully, someone can make it better than I could.

#ifndef BARETORRENT_VERSION_INCLUDED
#define BARETORRENT_VERSION_INCLUDED

#define BARETORRENT_VERSION_MAJOR 0
#define BARETORRENT_VERSION_MINOR 4
#define BARETORRENT_VERSION_MAINTENANCE 4

//   BARETORRENT_VERSION_MAJOR
//   BARETORRENT_VERSION_MINOR
//   BARETORRENT_VERSION_MAINTENANCE
//   BARETORRENT_VERSION_TAG
//
// where either the maintenance or tag define can be left off.
// BARETORRENT_VERSION_MAJOR and BARETORRENT_VERSION_MINOR are mandatory
//
// after including this file, three macros are available:
//
//	BARETORRENT_USER_VERSION		"major.minor[.maintenance][-tag]"
//	BARETORRENT_USER_BUILD_DATE		the date and time of this build
//
//	visual c++ only:
//	BARETORRENT_USER_RES_VERSION	"major,minor,maintenance,0"
//									suitable for the strings section of
//									a visual c++ .rc file
//	BARETORRENT_USER_FILEFLAGS		combination of FILEFLAGS


#if !defined(BARETORRENT_VERSION_MAJOR) ||\
	!defined(BARETORRENT_VERSION_MINOR)
		#error	BARETORRENT_VERSION_MAJOR and BARETORRENT_VERSION_MINOR\
				need to be defined
#endif


#if BARETORRENT_VISUALC
	#ifdef BARETORRENT_VERSION_MAINTENANCE
		// no spaces between comma 
		#define BARETORRENT_USER_RES_VERSION						\
			BARETORRENT_VERSION_MAJOR,BARETORRENT_VERSION_MINOR,	\
			BARETORRENT_VERSION_MAINTENANCE,0
	#else
		// no spaces between comma
		#define BARETORRENT_USER_RES_VERSION						\
			BARETORRENT_VERSION_MAJOR,BARETORRENT_VERSION_MINOR,0,0
	#endif

	#define BARETORRENT_FILEFLAG_DEBUG 0
	#define BARETORRENT_FILEFLAG_PRERELEASE 0

	#if BARETORRENT_DEBUG && BARETORRENT_VERSION_MAJOR == 0
		#undef BARETORRENT_USER_FILEFLAGS
		#define BARETORRENT_USER_FILEFLAGS 0x03L
	#elif BARETORRENT_DEBUG
		#undef BARETORRENT_USER_FILEFLAGS
		#define BARETORRENT_USER_FILEFLAGS 0x01L
	#elif BARETORRENT_VERSION_MAJOR == 0
		#undef BARETORRENT_USER_FILEFLAGS
		#define BARETORRENT_USER_FILEFLAGS 0x02L
	#endif
#endif

#ifdef BARETORRENT_VERSION_MAINTENANCE
	#define BARETORRENT_INTERNAL_VERSION_1					\
		BOOST_PP_STRINGIZE(BARETORRENT_VERSION_MAJOR) "."	\
		BOOST_PP_STRINGIZE(BARETORRENT_VERSION_MINOR) "."	\
		BOOST_PP_STRINGIZE(BARETORRENT_VERSION_MAINTENANCE)
#else
	#define BARETORRENT_INTERNAL_VERSION_1					\
		BOOST_PP_STRINGIZE(BARETORRENT_VERSION_MAJOR) "."	\
		BOOST_PP_STRINGIZE(BARETORRENT_VERSION_MINOR)
#endif

#ifdef BARETORRENT_VERSION_TAG
	#define BARETORRENT_INTERNAL_VERSION_2					\
		BARETORRENT_INTERNAL_VERSION_1 "-"					\
		BARETORRENT_VERSION_TAG
#else
	#define BARETORRENT_INTERNAL_VERSION_2					\
		BARETORRENT_INTERNAL_VERSION_1
#endif

#define BARETORRENT_USER_VERSION BARETORRENT_INTERNAL_VERSION_2
#define BARETORRENT_USER_BUILD_DATE								\
			BARETORRENT_DATE_MACRO " " BARETORRENT_TIME_MACRO

#ifdef major
#undef major
#endif

#ifdef minor
#undef minor
#endif

namespace baretorrent
{

struct baretorrent_version
{
	int major;
	int minor;
	int maintenance;
	std::string tag;
	std::string build_date;
	std::string string;

	baretorrent_version()
		: major(0), minor(0), maintenance(0)
	{
	}

	bool bad() const
	{
		return (major == 0 && minor == 0 && maintenance == 0);
	}

	// parses the given version string
	//
	static baretorrent_version parse(const std::string& v);
};

std::ostream& operator<<(std::ostream& o, const baretorrent_version& v);

bool operator==(
	const baretorrent_version& left, const baretorrent_version& right);
bool operator!=(
	const baretorrent_version& left, const baretorrent_version& right);

bool operator<(
	const baretorrent_version& left, const baretorrent_version& right);
bool operator<=(
	const baretorrent_version& left, const baretorrent_version& right);
bool operator>(
	const baretorrent_version& left, const baretorrent_version& right);
bool operator>=(
	const baretorrent_version& left, const baretorrent_version& right);

inline baretorrent_version version()
{
	baretorrent_version v;

	v.major = BARETORRENT_VERSION_MAJOR;
	v.minor = BARETORRENT_VERSION_MINOR;

#ifdef BARETORRENT_VERSION_MAINTENANCE
	v.maintenance = BARETORRENT_VERSION_MAINTENANCE;
#endif

#ifdef BARETORRENT_VERSION_TAG
	v.tag = BARETORRENT_VERSION_TAG;
#endif

	v.build_date = BARETORRENT_USER_BUILD_DATE;
	v.string = BARETORRENT_USER_VERSION;

	return v;
}

}	// namespace baretorrent

#endif	// BARETORRENT_VERSION_INCLUDED
